#Set the serial port to a baud rate of 115200
proc initializeSerials {} {
    global gvar

    foreach tty $gvar(ttyList) {
	if {$opSystem == "HP-UX"} {
	    exec stty raw $baud cs8 cread crts clocal rtsxoff ctsxon < $tty
	} elseif {$opSystem == "SunOS"} {
	    exec stty raw 38400 cs8 cread crtscts clocal crtsxoff < $tty
	} else {
	    #...
	}
    }
}

#Open the serial port and set it to a baud rate of 115200
proc openSerialPorts {} {
    global gvar
    
    foreach tty $gvar(ttyList) {
	#set baud [lindex $gvar(speedList) [lsearch -exact $gvar(ttyList) $tty]]
	set index  [lsearch -exact $gvar(ttyList) $tty]
	set gvar(baudRate)   [lindex [lindex $gvar(speedList) 0] $index]
	
	set gvar($tty) [open $tty r+]
	fconfigure $gvar($tty)  -mode $gvar(baudRate),n,8,1
	set gvar(serialPortConfig) [fconfigure $gvar($tty)]
	setSerialPort $tty $gvar(baudRate)
    }
}

proc setSerialPort {tty baud} {

    set opSystem [exec uname]
    if {$opSystem == "HP-UX"} {
	exec stty raw $baud cs8 cread crts clocal rtsxoff ctsxon < $tty
    } elseif {$opSystem == "SunOS"} {
	exec stty raw $baud cs8 cread crtscts clocal crtsxoff < $tty
    } else {
	#Use the same of Solaris...for the moment...
	exec stty raw $baud cs8 cread crtscts clocal crtsxoff < $tty
    }
}

proc cleanBuffer {} {
    global gvar

    set tty $gvar($gvar(tty))
    set buffer "start"

    while {$buffer != ""} {
	flush $tty
	fconfigure $tty -blocking 0
	if {[catch {set buffer [read $gvar($gvar(tty))]}]} {
	    Log "An error occoured, while reading serial port: "
	    puts "An error occoured, while reading serial port: "
	    return 1
	}	
	fconfigure $tty -blocking 1
    }
}

#Identify the robot model.
proc getRobotType {tty} {
	global robotType

    
    return $robotType($tty)
    
    #Try first with the RIMAGE model

    #cleanBuffer 
    # set commandString "\x1B\x21\x65\x30\x43"
#     sendMessage $commandString 
#     set returnValue [readPort 39]
#     if {[string match "*Transporter*" $returnValue]} {
# 	Debug "We have RIMAGE robot." 3
# 	return "RIMAGE"
#     } else {
	#Check if we have an LSK model.
	#cleanBuffer
	# set commandString "\x56\X7A"
# 	sendMessage $commandString 
# 	set returnValue [readPort 20]
# 	if {[string match "*Discjet*" $returnValue ]} {
# 	    Debug "We have an LSK robot." 3
# 	    return "LSK"
# 	} else {
# 	    Debug "Unknown robot" 3
# 	    return "unknown"
# 	}
#     }

}

#Send a string to the robot via the serial port.
proc sendMessage {action} {
    global gvar 

    set portConfig [fconfigure $gvar($gvar(tty))]
    if {$portConfig != $gvar(serialPortConfig)} {
	Debug "Serial port configuration has changed. Reconfigure." 1
	fconfigure $gvar($gvar(tty)) -mode $gvar(baudRate),n,8,1
	setSerialPort $gvar(tty) $gvar(baudRate)
    }   
    
    Debug "Send message: puts $gvar(tty) $action" 3
    puts $gvar($gvar(tty)) $action
    flush $gvar($gvar(tty))
}

#Read the serial port. This operation MUST be done even if
#we don't care about the output, otherwise the root gets "confused"
proc readPort {bytes} {
    global gvar
    
    #set tty [open /dev/tty1p0 r]
    if {[catch {set buffer [read $gvar($gvar(tty)) $bytes]}]} {
	Log "An error occoured, while reading serial port: "
	after 3000
	readPort $bytes
    }	
    return $buffer
}

#Query all the robots connected and wait for an answer.
proc romanicRequestSenseAll {} {
    global gvar
    
    foreach tty $gvar(ttyList) {
	set gvar(tty) $tty
	Debug "Detect the robot model" 1
	set gvar(robotType) [getRobotType $gvar(tty)]
	if {$gvar(robotType) == "unknown"} {
	    puts "Robot, connected to $tty, is of unknown type"
	    puts "Exit."
	    exit 1
	}
	
	Debug "Define commands for robot $gvar(robotType)" 1
	romanicDefineCommands
	
	puts "Calibrate autoloader - $tty"
	Debug "Calibrate autoloader - $tty" 1
	calibrateArm $tty
	
	
	Debug "Enquiry robot $tty" 2
	if {[waitForOk]} {
	    puts "Cannot communicate with the Robot connected to $tty"
	    puts "Verify that the robot:"
	    puts "- Is powered on"
			puts "- Has its door closed"
	    exit 1
	}
    }
}

#Hacked procedure written in paranal, used to print the
# DP-SUMMARY of User CDs.
proc printSummary {volume} {
    
    #The user path is here hardcoded, a better solution is required. SOG
    set oldPath [pwd]
    set path "/media/user/4.USR"
    cd $path
    catch {eval exec a2ps -l -F7 -1 $volume/DP-SUMMARY}
    cd $oldPath
}

# Retrieve the LSK writer code.
proc getWriterAddress {writer} {
    global writerAddressList

Debug "Writer is: $writer" 3

set writerAddress $writerAddressList($writer)

Debug "Writer address is: $writerAddress" 3
return $writerAddress    
}



#Query the robot up to 3 times. If the output is not OK 
#generate an error.
proc waitForOk {} {
    global gvar
    
    set i 0
    while {[romanicRequestSense] != "Ok --- Ok"} {
	Debug "Robot is not ok. Try again." 1
	if {$i == 3} {
	    return 1
	}
	after 3000
	incr i
    }
    return 0
}

#Reset a specific CD writer.
proc resetCD {device} {
    
    Log "Reset CD: $device"
    catch {exec cdrecord dev=$device -reset}
}

#Open the CD tray.
proc ejectCD {device} {
    
    Log "Eject CD: $device"
    catch {exec cdrecord dev=$device -eject}
}

#Close the CD tray.
proc loadCD {device} {

    Log "Load CD: $device"
    catch {exec cdrecord dev=$device -load} output
 
    if {[string match "*cdrecord: No disk / Wrong disk!*" $output] ||
	[string match "*cdrecord: CD/DVD-Recorder not ready*" $output]} {
	return "error"
    } else {
	return "ok"
    }
}

#Execute cdrecord.
proc writeCD {id} {
    global gvar

    Log "writeCD: $gvar($id-writerAddress)"
#    set cdrecordOptions "-v -waiti dev=$gvar($id-writerAddress) fs=16m speed=$gvar($id-writerSpeed)"
    set cdrecordOptions "-v -waiti dev=$gvar($id-writerAddress) fs=16m speed=$gvar($id-writerSpeed)  driveropts=burnfree"

    if {$gvar($id-dummyMode)} {
	append cdrecordOptions " -dummy"
    }
    
    Log "Execute: cdrecord $cdrecordOptions $gvar($id-volumeFile).vol*"

    #With the latest writers I had to add the option -waiti and a trick to
    #send something to the standard input.
    if {[catch { eval "exec sh -c \"sleep 3 ; echo p | cdrecord $cdrecordOptions $gvar($id-volumeFile).vol* > $gvar(reqDir)/cdrecord-$id.romanic\" &"} msg ]} {
	Log "Error executing cdrecord: $msg" 
	close $logFile
	return "error"
    }

    set logFile [open "$gvar(reqDir)/cdrecord-$id.romanic" w+]
    set line [split [read $logFile]]

    #Calculate here the total size of the medium.
    set totalSize 0
    while {[string match "*Total size:*" $line] == 0} {
	if {[string match "*error*" $line] || 
	    [string match "*Error*" $line] ||
	    [string match "*Recorder not ready*" $line] ||
	    [string match "*Illegal Request*" $line]} {
	    close $logFile
	    return "error"}
	set line [split [read $logFile]]
    }
    if {[string match "*error*" $line] || 
	[string match "*Error*" $line] ||
	[string match "*Recorder not ready*" $line] ||
	[string match "*Illegal Request*" $line]} {
	close $logFile
	return "error"}
    
    set token [ctoken line "MB"]
    while {$token != ""} {
	if {[string match "*Track *: data*" $token]} {
	    set trackSize [lindex $token end]
	    set totalSize [incr totalSize +$trackSize]
	}
	set token [ctoken line "MB"]
    }
    
    #Calculate the progress
    set progress 0
    while {[string match "*Fixating...*" $line] == 0} { 
	
	if {[string match "*error*" $line] ||
	    [string match "*Error*" $line] ||
	    [string match "*Recorder not ready*" $line] ||
	    [string match "*Illegal Request*" $line]} {
	    close $logFile
	    return "error"}
	
	regsub -all "{" $line "" line
	regsub -all "}" $line "" line
	if {[lindex $line 0] == "Track" && [lindex $line 2] != "Total"} { 
	    set megaByte [lindex $line 2]
	    if {$megaByte != 0} {incr progress +1}
	    write_file $gvar(reqDir)/count-$id.romanic  "$progress of $totalSize"
	}
	set line [split [read $logFile]]
    }
	if {[string match "*error*" $line] || 
	    [string match "*Error*" $line] ||
	    [string match "*Recorder not ready*" $line] ||
	    [string match "*Illegal Request*" $line]} {
	    close $logFile
	    return "error"}
    
    #Writing in progress
    while {[string match "*Fixating...*" $line] == 0} { 
		
		set line [split [read $logFile]]
		if {[string match "*error*" $line] ||
		    [string match "*Error*" $line] ||
		    [string match "*Recorder not ready*" $line] ||
		    [string match "*Illegal Request*" $line]} {
			Log "Error writing the CD" 
			close $logFile
			return "error"}
	
		regsub -all "{" $line "" line
		regsub -all "}" $line "" line
		if {[lindex $line 0] != "Track"} { 
			continue
		} else {
			write_file $gvar(reqDir)/count-$id.romanic  [lrange $line 2 4]
		}
	}
	if {[string match "*error*" $line] || 
	    [string match "*Error*" $line] ||
	    [string match "*Recorder not ready*" $line] ||
	    [string match "*Illegal Request*" $line]} {
	    Log "Error writing the CD" 
	    close $logFile
	    return "error"}
	
    #Medium written
    while {[string match "*Fixating time:*" $line] == 0} {
	if {[string match "*error*" $line]||
	    [string match "*Recorder not ready*" $line] ||
	    [string match "*Error*" $line]} {
	    Log "Error writing the CD" 
	    close $logFile
	    return "error"}
	set line [split [read $logFile]]
	after 2000
    }
    if {[string match "*error*" $line] ||
	[string match "*Recorder not ready*" $line] ||
	[string match "*Error*" $line]} {
	Log "Error writing the CD" 
	close $logFile
	return "error"
    }
    
    Log  "CD $gvar($id-volumeFile) written."
    return ""
}



#Close all the channels.
proc romanicPilotQuit {} {
    global gvar
    
    foreach tty $gvar(ttyList) {
	close $gvar($tty)
    }
    exit
}


#Parse the command line options and set some variables.
proc romanicSetOptions {} {
    global gvar
    
    regsub -all "{" $gvar(args) "" gvar(args)
    regsub -all "}" $gvar(args) "" gvar(args)
    
    for { set i 0} {$i <[llength $gvar(args)]} {incr i} {

        switch -- [lindex $gvar(args) $i] {
	    -info {
		incr i
		explainAll
	    } 
	    -debug {
		incr i
		set level [lindex $gvar(args) $i]
		if {$level == "" || $level > 3 || $level < 1} {
		    explain
		}
		set gvar(debugMode) $level
	    }
	    -log {
		incr i
		set logFile [lindex $gvar(args) $i]
		if {[catch {set gvar(logFile) [open $logFile a]} var]} {
		    puts "Cannot open $logFile"
		    exit }
		if {![info exists gvar(debugMode)]} { set gvar(debugMode) 1 }
	    } 
	    -requestDir {
		incr i
		set gvar(reqDir) [lindex $gvar(args) $i] 
	    }
	    -tty {
		incr i
		set gvar(forcetty)  [lindex $gvar(args) $i]
	    }
	    default {
		puts "Unknown option: [lindex $gvar(args) $i]"
		explain
	    }
	}
    }
    
    checkArguments
}

#Be sure that the options specified are correct.
proc checkArguments {} {
    global gvar

    if {![info exists gvar(reqDir)]} {
	puts "Request Directory not specified."
	explain
    }

    if {![file exists $gvar(reqDir)]} {
	puts "Request directory $gvar(reqDir) doesn't exist"
	exit 1
    }

    if {![file readable $gvar(reqDir)]} {
	puts "Request directory $gvar(reqDir) is not readable"
	exit 1
    }
}

proc checkBaud {baud} {

    switch $baud   { 
	"1200"     { set returnCode "ok" }
	"2400"     { set returnCode "ok" }
	"4800"     { set returnCode "ok" }
	"9600"     { set returnCode "ok" }
	"19200"    { set returnCode "ok" }
	"38400"    { set returnCode "ok" }
	"57600"    { set returnCode "ok" }
	"115200"   { set returnCode "ok" }
	"230400"   { set returnCode "ok" }
	default    { set returnCode "error"}
    }

    return $returnCode
}

proc checkRobotType {type} {
    
    set robotList {RIMAGE LSK}
    if {[lsearch -exact $robotList $type ] == -1} {
	return $robotList
    } else {
	return "Ok"
    }
}

proc checkRobotMode {mode} {
    
    set robotModeList {3-magazine efficient mixed}
    if {[lsearch -exact $robotModeList $mode ] == -1} {
	return $robotModeList
    } else {
	return "Ok"
    }
}


#I won't explain explain =;-)
proc explain {} {
    global gvar

    puts "Usage: romanic -info\n-debug \[1-3\]\n-log <log file>\n-requestDir <directory>"
    exit 1
}


proc explainAll {} {
    puts "-info                   : This message"
    puts "-log <filename>         : Puts all the messages in the specified file"
    puts "                        : If you don't specify -debug the verbose level"
    puts "                        : will be set to 1"
    puts "-debug \[1-3\]          : Define the verbose level"
    puts "-requestDir  <dir>      : Directory containig request files."
    exit 1
}

#Loop looking for requests under the specified directory.
proc romanicGlobLoop { time cmd } {
    global gvar
    
    if {$gvar(cancelGlobLoop) == 1} {
	set gvar(cancelGlobLoop) 0
	after cancel $gvar(globLoopId)
	Debug  "Loop - $gvar(globLoopId) - $cmd - killed " 1
    } else {
	eval $cmd
	set gvar(globLoopId) [after $time romanicGlobLoop $time \"$cmd\" ]
	Debug "LoopID = $gvar(globLoopId) -- cmd = $cmd" 3
    }
}

#Serve requests in queue.
proc romanicServeRequestsLoop { time cmd } {
    global gvar
    
    if {$gvar(cancelServeRequestsLoop) == 1} {
	set gvar(cancelServeRequestsLoop) 0
	after cancel $gvar(serveRequestsLoopId)
	Debug " Loop - $gvar(serveRequestsLoopId) - $cmd - killed " 1
    } else {
	eval $cmd
	set gvar(serveRequestsLoopId) [after $time romanicServeRequestsLoop $time \"$cmd\" ]
	Debug "LoopID = $gvar(serveRequestsLoopId) -- cmd = $cmd" 3
    }
}

#Create a child process to serve two requests in parallel.
proc forkProcess {command} {
    global gvar

    signal default SIGCHLD
    Debug  "Fork process with command: $command" 1
    set pid [ fork ]
    Debug "fork id: $pid" 2
    set gvar(pid) $pid
    
    # This is the father
    if { $pid != 0} {
	set getSignal [signal get SIGCHLD]
	Debug "I'm the father. Return" 2
	signal trap SIGCHLD reaper
	return
    } else {
    	# This is the son
    	Debug "I'm the son, execute: eval $command" 2
	set gvar(cancelServeRequestsLoop) 1
	set gvar(cancelGlobLoop) 1
	eval $command
	exit
    }
}

#Used by fork procedure.
proc reaper {} {
    wait -nohang
    #signal trap SIGCHLD reaper
}


#Lock the arm. Other arm operations are frozen.
proc lockArm {tty} {
    global gvar 

    write_file $gvar(reqDir)/$tty-$gvar(id).lock ""
}

#Unlock the arm.
proc releaseArm {tty} {
    global gvar

    file delete $gvar(reqDir)/$tty-$gvar(id).lock
    file delete $gvar(reqDir)/$tty-$gvar(id).mylock
}

#Wait for the arm to be unlocked.
proc justWait {tty} {
    global gvar
    
#    while {[file exists $gvar(reqDir)/$tty.lock]}#
    while {[glob -nocomplain $gvar(reqDir)/$tty-*.lock] != ""} {
	if {![file exists $gvar(reqDir)/request-$gvar(id).romanic]} {
	    Log "Request cancelled"
	    cleanUpQueue $gvar(id)
	    exit 1
	}
	after 3000}
}

proc loopCommandRobot {cmd attempts {returnCode ""}} {
    global gvar 

    Debug "Executing command: $cmd with $attempts attempts" 2
    set result [eval $cmd]
    
    if {$result != $returnCode} {
	set inc 0
        while {$inc < $attempts && $result != $returnCode } {
	    incr inc
            set result [eval $cmd]
        }
		
        if {$inc == $attempts} {
	    Debug "Failed executing $cmd after $attempts attempts" 2
	    return $result
        } else {
	    return "ok"
	}
    } else {
	return "ok"
    }
}


#Write the operation to be executed to a log file.
proc Log {txt} {
    global gvar 
    
    if {![info exists gvar(id)]} { return }
    set file [open $gvar(reqDir)/log-$gvar(id)-[file tail $gvar(ttyList)].romanic a+]
    puts $file "$txt \[$gvar(tty)\]"
    close $file
    if {[info exists gvar(debugMode)]} { Debug "$txt \[$gvar(tty)\]" 1 }
}

#Print debug messages.
proc Debug {text level} {
    global gvar

    #if {![info exists gvar(HtVerbose)] || $gvar(HtVerbose) == "0" } { return }
    if {![info exists gvar(debugMode)]} { return }

    if { $gvar(debugMode) <  $level } { return }
    set date [clock format [clock seconds] -format "%Y-%m-%d %T"]

    if {[info exists gvar(logFile)]} {puts $gvar(logFile) "Debug>\[$date\] $text"
    } else { puts "Debug>\[$date\] $text" }
}

proc findVLTFile { file { confirm "yes" } } {
    global env
     
    #
    # Take the only file name and check in the current directory
    #
    set myFile  [file tail $file] 
    set myDir   [pwd]

    if {[file exist $myFile]} { return  ${myDir}/${myFile} }

    #
    # Does the file exist in the current sub-directory?
    #
    
    set fileName ${myDir}/$file
    if {[file exist $fileName]} { return  $fileName }
  
    #
    # Does the file exist in the current module?
    # (the current module is assumed to be the parent directory 
    #  of the current pwd)
    
    set fileName ../$file
    if {[file exist $fileName]} { return  $fileName }
    #
    # Does the file exist in the current integration area, if any?
    #
    if {[info exists env(INTROOT)]} {
        set fileName $env(INTROOT)/$file
        if {[file exist $fileName]} { return  $fileName }
    }
    #
    # Does the file exist in the current VLT data area, if any?
    #
    if {[info exists env(VLTDATA)]} {
        set fileName $env(VLTDATA)/$file
        if {[file exist $fileName]} { return  $fileName }
    }
    #
    # Does the file exist in the current VLT root area, if any?
    #
    if {[info exists env(VLTROOT)]} {
        set fileName $env(VLTROOT)/$file
        if {[file exist $fileName]} { return  $fileName }
    }

    #
    # File does not exist.
    #
    if { $confirm == "yes" } {
       set tmp "File  $myFile  NOT FOUND"
       puts $tmp
    }

    return ""
}


#Return the number of CDs starting from and encoder value.
proc calculateValue {value} {

    set numDisc [expr {(36150 - $value) / 120}]
    if {$numDisc < 0 } { set numDisc 0 }
    return $numDisc
}

proc closeWriters {} {
    global gvar
    
    set gvar(deviceList) [string trimright [string trimleft $gvar(deviceList) "{"] "}"]
    Log "Device list is: $gvar(deviceList)"
    foreach cd $gvar(deviceList) {
	puts "Reset writer - $cd"
	puts "Load writer - $cd"
	loadCD $cd
    }
}
##########################################
