/**
 * These classes have been developed for the DFSLog system.
 *
 * @author N. Kornweibel nkornwei@eso.org
 *
 * TO DO:
 * need @revision, and SNIFF info here.
 * 
 */
package org.eso.dfslog;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
 
// JDBC
import java.sql.*;
 

/**
 * LogEntry:
 *
 * This is a JFrame which encapsulates the controls to configure and 
 * send a DFSMessage. The Frame expects a Connection object in its
 * constructor, which it uses to add the new message.
 *
 * The date/time is set automatically, while priority is set from
 * a Combobox.
 *
 */


public class LogEntry extends JFrame implements ActionListener {

    private DFSLogConnectionServer dbServer;
    private JComboBox prioritySelection;
    private String[] priorities = { "Emergency", "Alert", "Critical", "Error",
        "Warning", "Notice", "Info", "Debug" };
    private JButton sendButton;
    private JButton quitButton;
    private JTextField workstationEntry;
    private JTextField componentEntry;
    private JTextField labelEntry;
    private JTextField dateTimeEntry;
    private JTextField messageEntry;

    public LogEntry(String title, DFSLogConnectionServer c) {
        super(title);
        dbServer = c;

        getContentPane().setLayout(new BorderLayout());
        GridLayout g = new GridLayout(5, 2);
        g.setHgap(5);
        g.setVgap(5);
        JPanel northPanel = new JPanel(g);
        // add the widgets to the northPanel.
        northPanel.add(new JLabel("Priority:", JLabel.RIGHT));
        prioritySelection = new JComboBox(priorities);
        prioritySelection.setSelectedIndex(6);
        northPanel.add(prioritySelection);

        northPanel.add(new JLabel("Workstation:", JLabel.RIGHT));
        workstationEntry = new JTextField(25);
        northPanel.add(workstationEntry);

        northPanel.add(new JLabel("Component:", JLabel.RIGHT));
        componentEntry = new JTextField(25);
        componentEntry.setText("DFS Operator");
        northPanel.add(componentEntry);

        northPanel.add(new JLabel("Label:", JLabel.RIGHT));
        labelEntry = new JTextField(25);
        // labelEntry.setText("Samual Operator");
        northPanel.add(labelEntry);

        getContentPane().add(northPanel, "North");

        JPanel centerPanel = new JPanel(new FlowLayout());

        northPanel.add(new JLabel("Message:", JLabel.RIGHT));
        messageEntry = new JTextField(30);
        northPanel.add(messageEntry);

        JPanel southPanel = new JPanel(new FlowLayout());

        sendButton = new JButton("Send");
        sendButton.addActionListener(this);
        southPanel.add(sendButton);
        quitButton = new JButton("Quit");
        quitButton.addActionListener(this);
        southPanel.add(quitButton);

        getContentPane().add(southPanel, "South");

    }




    // implementation of the ActionListener interface
    public void actionPerformed(ActionEvent a) {
        if (a.getSource() == quitButton) {
            setVisible(false);
            dispose();
        } else if (a.getSource() == sendButton) {
            if (labelEntry.getText().length() == 0) {
                JOptionPane.showMessageDialog(this,
                    "Error: label field can not be blank");
            } else {
                sendMessage();
            }
        } else if (a.getSource() == prioritySelection) {
            sendButton.setEnabled(prioritySelection.getSelectedItem() != null);
        }
    }




    private void sendMessage() {
        boolean errors = false;
        try {
            dbServer.addMessage(
                workstationEntry.getText(),
                componentEntry.getText(),
                labelEntry.getText(),
                (String)prioritySelection.getSelectedItem(),
                messageEntry.getText(), true);
                JOptionPane.showMessageDialog(this,
                    "Message Sent Successfully");
        } catch (SQLException e) {
            JOptionPane.showMessageDialog(this,
                "Failed to send message:\n"+e.toString());
        }
    }
} // end of class LogEntry.

// end of file LogEntry.java
