/**
 * These classes have been developed for the DFSLog system.
 *
 * @author N. Kornweibel nkornwei@eso.org
 *
 * TO DO:
 * need @revision, and SNIFF info here.
 * 
 */
package org.eso.dfslog;

import java.io.*;

/**
 * Report:
 *
 * An interface for report generation.
 *
 * Column Width information can be requested from the implementing class
 * once the report is generated.
 *
 * The report is generated with a generateReport() call, where a 
 * PrintWriter object is expected.
 *
 * All other errors than IOExceptions are reported in the report body.
 * If all goes well, true is returned, else false.
 *
 * As the Report is run as a seperate thread, the interrupt() method
 * allows ReportGeneration to be interrupted, with resources being
 * deallocated and so forth.
 *
 */

public abstract class Report {

    private boolean interrupted = false;
    public abstract int[] getColumnWidths();
    public abstract void setPrintWriter(PrintWriter p);
    public abstract boolean generateReport();
    public boolean isInterrupted() {
        return interrupted;
    }
    public void interrupt() {
        interrupted = true;
    }

}  // end of interface Report.
// end of file Report.java.
