/**
 * These classes have been developed for the DFSLog system.
 *
 * @author N. Kornweibel nkornwei@eso.org
 *
 * TO DO:
 * need @revision, and SNIFF info here.
 * 
 */
package org.eso.dfslog;


import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;

/**
 * ScrollableLogTable:
 *
 * ScrollableLogTable is a JScrollPane containing a JTable.
 * An important member is the ColoredTableCellRenderer, which is
 * used to color the table rows based on the value of the priority
 * column (priorid).
 */

class ScrollableLogTable extends JScrollPane {

    public static final String workstationCol = "Host";
    public static final String dateLoggedCol = "Date Logged";
    public static final String priorityCol = "Priority";
    public static final String priorityIdCol = "Pty #";
    public static final String componentCol = "Component";
    public static final String messageCol = "Message";
    public static final String messageIdCol = "Msg Id";
    public static final String labelCol = "Label";
    public static final String archiveCol = "Archive";

    private JTable table = null;
    private TableCellRenderer defaultCellRenderer = null;
    ScrollableLogTable() {
        setup();
    }
 

    private void setup() {
        setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        table = new JTable();
        table.setShowVerticalLines(false);
        table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        // Create the scroll pane and add the table to it. 
        setViewportView(table);
    }


    public JTable getTable() {
        return table;
    }

   


    public void setModel(AbstractTableModel m) {
        table.setModel(m);
        TableColumnModel tcm = table.getColumnModel();
        for (int i = 0; i < tcm.getColumnCount(); i++) {
            TableColumn tCol = tcm.getColumn(i);
            String colName = tCol.getHeaderValue().toString();
            if (colName == null) {
                continue;
            }
            if (colName.equals("priorstr")) {
                tCol.setMinWidth(80);
                tCol.setPreferredWidth(80);
                tCol.setHeaderValue(priorityCol);

            } else if (colName.equals("wsid")) {
                tCol.setMinWidth(70);
                tCol.setPreferredWidth(70);
                tCol.setHeaderValue(workstationCol);

            } else if (colName.equals("labelid")) {
                tCol.setMinWidth(100);
                tCol.setPreferredWidth(100);
                tCol.setHeaderValue(labelCol);

            } else if (colName.equals("compid")) {
                tCol.setMinWidth(190);
                tCol.setPreferredWidth(190);
                tCol.setHeaderValue(componentCol);

            } else if (colName.equals("logdt")) {
                tCol.setMinWidth(200);
                tCol.setPreferredWidth(200);
                tCol.setHeaderValue(dateLoggedCol);

            } else if (colName.equals("msg_id")) {
                tCol.setMinWidth(80);
                tCol.setPreferredWidth(80);
                tCol.setHeaderValue(messageIdCol);

            } else if (colName.equals("archive_status")) {
                tCol.setMinWidth(50);
                tCol.setPreferredWidth(50);
                tCol.setHeaderValue(archiveCol);

            } else if (colName.equals("priorid")) {
                tCol.setMinWidth(50);
                tCol.setPreferredWidth(50);
                tCol.setHeaderValue(priorityIdCol);

            } else if (colName.equals("logmsg")) {
                tCol.setMinWidth(600);
                tCol.setPreferredWidth(600);
                tCol.setHeaderValue(messageCol);

            } else {
                tCol.setPreferredWidth(100);
            }
        }
   
        // insert our renderer between the default cell renderer and
        // the table.
        if (defaultCellRenderer == null) {
            defaultCellRenderer 
                = table.getDefaultRenderer(table.getColumnClass(0));
        }
        ColoredTableCellRenderer ctcr 
            = new ColoredTableCellRenderer(defaultCellRenderer);
        // add it as a TableColumnListener.
        table.getColumnModel().addColumnModelListener(ctcr);
        table.setDefaultRenderer(table.getColumnClass(0), ctcr);
    }

} // end of class ScrollableLogTable.




/**
 * ColoredTableCellRenderer:
 *
 * The ColoredTableCellRenderer modifies the foreground and background
 * color of renderer based on the value of the priorid column.
 * It assumes (possibly a bad assumption?) that the priorid column is
 * the third column.
 */

class ColoredTableCellRenderer extends DefaultTableCellRenderer 
    implements TableColumnModelListener {

    private static final int InvalidPriorityColumn = -2;
    private static final int UnknownPriorityColumn = -1;
    private int priorityColumn = InvalidPriorityColumn;
    private TableCellRenderer tableCellRenderer;


    public ColoredTableCellRenderer(TableCellRenderer tcr) {
        tableCellRenderer = tcr;
    }


    public java.awt.Component getTableCellRendererComponent(JTable table,
        java.lang.Object value, boolean isSelected, boolean hasFocus,
        int row, int column) {
        Component c = tableCellRenderer.getTableCellRendererComponent(
            table, value, isSelected, hasFocus, row, column);

        setColors(c, row, table);
        return c;
    }



    public void setColors(Component component, int row, JTable table) {
        boolean rowSelected = table.isRowSelected(row);
        if (getPriorityColumn(table) < 0) {
            if (rowSelected) {
                component.setBackground(java.awt.Color.white);
            } else {
                component.setBackground(java.awt.Color.lightGray);
            }
            component.setForeground(java.awt.Color.black);
        } else if (isErrorRow(row, table)) {
            if (rowSelected) {
                component.setBackground(java.awt.Color.white);
                component.setForeground(java.awt.Color.red);
            } else {
                component.setBackground(java.awt.Color.red);
                component.setForeground(java.awt.Color.yellow);
            }
        } else if (isWarningRow(row, table)) {
            if (rowSelected) {
                component.setBackground(java.awt.Color.white);
            } else {
                component.setBackground(java.awt.Color.lightGray);
            }
            component.setForeground(java.awt.Color.red);
        } else {
            if (rowSelected) {
                component.setBackground(java.awt.Color.white);
            } else {
                component.setBackground(java.awt.Color.lightGray);
            }
            component.setForeground(java.awt.Color.black);
        }
        return;
    }


    private boolean isErrorRow(int row, JTable table) {
        boolean rc = false;
        String v = (String)table.getValueAt(row, getPriorityColumn(table));
        if ((v != null) && ((v.equals("Emergency")
            ||v.equals("Alert")||v.equals("Critical")
            ||v.equals("Error")))) {
            rc = true;
        }
        return rc;
    }


    private int getPriorityColumn(JTable t) {
        if (priorityColumn == InvalidPriorityColumn) {
            priorityColumn = UnknownPriorityColumn;
            int cols = t.getColumnCount();
            for (int i = 0; i < cols; i++) {
                String s = t.getColumnName(i);
                if (s.equals("priorstr")) {
                    priorityColumn = i;
                    break;
                }
            }
        }
        return priorityColumn;
    }


    private boolean isWarningRow(int row, JTable table) {
        boolean rc = false;
        String v = (String)table.getValueAt(row, getPriorityColumn(table));
        if ((v != null) && (v.equals("Warning"))) {
            rc = true;
        }
        return rc;
    }

    // implement the TableColumnModelListener interface.
    public void columnAdded(TableColumnModelEvent e) {
        priorityColumn = InvalidPriorityColumn;
    }

    public void columnMarginChanged(ChangeEvent e) {
        // do nothing.
    }

    public void columnMoved(TableColumnModelEvent e) {
        priorityColumn = InvalidPriorityColumn;
    }

    public void columnRemoved(TableColumnModelEvent e) {
        priorityColumn = InvalidPriorityColumn;
    }

    public void columnSelectionChanged(ListSelectionEvent e) {
        // do nothing.
    }

} // end of class ColoredTableCellRenderer

// end of file ScrollableLogTable.java
