/**
 * These classes have been developed for the DFSLog system.
 *
 * @author N. Kornweibel nkornwei@eso.org
 *
 * TO DO:
 * need @revision, and SNIFF info here.
 * 
 */
package org.eso.dfslog;


/**
 * SlidingArray:
 *
 * This class represents a cache of Objects.
 * The array allows for pre-pending objects, appending objects
 * and getting objects at a certain index.
 * The initial start number must be set, along with the size, in
 * object constructor.
 */


public class SlidingArray {
    private Object cache[];
    private int currentSize;
    private int maxSize;
    private int offsetOfLowestIndexElement;
    private int lowestIndexStored;

    public static void main(String args[]) {
        SlidingArray s = new SlidingArray(30, 7);
        System.out.println("array start:30 size: 7");
        System.out.println("append 10 objects...");
        String f[] = {"hi", "there", "folks"};
        for (int i=0; i < 10; i++) { s.append(s); }
        System.out.println("prepend 6 objects...");
        for (int i=0; i < 6; i++) { s.prepend(s); }
        System.out.println("prepend 43 objects...");
        for (int i=0; i < 43; i++) { s.prepend(s); }
 
        for (int i=-50; i < 50; i++) {
            if (i%10 == 0) { System.out.print("\n"+i+"\t"); }
            if (s.get(i) == null) { System.out.print(".");
            } else { System.out.print("y"); }
        }
        System.out.println("");
    }

    private int increment(int i) {
        i++;
        if (i >= maxSize) {
           i = 0;
        }
        return i;
    }


    private int decrement(int i) {
        i--;
        if (i<0) {
            i = maxSize-1;
        }
        return i;
    }

    public SlidingArray(int sz) {
        init(0, sz);
    }

    public SlidingArray(int start, int sz) {
        init(start, sz);
    }

    private void init(int start, int sz) {
        lowestIndexStored = start;
        maxSize = sz;
        currentSize = 0;
        cache = new Object[maxSize];
        offsetOfLowestIndexElement = 0;
    }

    public int getSize() { return currentSize; }
    public int startIndex() { return lowestIndexStored; }
    public int endIndex() { return lowestIndexStored+currentSize-1; }

    public void append(Object o) {
        // appending an object results in removal of the lowest stored
        // object if the cache is already full.
        if (currentSize == maxSize) {
            cache[offsetOfLowestIndexElement] = o;
            offsetOfLowestIndexElement = increment(offsetOfLowestIndexElement);
            lowestIndexStored++;
        } else {
            int indexOfElement = offsetOfLowestIndexElement+currentSize;
            if (indexOfElement >= maxSize) {
                indexOfElement = indexOfElement-maxSize;
            }
            cache[indexOfElement] = o;
            currentSize++;
        }
    }




    public void empty() {
        currentSize = 0;
    }




    public void prepend(Object o) {
        // prepending an object results in removal of the top most
        // object if the cache is full.
        if (currentSize < maxSize) {
            currentSize++;
        }
        offsetOfLowestIndexElement =
            decrement(offsetOfLowestIndexElement);
        cache[offsetOfLowestIndexElement] = o;
        lowestIndexStored--;
    }




    public Object get(int i) {
        Object rc = null;
        i = i-lowestIndexStored;
        if (i >= 0 && i < currentSize) {
            i = i+offsetOfLowestIndexElement;
            if (i >= maxSize) {
                i = i-maxSize;
            }
            rc = cache[i];
        }
        return rc;
    }
} // end of SlidingArray.

// end of file SlidingArray.java
