package org.eso.ohs.core.dbb.client;

// $Id: DbbFormatterDateTextField.java,v 1.1 2004/02/17 11:16:24 tcanavan Exp $

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JFormattedTextField;
import javax.swing.text.MaskFormatter;

import org.eso.ohs.core.dbb.sql.DbbSqlChunk;

/**  
 * This is an implementation of the DbbWidget abstract class,
 * defining a text field widget.
 * 
 * <br>
 * <br>
 * @author Carlo Boarotto, ESO 20-Jul-1999
 * <br>ChangeLog: cmavros 08-Oct-2001 Add a default 
 *   column size to the DbbTextField.
 * <p>
 * 
 */

public class DbbFormatterDateTextField extends DbbWidget implements KeyListener {

    /** Deafult width for text widgets */
    public final static int COLUMNSIZE = 15;

    //----------------------------------------
    // Constructors
    //----------------------------------------

    /** Create a DbbTextField object.
     * 
     * @param labeltext	Label text to be displayed.
     * @param chunk	DbbSqlChunk object
     * @param width	Width of the widget in characters
     */
	public DbbFormatterDateTextField( String labeltext, DbbSqlChunk chunk, int width ) {

		super(labeltext, chunk);
		setDefault( "" );
		MaskFormatter fmt = null;
		try{
			fmt = new MaskFormatter("ULL ##, ####");
			fmt.setPlaceholderCharacter('*');
		} catch (java.text.ParseException e) {
			stdlog_.debug(e);
		}
		JFormattedTextField tf = new JFormattedTextField(fmt); 
		tf.setFocusLostBehavior(JFormattedTextField.PERSIST);
		widget_ = tf;
		label_.setLabelFor(widget_);
		((JFormattedTextField) widget_).addKeyListener(this);
	}

    /** Create a DbbTextField object with a default width.
     * 
     * @param labeltext	Label text to be displayed.
     * @param chunk	DbbSqlChunk object
     * @see #COLUMNSIZE
     */
	public DbbFormatterDateTextField(String labeltext, DbbSqlChunk chunk) {
		this( labeltext, chunk, COLUMNSIZE );
    }




    //----------------------------------------
    // Public Methods
    //----------------------------------------

    /** 
     * Get the widget value.
     *
     * @return The currently displayed string value. 
     */
    public String get() {
    	String retValue = ((JFormattedTextField) widget_).getText();
    	if ("*** **, ****".equals(retValue)) {
    		retValue = "";
    	}
	return retValue;
    }

    /** 
     * Set the displayed string item.
     * @param Integer value to be displayed.
     */
    public void set(Object val) {
	((JFormattedTextField) widget_).setText(String.valueOf(val));
	fireDbbWidgetAction();
    }

    /**
     * Reset the widget to its default state
     */
    public void reset () {
	if(widget_.isEnabled())
	    set( "" );
	else 
	    set(((JFormattedTextField)widget_).getText());
    }

    /** 
     * Set the widget value parameters.
     * @param s String value to be displayed.
     */
    public void set(String s) {
	((JFormattedTextField) widget_).setText(s);
	fireDbbWidgetAction();
    }

    /**
     * Method to check if the widget contents have been cleared.
     *
     * @return A boolean value: <code>true</code> if it is not empty,
     *         <code>false</code> otherwise.  
     */
    public boolean isEmpty() {
	String sel = get();
	return (sel.equals( "" ));
    }

 

    //----------------------------------------
    // Event methods
    //----------------------------------------

    /** Listener interface implementation. Not used.*/
    public void keyPressed(KeyEvent ev) {
    }

    /** Listener interface implementation. Not used.*/
    public void keyTyped(KeyEvent ev) {
    }

    /**
     * Listener interface implementation.
     * @param ev	 Event triggering the action.
     */
    public void keyReleased(KeyEvent ev) {
	fireDbbWidgetAction();
    }
}

