package org.eso.ohs.core.dbb.client;

import java.awt.Component;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.Action;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import org.apache.log4j.Logger;

/**
 * Keeps the menu widget configuration actions, folders, menuItem reference in a map.
 * This object is intended to be used by SaveConfigAction and DelConfigAction
 * <br>
 * @version $Revision:
 */
public class MenuMap {

    public static final String cvsID_ =
        "$Id: MenuMap.java,v 1.1 2005/02/11 10:41:43 tcanavan Exp $";

    private Hashtable menuFoldersMap_ = null;
    private Hashtable menuItemsMap_ = null;
    private Hashtable menuActionMap_ = null;

    private static Logger stdlog_ = Logger.getLogger(MenuMap.class);

    /**
     * initializes the menuItem, Actions, Folder map
     */
    public MenuMap() {
        menuItemsMap_ = new Hashtable();
        menuActionMap_ = new Hashtable();
        menuFoldersMap_ = new Hashtable();
    }

    /**
     *
     * @return the menu folder or null if does'n found
     */
    public JMenu getMenuFolder(String menuFolder) {
        return (JMenu)menuFoldersMap_.get(menuFolder);
    }

    /**
     *
     * @param menuName the menu name is in the main window
     * @return the action to which the menu is associated or null if does'n found
     */
    public Action getActionMenu(String menuName) {
        MenuKey menuKey = new MenuKey(menuName);
        return getActionMenu(menuKey);
    }

    /**
     *
     * @param menuFolder the menu folder
     * @param menuName the menu name is in folder menu
     * @return the action to which the menu is associated or null if does'n found
     */
    public Action getActionMenu(String menuFolder, String menuName) {
        MenuKey menuKey = new MenuKey(menuFolder, menuName);
        return getActionMenu(menuKey);
    }

    /**
     *
     * @param menuKey the menu key map
     * @return the action to which the menu is associated or null if does'n found
     */
    protected Action getActionMenu(MenuKey menuKey) {
        Action action = null;
        for (Enumeration e = menuActionMap_.keys(); e.hasMoreElements();) {
            MenuKey next = (MenuKey)e.nextElement();
            if (menuKey.equals(next)) {
                action = (Action)menuActionMap_.get(next);
                break;
            }
        }
        return action;
    }

    /**
     *
     * @param menuFolder the menu folder name
     * @param menuItem container of submenus
     */
    public void addMenuFolder(String menuFolder, Component menuItem) {
        menuFoldersMap_.put(menuFolder, menuItem);
    }

    /**
     *
     * @param menuKey the menu key to menu item and action associated
     * @param menuItem menu
     * @param action associated to menuItem
     */
    protected void addMenuItem(MenuKey menuKey, Component menuItem, Action action) {
        menuItemsMap_.put(menuKey, menuItem);
        menuActionMap_.put(menuKey, action);
    }

    /**
     *
     * @param menuName the menu name is in main menu popup
     * @param menuItem menu
     * @param action associated to menuItem
     */
    public void addMenuItem(String menuName, Component menuItem, Action action) {
        addMenuItem(new MenuKey(menuName), menuItem, action);

    }

    /**
     *
     * @param menuFolder the menu folder name
     * @param menuName the menu name is in menuFolder
     * @param menuItem menu
     * @param action associated to menuItem
     */
    public void addMenuItem(
        String menuFolder,
        String menuName,
        Component menuItem,
        Action action) {
        addMenuItem(new MenuKey(menuFolder, menuName), menuItem, action);

    }

    /**
     * Removes action and menu associated
     * @param menuFolder the menu folder name
     * @param menuName the menu name is in menuFolder
     * @return the removed menu or null if not found
     */
    public JMenuItem removeMenuItem(String menuFolder, String menuName) {
        MenuKey menuKey = new MenuKey(menuFolder, menuName);
        return removeMenuItem(menuKey);
    }

    /**
     * Removes action and menu associated
     * @param menuName the menu name is in main popup window
     * @return the removed menu or null if not found
     */
    public JMenuItem removeMenuItem(String menuName) {
        MenuKey menuKey = new MenuKey(menuName);
        return removeMenuItem(menuKey);
    }

    /**
     * Removes action and menu associated
     * @param menuKey the menu key
     * @return the removed menu or null if not found
     */
    protected JMenuItem removeMenuItem(MenuKey menuKey) {
        JMenuItem menuItem = null;
        for (Enumeration e = menuItemsMap_.keys(); e.hasMoreElements();) {
            MenuKey next = (MenuKey)e.nextElement();
            if (menuKey.equals(next)) {
                menuItem = (JMenuItem)menuItemsMap_.get(next);
                menuItemsMap_.remove(next);
                menuActionMap_.remove(next);
                break;
            }
        }
        return menuItem;
    }

    /**
     * Removes menu folder if no menus are associated
     * @param menuKey the menuFolder
     * @return the removed menu or null if not found
     */
    protected JMenu removeMenuFolder(String menuFolder) {
        JMenu menu = null;
        boolean foundFolder = false;
        if (menuFolder != null) {
            for (Enumeration e = menuItemsMap_.keys(); e.hasMoreElements();) {
                MenuKey next = (MenuKey)e.nextElement();
                if (next.getFolder().equals(menuFolder)) {
                    stdlog_.warn(
                        "The menu: " + next.getName() + " is still present. "
                            + "The menu folder \"" + menuFolder
                            + "\" can be removed only if it is empty");
                    foundFolder = true;
                    break;
                }
            }
            if (!foundFolder) {
                stdlog_.debug("menuFolder = " + menuFolder);
                menu = (JMenu)menuFoldersMap_.remove(menuFolder);
            }
        }
        return menu;
    }

    /**
     * The Menu key to the maps of menuItems and actions
     */
    public class MenuKey {

        private String menuFolder_ = "";
        private String menuName_ = "";

        public MenuKey(String menuFolder, String menuName) {
            menuFolder_ = menuFolder;
            menuName_ = menuName;
        }

        public MenuKey(String menuName) {
            menuName_ = menuName;
        }

        public String getName() {
            return menuName_;
        }

        public String getFolder() {
            return menuFolder_;
        }

        public String toString() {
            return "Folder: \"" + menuFolder_ + "\" Name: \"" + menuName_ + "\"";
        }

        /**
         * @return true if obj is a MenuKey and has the same folder and name as this.
         */
        public boolean equals(Object obj) {
            boolean rc = false;
            MenuKey node = null;
            if (obj != null && obj instanceof MenuKey) {
                node = (MenuKey)obj;
                if (getFolder().equals(node.getFolder())) {
                    rc = getName().equals(node.getName());
                }
            }
            return rc;
        }
    }
}

