package org.eso.ohs.core.dbb.gui;

// $Id: DbbThreeStateButton.java,v 1.1 2004/02/17 11:16:25 tcanavan Exp $

import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.Icon;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.border.EmptyBorder;

/**
 * Three state button.
 * 
 * <br>
 * <br>
 * @author Carlo Boarotto, ESO 13-Jul-1999
 * <p>
 *  
 */
public class DbbThreeStateButton extends JCheckBox {

  /** State 0 icon. */
  private Icon stateZero_ = null;

  /** State 1 icon. */
  private Icon stateOne_ = null;

  /** State 2 icon. */
  private Icon stateTwo_ = null;

  /** Internal variable describing the button state. */
  private int state_ = 0;

  /** Maximum width. */
  private int max_w = 0;

  /** Maximum height. */
  private int max_h = 0;

  
  /**
   * Specific action listener to change the icon and the value when
   * the button is clicked.
   */
  protected class DbbThreeStateListener implements ActionListener { 
    
    protected DbbThreeStateButton button_;

    public DbbThreeStateListener(DbbThreeStateButton b) {
      button_ = b;
    }

    public void actionPerformed(ActionEvent event){
      int val = button_.get();
 
      if(val >= 2)
	val = 0;
      else
	val++;
      button_.set(val);
    }
  }


  //----------------------------------------
  // Constructors
  //----------------------------------------

  /**
   * Create a three state button, with three different icons
   * displaying the component state.
   *
   * @param zero	State 0 icon.
   * @param one		State 1 icon.
   * @param two		State 2 icon.
   */
  public DbbThreeStateButton(Icon zero, Icon one, Icon two) {

    super(null,zero);

    stateZero_ = zero;
    stateOne_ = one;
    stateTwo_ = two;

    setBorder( new EmptyBorder( new Insets( 1,1,1,1 )));

    ActionListener l = new DbbThreeStateListener(this);
    this.addActionListener(l);
  }

  //----------------------------------------
  // Public methods
  //----------------------------------------

  /**
   * Get the resource state.
   * @return An integer (0,1,2) according to the button state.
   */
  public int get() { return state_; }


  /**
   * Set the resource state (setting its internal variable and the
   * corresponding icon).
   *
   * @param val Integer value (0,1,2) to describe the state.  
   */
  public void set(int val) {
    if(val > 2)
      state_ = 0;
    else 
      state_ = val;
   
    switch(val) {
    case 0:
      setIcon(stateZero_);
      break;
    case 1: 
      setIcon(stateOne_);
      break;
    case 2: 
      setIcon(stateTwo_);
      break;
    }
  }


  //----------------------------------------
  // Event methods
  //----------------------------------------

  /**
   * Fire the events starting from the first registered
   * listener. It overrides the super class method (LIFO
   * access to the listener list).
   */
  protected void fireActionPerformed(ActionEvent e) {
    Object[] listeners = listenerList.getListenerList();
    for(int i=0; i< listeners.length; i++) {
      if(listeners[i] == ActionListener.class) {
	((ActionListener)listeners[i+1]).actionPerformed(e);
      }
    }
  }


  //----------------------------------------
  // Debug methods
  //----------------------------------------

  public static void main(String[] args) {

    DbbThreeStateButton b1 = new DbbThreeStateButton(new DbbArrowIcon(0),
					 new DbbArrowIcon(1),
					 new DbbArrowIcon(2));

    JFrame f = new JFrame("Test FancyButton");

    Container c = f.getContentPane();   
    c.setLayout(new FlowLayout());

    c.add(b1);

    WindowListener l = new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
	System.exit(0);
      }
    };

    f.addWindowListener(l);
    f.pack();
    f.setVisible(true);
  }
}
