package org.eso.ohs.core.dbb.sql;


import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.Convert;
/** 
 * This is a special OR chunk that does the following
 * fist is between value1 and value2 
 * or second is between value 1 and value2.
 * @author Nick Kornweibel for ESO
 */

public class BetweenChunkGroup extends DbbSqlChunkGroup {

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( BetweenChunkGroup.class );

    public BetweenChunkGroup(DbbSqlChunkGroupMember first,
            DbbSqlChunkGroupMember second) {
        addMember(first);
        addMember(second);
    }

    /**
     * See header for details on this expression.
     *
     * @return String containing the sql clause.
     */

    public String getExpressionClause() {
        String rc;
        DbbSqlChunk first = (DbbSqlChunk)members_.elementAt(0);
        DbbSqlChunk second = (DbbSqlChunk)members_.elementAt(1);
      
        String firstStr = null;
        String secondStr = null;

        // try and get a value for firstStr
        if (first.isActive()) {
	    
            firstStr = first.getValue();	    
            if ((firstStr != null) &&( firstStr.length() == 0)) {             
                firstStr = null;
            }
        }

        // try and get a value for secondStr
        if (second.isActive()) {
            secondStr = second.getValue();
            if (secondStr != null && secondStr.length() == 0) {
                secondStr = null;
            }
        }
        
        if(firstStr == null || secondStr == null) 
            return "";

        String firstStrExp = null;
        String secondStrExp = null;

        long firstTime = -1;
        long secondTime = -1;
        
        try {
            firstTime = new Integer(firstStr).longValue(); 
            secondTime = new Integer(secondStr).longValue();             
        } catch(NumberFormatException e) {
            return "";              
        }
        if(firstTime!=-1 && secondTime !=-1) {
            if(secondTime < firstTime) {
                 secondTime = secondTime + Convert.SEC_PER_DAY;
                 secondStr = new Long(secondTime).toString();        
            }
        }
        // create the expressions if we have good values.
        if (firstStr != null && secondStr != null) {
            DbbSqlOperator op = first.getOperator();
            firstStrExp = "("+first.getColumnSpec()
                +" "+"between"+" "+firstStr+" and " + secondStr + ")";                
            secondStrExp = "("+second.getColumnSpec()
                +" "+"between"+" "+firstStr+" and " + secondStr + ")";
            rc = "("+firstStrExp+" OR "+secondStrExp+")";           
            return rc;        
        }
        rc = "";
        return rc;
    }
}
