package org.eso.ohs.core.dbb.sql;

import java.io.Serializable;
import java.util.Vector;

/** 
 * This is an ABSTRACT CLASS for building composite
 * expressions in the WHERE clause of an SQL statement. 
 *
 * It is a group of DbbSqlChunkGroupMember objects that are used
 * to build composite expressions involving multiple SqlChunks.
 *
 * The composite expressions are returned for each requesting chunk member.
 * It the responsibility of the code collecting the chunks to ensure that
 * the same expression is not entered multiple time (though this does not
 * affect the resulting SQL statement.
 *
 * To extend this class, the getExpressionClause() method must be implemented
 * using data from the members_ member, and return the composite expression.
 * The handling of which chunk member the expression is returned through is
 * done in this class.
 *
 * Through this mechanism OR statements and nested conditions may be built
 * into the DBB.
 *
 * @author Nick Kornweibel for ESO
 */

public abstract class DbbSqlChunkGroup implements Serializable {

    protected Vector members_ = new Vector();

    /**
     * Get the group expression....to be implemented by extending class.
     *
     * @return String containing the sql clause.
     */

    public abstract String getExpressionClause();

    /**
     * Called from DbbSqlChunkGroupMember objects that are members
     * of this group.
     * The DbbSqlChunkGroupMember parameter is not currently used.
     */

    protected String getExpressionClause(DbbSqlChunkGroupMember m) {
        return getExpressionClause();
    }

    /**
     * Add an SqlChunkGroupMember to this group.
     * Set the members group to this.
     */

    public void addMember(DbbSqlChunkGroupMember m) {
        members_.addElement(m);
        m.setSqlChunkGroup(this);
    }

    /**
     * Remove an SqlChunkGroupMember from this group.
     * Set this members group to null.
     */

     public void removeMember(DbbSqlChunkGroupMember m) {
         members_.removeElement(m);
     }
}
