package org.eso.ohs.core.dbb.xml;

import java.io.IOException;
import java.io.Reader;
import java.sql.SQLException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import org.eso.ohs.core.dbb.client.DbbArrowButton;
import org.eso.ohs.core.dbb.client.DbbCheckBox;
import org.eso.ohs.core.dbb.client.DbbChoice;
import org.eso.ohs.core.dbb.client.DbbComboBox;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbTextField;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.client.DbbWidget;
import org.eso.ohs.core.dbb.server.DbbSession;
import org.eso.ohs.core.dbb.server.DbbSqlEngineImpl;
import org.eso.ohs.core.dbb.sql.DbbInvalidSqlOperatorType;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;
import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.xml.sax.Locator;

/**
 * An XML-based factory for DBB objects: builds DBB
 * instances by parsing an XML file.
 * <BR>
 * Implements the Singleton pattern.
 * </P>
 * 
 * See ExampleBrowser4.java for an example usage.
 *
 * @version $Revision: 1.3 $
 * @author amchavan 
 */

// $Id: DbbJdomFactory.java,v 1.3 2005/01/21 16:22:29 tcanavan Exp $

public class DbbJdomFactory {
	
	/** Global Variables - Panel Co-ordinates*/
	String type_;
	String position_;
	boolean visible_;
	int x_;
	int y_;  
    
	/** Singleton instance */
	private static DbbJdomFactory factory = null;

	///** The DbbPanel we are currently processing */
	//private DbbPanel currPanel = null;
    
	/** The fabricated DbbView */
	private DbbView dbbView = null;
    
	/** The list of dafault column chunks. */
	private List defcols = null;
    
	/** A registry of objects we create from the XML file.
	 * This objects will be referenced by other objects
	 * in different areas of the DBB.
	 */
	private HashMap elementRegistry = null;

	/** To keep track of where we are in the file */
	private Locator locator = null;

	/** The fabricated DbbSession */
	private DbbSession session = null;
    
	/** A registry of DBB widgets we create from the XML file.
	 */
	private HashMap widgetRegistry = null;
    
	///** A temporary list of DbbWidgets we create from the XML file. */
	//private List widgetsTMP = null;
    
	/** Private constructor (this is a Singleton) */
	private DbbJdomFactory() {
		// nothing to do...
	}
    
	/** Clean up all internal data structures */
	private void clear() {
		//widgetsTMP      = new LinkedList();
		defcols         = new LinkedList();
		elementRegistry = new HashMap();
		widgetRegistry  = new HashMap();
		//currPanel = null;
		dbbView   = null;
		session   = null;
	}
	
	/** @return The current input line. */
	private int currentLine() {
		// return locator.getLineNumber();
		// TODO: reimplement
		return -99;
	}

	/**
	 * @param text Text of the error message.
	 * @return A nicely formatted error message, including
	 *         input line number.
	 */
	private String errmsg( String text ) {
		StringBuffer sb = new StringBuffer();
		sb.append( "line " );
		sb.append( currentLine() );
		sb.append( ": " );
		sb.append( text );
		return sb.toString();
	}
    
	/**
	 * Utility method: filter out all non-Element children
	 * of an Element.
	 * 
	 * @param element A JDOM Element.
	 * @return An iterator over all child Elements of the
	 *         input Element.
	 */
	private Iterator getChildElements( Element element ) {
        
		Vector   children = new Vector();
		List      content = element.getContent();
		Iterator iterator = content.iterator();
        
		while( iterator.hasNext() ) {
			Object o = iterator.next();
			if( o instanceof Element ) {
				Element child = (Element) o;
				children.add( child );
			}
		}
		return children.iterator();
	}

//	  /** Fill up the current DbbPanel with the DbbSqlChunks
//	   * currently on the widgetsTMP list. */ 
//	  private void fillDbbPanel() {
//        
//		  // List widgetsTMP is populated with a bunch of DbbWidgets
//		  while( widgetsTMP.size() > 0 )           
//			  currPanel.addWidget( (DbbWidget) widgetsTMP.remove( 0 ));
//	  }
    
//	  /** @return The current DbbPanel. For debugging only. */
//	  public DbbPanel getCurrentPanel() {
//		  return currPanel;
//	  }

	/** @return The fabricated DbbSession */
	public DbbSession getDbbSession() {
		return session;
	}
    
	/** @return The fabricated DbbView */
	public DbbView getDbbView() {
		return dbbView;
	}
    
	/**
	 * Retrieve a DBB element by id.
	 * 
	 * @param id   Key to retrieve the object. If the input XML document
	 *             contains for instance an element like:
	 *             <pre>
	 *               &lt;DbbSqlChunk id="chunk1" column="surname_dl" ... /&gt;
	 *             </pre>
	 *             this method allows to retrieve the corresponding
	 *             <code>DbbSqlChunk</code> using key <em>chunk1</em>.
	 * 
	 * @return The object registered with the input id, if any;
	 *         <tt>null</tt> otherwise.
	 */
	public Object getElement( String id ) {
		return elementRegistry.get( id );
	}

	/** @return The registry of (non-widget) DBB elements, like chunks, etc.
	 *          This  is public for debugging purposes. */
	public HashMap getElementRegistry() {
		return elementRegistry;
	}
    
	/**
	 * @return All the elements defined in the input XML document.
	 */
	public Iterator getElements() {
		Collection values = elementRegistry.values();
		Iterator iterator = values.iterator();
		return iterator;
	}
    
	/**
	 * Retrieve a widget by ID.
	 * 
	 * @param id   Key to retrieve the object. If the input XML document
	 *             contains for instance an element like:
	 *             <pre>
	 *               &lt;DbbTextField chunk="chunk1" /&gt;
	 *             </pre>
	 *             this method allows to retrieve the that <code>DbbWidget</code>
	 *             using key <em>chunk1</em>.
	 * 
	 * @return The object registered with the input key, if any;
	 *         <tt>null</tt> otherwise.
	 */
	public DbbWidget getWidget( Object id ) {
		return (DbbWidget) widgetRegistry.get( id );
	}
    
	/** @return The registry of <code>DbbWidget</code>s created
	 *          by this factory when parsing the input XML document.
	 *          This method is public for debugging purposes.
	 */
	public HashMap getWidgetRegistry() {
		return widgetRegistry;
	}
    
	/**
	 * @return All the widgets created by this factory when parsing
	 *         the input XML document.
	 */
	public Iterator getWidgets() {
		Collection values = widgetRegistry.values();
		Iterator iterator = values.iterator();
		return iterator;
	}//getWidgets

	
	/** Make a DbbChoice*/
	//private void makeDbbChoice(Attributes attrs) throws DbbXMLException {
	private DbbChoice makeDbbChoice(HashMap attrs) throws DbbXMLException {
		if (attrs == null)
			throw new DbbXMLException (errmsg("no attributes"));
		 
		String aLabeltext 	= 	(String)attrs.get("labeltext");
		String aChunk		=	(String)attrs.get("chunk");
		 
		if(aChunk == null || aChunk.length() == 0)
			throw new DbbXMLException(errmsg("Empty chunk id"));
		DbbSqlChunk c = (DbbSqlChunk)getElement(aChunk);
		if(c == null)
			throw new DbbXMLException(errmsg("Chunk id '" + aChunk + "' not found"));
		 	
		if(aLabeltext == null || aLabeltext.length() == 0)
			aLabeltext = c.getTable().getTable() + "." + c.getColumn();
		 		
		DbbChoice choice = new DbbChoice (aLabeltext, c);
		putWidget(aChunk, choice);
		return choice;		 		
	}//makeDbbChoice
    
//	  /** @return The current list of DbbWidgets. It is public for debugging
//	   *          purposes. */
//	  public List getWidgetsTMP() {
//		  return widgetsTMP;
//	  }

	  /** @return true if input object is a DbbPanel, false otherwise */
	  private boolean isDbbPanel( Object o ) {
			boolean b;
	  		if(o == null) b = false;
	  		else {
				Class   c = o.getClass();
				Class   p = DbbPanel.class;
				b = p.isAssignableFrom( c );
	  		}
			return b;
	  }//isDbbPanel
	  
    
	  /** @return true if input object is a DbbWidget, false otherwise */
	  private boolean isDbbWidget( Object o ) {
	  	  boolean b;
	  	  if(o == null) b = false;
	  	  else {
			Class   c = o.getClass();
			Class   w = DbbWidget.class;
			b = w.isAssignableFrom( c );
	  	  }
		  return b;
	  }

    
	  private DbbArrowButton makeDbbArrowButton( HashMap attrs ) throws DbbXMLException {
    
		if( attrs.size() == 0 )
			throw new DbbXMLException( errmsg( "no attributes" ));
    
		  String aLabeltext = (String)attrs.get( "labeltext" );
		  String aChunk     = (String)attrs.get( "chunk" );
		  String aInitial   = (String)attrs.get( "init" );
    
		  if( aChunk == null || aChunk.length() == 0 )
			  throw new DbbXMLException( errmsg( "Empty chunk id" ) );
		  DbbSqlChunk c = (DbbSqlChunk) getElement( aChunk );
		  if( c == null )   
			  throw new 
				  DbbXMLException( errmsg( "Chunk id '" + aChunk + "' not found" ));
                
		  if( aLabeltext == null || aLabeltext.length() == 0 )
			  aLabeltext = c.getTable().getTable() + "." + c.getColumn();

		  int initial;
		  if( aInitial == null || aInitial.length() == 0 )
			  initial = 0;
		  else
			  initial = Integer.parseInt( aInitial );

		  DbbArrowButton ab = new DbbArrowButton( aLabeltext, c, initial );
		  putWidget( aChunk, ab );
		  return ab;
	  }//makeDbbArrowButton


	/** Make a DbbComboBox */
	//private void makeDbbComboBox (Attributes attrs) throws DbbXMLException {
	private DbbComboBox makeDbbComboBox (HashMap attrs) throws DbbXMLException {
		if(attrs == null)
			throw new DbbXMLException(errmsg("no attributes"));
			
		String aLabeltext = 	(String)attrs.get("labeltext");
		String aChunk = 		(String)attrs.get("chunk");
		String aItems = 		(String)attrs.get("items");
		String aDbArray = 		(String)attrs.get("dbArray");
		String aEmptyValue = 	(String)attrs.get("emptyValue");
		
		String aDelim 	   =	(String)attrs.get("delim");
		
		//chunk
		if(aChunk == null || aChunk.length() == 0)
			throw new DbbXMLException(errmsg("Empty chunk id"));
		DbbSqlChunk c = (DbbSqlChunk)getElement(aChunk);
		if(c == null)
			throw new DbbXMLException(errmsg("Chunk id '" + aChunk + "' not found"));

		//labeltext
		if(aLabeltext == null || aLabeltext.length() == 0)
			aLabeltext = c.getTable().getTable() + "." + c.getColumn();

		//~~~~~~~~~~~~~~~~~~~~~~~~~TBD~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		if(aItems == null || aItems.length() == 0 || aDelim == null || aDelim.length() == 0)
			throw new DbbXMLException("Empty Items list");
		else 
		{
			StringTokenizer st = new StringTokenizer(aItems, aDelim);
			Vector v = new Vector();								
			while (st.hasMoreElements()){
				v.addElement(st.nextToken());
			}//while
			Object[] obj = v.toArray();
			DbbComboBox cb = new DbbComboBox( aLabeltext, c, obj, obj, true);
//			widgetsTMP.add( cb );
			putWidget( aChunk, cb );
			return cb;
		}
		//~~~~~~~~~~~~~~~~~~~~~~~~~TBD~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~			 											
	}//makeDbbComboBox
	
	
	//private void makeDbbCheckBox( HashMap attrs ) throws DbbXMLException {
	private DbbCheckBox makeDbbCheckBox( HashMap attrs ) throws DbbXMLException {
    
		if( attrs.size() == 0 )
			throw new DbbXMLException( errmsg( "no attributes" ));
    
		// The DbbCheckBox constructor requires the following parameters:
		//         labeltext  Label text to be displayed.
		//         chunk  DbbSqlChunk object [MANDATORY]
    
		//String aLabeltext = attrs.getValue( "labeltext" );
		String aLabeltext = (String)attrs.get( "labeltext" );
		//String aChunk     = attrs.getValue( "chunk" );
		String aChunk     = (String)attrs.get( "chunk" );
    
		  if( aChunk == null || aChunk.length() == 0 )
			  throw new DbbXMLException( errmsg( "Empty chunk id" ) );
		  DbbSqlChunk c = (DbbSqlChunk) getElement( aChunk );
		  if( c == null )   
			  throw new 
				  DbbXMLException( errmsg( "Chunk id '" + aChunk + "' not found" ));
                
		  if( aLabeltext == null || aLabeltext.length() == 0 )
			  aLabeltext = c.getTable().getTable() + "." + c.getColumn();                    
		  DbbCheckBox cb = new DbbCheckBox( aLabeltext, c );
		  //widgetsTMP.add( cb );
		  putWidget( aChunk, cb );
		  return cb;
	  }//makeDbbCheckBox
	  
	  
    
	//private void makeDbbPanel( HashMap attrs, Element element )
	private DbbPanel makeDbbPanel( HashMap attrs, Element element )
		throws DbbXMLException {
                
		if( attrs.size() == 0 )
			throw new DbbXMLException( errmsg( "no attributes" ));
        
		// The DbbPanel constructor requires the following parameters:
		//      title      Title label of the panel. [MANDATORY]
		//      col_number Number of widget columns.  
        
		String aTitle   = (String) attrs.get( "title" );
		String aColNum  = (String) attrs.get( "colnumber" );
		String id       = (String) attrs.get( "id" );
		String type     = (String) attrs.get( "type" );
		String pos      = (String) attrs.get( "position" );
		String aVisible = (String) attrs.get( "visible" );
		String aX       = (String) attrs.get( "x" );
		String aY       = (String) attrs.get( "y" );                
        
		// Now some checks
		//-------------------------------------
                    
		if( type == null )
			throw new DbbXMLException( errmsg( "Missing DbbPanel type." ));
		else type_ = type;			

		if( aTitle == null )
			aTitle = "";
        
		int colNum = 0;
		if( aColNum != null && aColNum.length() != 0 )
			colNum = Integer.parseInt(aColNum);

		if( pos == null )
		   pos = "N";
		   position_ = pos;
        
		boolean visible = true;
		if( aVisible != null )
			visible = Boolean.valueOf( aVisible ).booleanValue();
			visible_ = visible;
        
		int x = -1;
		if( aX != null )
			x = Integer.valueOf( aX ).intValue();
			x_ = x;
        
		int y = -1;
		if( aY != null )
			y = Integer.valueOf( aY ).intValue();
			y_ = y;
            
		DbbPanel panel = new DbbPanel( aTitle, colNum );
		
		// Make this panel accessible from outside the factory
		putElement( aTitle, panel );
        
		// Now recurse down to the panel's children
		Iterator iterator = getChildElements( element );
		while( iterator.hasNext() ) {
			Element child = (Element) iterator.next();
			Object object = visit( child ); 
			//if( object != null && object instanceof DbbWidget)
			if( isDbbWidget(object))
				panel.addWidget( (DbbWidget)object );
		}
		return panel;
	}//makeDbbPanel

	private void makeDbbSession( HashMap attrs ) throws DbbXMLException {
    
		if( attrs.size() == 0 )
			throw new DbbXMLException( errmsg( "no attributes" ));
    
		// The DbbSession constructor requires the following parameters:
		// url        JDBC URL.
		// default_db Default database.
		// username   Login name.      [we'll pass some bogus value]
		// password   Login password.  [we'll pass some bogus value]
    
		String url   = (String) attrs.get( "url" );
		String defdb = (String) attrs.get( "defdb" );
    
		if( url == null || url.length() == 0 )
			throw new DbbXMLException( errmsg( "JDBC URL missing or empty" ));
                
		if( defdb == null || defdb.length() == 0 )
			throw new DbbXMLException( errmsg( "Default database missing or empty" ));
                    
		session = new DbbSession( url, defdb, "", "" );
	}
    
	private void makeDbbSqlChunk( HashMap attrs ) throws DbbXMLException {
                
		if( attrs.size() == 0 )
			throw new DbbXMLException( errmsg( "no attributes" ));
        
		// The DbbSqlChunk constructor requires the following parameters:
		//      DbbSqlTable table (MANDATORY)
		//      String column     (MANDATORY)
		//      String label
		//      DbbSqlOperator op
		//      Class data_type
		//      String function
        
		String id         = (String) attrs.get( "id" );		
		String aTable     = (String) attrs.get( "table" );
		String column     = (String) attrs.get( "column" );
		String label      = (String) attrs.get( "label" );
		String aOperator  = (String) attrs.get( "operator" );
		String aData_type = (String) attrs.get( "data_type" );
		String function   = (String) attrs.get( "function" );
		String aDefault   = (String) attrs.get( "default" );
                
		DbbSqlOperator op = null;
		DbbSqlTable table = null;
		Class   data_type = null;
        
		// Now some checks
		//-------------------------------------
        
		if( aTable == null )
			throw new DbbXMLException( errmsg( "No table specified" ));
        
		table = (DbbSqlTable) getElement( aTable );
		if( table == null )
			throw new DbbXMLException( errmsg( "table '" + aTable + "' not found"));
            
		if( column == null || column.length() == 0 )
			throw new DbbXMLException( "Column name not found" );
            
		if( aData_type == null || aData_type.length() == 0 )
			data_type = null;
		else
			try {
				data_type = Class.forName( aData_type );
			}
			catch (ClassNotFoundException e) {
				throw new DbbXMLException(
					errmsg( "class '" + aData_type + "' not found" ));
			}
            
		boolean def = false;
		if( aDefault != null && aDefault.length() > 0 )
			def = Boolean.valueOf(aDefault).booleanValue();
        
        
		// Some chunks have no operator.
		// If one is specified, look for it: if we don't find it
		// we try to create one.
		// If *that* fails, well, we're in trouble.
		//--------------------------------------------
		if( aOperator != null ) {
			op = (DbbSqlOperator) getElement( aOperator );
			if( op == null ) {
				try {
					op = new DbbSqlOperator( aOperator );
				}
				catch (DbbInvalidSqlOperatorType e1) {      
					throw new DbbXMLException( errmsg( e1.getMessage() ));
				}
			}
		}
            
		DbbSqlChunk chunk =
			new DbbSqlChunk( table, column, label, op, data_type, function );
        
		if( id != null )
			putElement( id, chunk );
		if( def ) {
			defcols.add( chunk );
		}
	}//makeDbbSqlChunk
    
    
	private void makeDbbSqlOperator( HashMap attrs ) throws DbbXMLException {
    
		String value = null;
        
		if( attrs.size() == 0 )
			throw new DbbXMLException( errmsg( "no attributes" ));
        
		value = (String) attrs.get( "value" );    
		if( value == null || value.length() == 0 )
			throw new DbbXMLException( errmsg( "empty operator value" ));
    
		DbbSqlOperator operator;
		try {
			operator = new DbbSqlOperator(value);
		} catch (DbbInvalidSqlOperatorType e) {
			throw new DbbXMLException( 
				errmsg( "invalid operator: '" + value + "'" ));
		}

		putElement( value, operator );
	}//makeDbbSqlOperator
	
    
	/**
	 * Create a DbbSqlTable and puts it in the elementRegistry.
	 * @param attrs
	 * @throws DbbXMLException
	 */
	private void makeDbbSqlTable( HashMap attrs ) throws DbbXMLException {
    
		if (attrs == null)
			throw new DbbXMLException(errmsg("no attributes"));
    
		// The DbbSqlTable constructor requires the following parameters:
		//      String db_name 
		//      String owner_name
		//      String table_name   (MANDATORY) 
		String db_name    = attrs.get("db_name").toString();
		String owner_name = attrs.get("owner_name").toString();
		String table_name = attrs.get("table_name").toString();
    
		if (table_name == null || table_name.length() == 0)
			throw new DbbXMLException(errmsg("Empty table name"));
    
		DbbSqlTable table = new DbbSqlTable(db_name, owner_name, table_name);
		putElement( table.toString(), table );
	}//makeDbbSqlTable
	

	private DbbTextField makeDbbTextField( HashMap attrs ) throws DbbXMLException {
    
		if( attrs.size() == 0 )
			throw new DbbXMLException( errmsg( "no attributes" ));
    
		// The DbbTextField constructor requires the following parameters:
		//         labeltext  Label text to be displayed.
		//         chunk  DbbSqlChunk object [MANDATORY]
		//         width  Width of the widget in characters
		String aLabeltext = (String) attrs.get( "labeltext" );
		String aChunk     = (String) attrs.get( "chunk" );
		String aWidth     = (String) attrs.get( "width" );
    
		if( aChunk == null || aChunk.length() == 0 )
			throw new DbbXMLException( errmsg( "Empty chunk id" ) );
		DbbSqlChunk c = (DbbSqlChunk) getElement( aChunk );
		if( c == null )   
			throw new 
				DbbXMLException( errmsg( "Chunk id '" + aChunk + "' not found" ));
                
		if( aLabeltext == null || aLabeltext.length() == 0 )
			aLabeltext = c.getTable().getTable() + "." + c.getColumn();
        
		int width;
		if( aWidth == null || aWidth.length() == 0 )
			width = DbbTextField.COLUMNSIZE;
		else
			width = Integer.parseInt( aWidth );
            
		DbbTextField tf = new DbbTextField( aLabeltext, c, width );
		putWidget( aChunk, tf );
		return tf;
	}//makeDbbTextField
	
    
	//private void makeDbbView( HashMap attrs, Element element ) throws DbbXMLException {
	private void makeDbbView( Element element ) throws DbbXMLException {
                
		//if( attrs.size() == 0 )
			//throw new DbbXMLException( errmsg( "no attributes" ));
		if( session == null )
			throw new DbbXMLException( errmsg( "no DbbSession found" ));
        
		// The DbbView constructor requires the following parameters:
		//      sqlEngine SQL engine of the browser [MANDATORY]
        
		DbbSqlEngineImpl e = null;
		try {
			e = new DbbSqlEngineImpl(session, false);
		}
		catch (SQLException e1) {
			// this in fact should never happen!
			e1.printStackTrace();
		}
        
		// List defcols is populated with DbbSqlChunks
		while( defcols.size() > 0 )           
			e.addDefaultColumnsChunk( (DbbSqlChunk) defcols.remove( 0 ));

		dbbView = new DbbView( e );
		
		//Now recurse down to add DbbPanel(s) to the view
		Iterator iterator = getChildElements( element );
		while( iterator.hasNext() ) {
			Element child = (Element) iterator.next();
			Object object = visit( child );
			//if( object != null && object instanceof DbbPanel)
			if( isDbbPanel(object))
			{ 			
				if(type_.equals("COLS")) {
					dbbView.setColumnsPanel((DbbPanel)object, position_, visible_, x_, y_);
				}
				else if(type_.equals("ROWS")){
					dbbView.setWherePanel((DbbPanel)object, position_, visible_, x_, y_);
				}					
				else if(type_.equals("SORT")) {
					dbbView.setSortPanel((DbbPanel)object, position_, visible_, x_, y_);
				}
				else{	
					throw new DbbXMLException(errmsg("Unknown DbbPanel type: " +
													 "should be one of COLS, ROWS, SORT."));										
				}//else
			}//if			
		}//while
	}//makeDbbView
	

	/**
	 * Parse an XML file and build all the DBB elements
	 * and widgets described therein.
	 * <BR>
	 * DBB elements like its view and session can be retrieved
	 * using accessor methods <code>getDbbSession()</code>,
	 * <code>getDbbView()</code>, <code>getElements()</code>
	 * and <code>getWidgets()</code>.
	 * 
	 * @param url  A Reader for the XML file/text to parse.
	 * 
	 * @throws JDOMException   If input file is not well-formed.
	 * @throws IOException     If input file cannot be read.
	 * @throws DbbXMLException If input file is not a valid DBB descriptor.
	 * 
	 * @see #getDbbSession()
	 * @see #getDbbView()
	 * @see #getElements()
	 * @see #getWidgets()
	 */    
	public void parse( Reader reader ) 
		throws JDOMException, IOException, DbbXMLException {
		SAXBuilder parser = new SAXBuilder();
		Document document = parser.build( reader );
		visit( document );
	}//parse

	/**
	 * Parse an XML file and build all the DBB elements
	 * and widgets described therein.
	 * <BR>
	 * DBB elements like its view and session can be retrieved
	 * using accessor methods <code>getDbbSession()</code>,
	 * <code>getDbbView()</code>, <code>getElements()</code>
	 * and <code>getWidgets()</code>.
	 * 
	 * @param url  URL of the XML file to parse.
	 * 
	 * @throws JDOMException   If input file is not well-formed.
	 * @throws IOException     If input file cannot be read.
	 * @throws DbbXMLException If input file is not a valid DBB descriptor.
	 * 
	 * @see #getDbbSession()
	 * @see #getDbbView()
	 * @see #getElements()
	 * @see #getWidgets()
	 */    
	public void parse( String url )
		throws JDOMException, IOException, DbbXMLException {
		SAXBuilder parser = new SAXBuilder();
		Document document = parser.build( url );
		visit( document );
	}//parse

   
	/**
	 * Register an element for future reference.
	 * @param key  Key to retrieve the object
	 * @param o    Object to register
	 * @throws DbbXMLException   If some object is already registered
	 *                           with the same key.
	 */
	private void putElement( String key, Object o ) throws DbbXMLException {
		if( getElement( key ) != null )
			throw new DbbXMLException( errmsg( "duplicate key: " + key ));
            
		elementRegistry.put( key, o );
	}//putElement
    
	/**
	 * Register a widget for future reference.
	 * @param key  Key to retrieve the object
	 * @param o    Object to register
	 * @throws DbbXMLException   If some object is already registered
	 *                           with the same key.
	 */
	private void putWidget( Object key, DbbWidget w ) throws DbbXMLException {
		if( getWidget( key ) != null )
			throw new DbbXMLException( errmsg( "duplicate key: " + key ));
            
		widgetRegistry.put( key, w );
	}//putWidget
    
	/**
	 * Converts a list of Attribute objects to a HashMap
	 * of name/value pairs.
	 */
	private HashMap toHashMap( List attributes ) {
		HashMap map = new HashMap(attributes.size());

		Iterator iterator = attributes.iterator();
		while (iterator.hasNext()) {
			Attribute attr = (Attribute) iterator.next();
			String   value = attr.getValue();
			String     key = attr.getName();
			map.put( key, value );
		}
		return map;
	}//toHashMap
	

	/**
	 * Process a JDOM document resulting from parsing a DBB descriptor
	 * and build all corresponding DBB elements.
	 * 
	 * @param document The input JDOM document.
	 * @return The input JDOM document, unchanged.
	 * 
	 * @throws DbbXMLException If input file is not a valid DBB descriptor.
	 */
	private void visit( Document document ) throws DbbXMLException {
		Element root = document.getRootElement();
		List content = root.getContent();
		visit( root );
	}//visit


	/**
	 * Process a JDOM tree node and build the corresponding DBB elements.
	 * 
	 * @param element  Input JDOM tree node.
	 * @return A DbbWidget if one was created; 
	 *         <code>null</code> otherwise.
	 */
	//private DbbWidget visit( Element element ) throws DbbXMLException {
	private Object visit( Element element ) throws DbbXMLException {

		String   name = element.getQualifiedName();
		List   lattrs = element.getAttributes();
		HashMap attrs = toHashMap( lattrs );
		//DbbWidget   w = null;
		//DbbPanel 	p = null;
		Object object = null;
        
		if( element.isRootElement() ) {
			Iterator iterator = getChildElements( element );
			while( iterator.hasNext() ) {
				Element child = (Element) iterator.next();
				visit( child );
			}
		}
		else if( name.equals( "DbbSqlChunk" ))    makeDbbSqlChunk( attrs );
		else if( name.equals( "DbbView" ))        makeDbbView( element );
		else if( name.equals( "DbbPanel" ))       object = makeDbbPanel( attrs, element );
		else if( name.equals( "DbbSqlTable" ))    makeDbbSqlTable( attrs );
		else if( name.equals( "DbbSqlOperator" )) makeDbbSqlOperator( attrs );
		else if( name.equals( "DbbTextField" ))   object = makeDbbTextField( attrs );
		else if( name.equals( "DbbCheckBox" ))    object = makeDbbCheckBox( attrs );
		else if( name.equals( "DbbArrowButton" )) object = makeDbbArrowButton( attrs );
		else if( name.equals( "DbbComboBox" ))    object = makeDbbComboBox( attrs );
		else if( name.equals( "DbbChoice" ))      object = makeDbbChoice( attrs ); 
		else if( name.equals( "DbbSession" ))     makeDbbSession( attrs );

		//return w;
		return object;
	}

	
	/** @return A DbbFactory object (Singleton pattern). */
	public static DbbJdomFactory getInstance() {
		if( factory == null )
			factory = new DbbJdomFactory();
		factory.clear();
		return factory;
	}
}
