package org.eso.ohs.core.docview.gui;

import java.awt.Color;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.eso.ohs.core.gui.widgets.JSpinnerInt;

/**
 * A GUI adaptor to interface between DocView and Swing JSpinner objects.
 * By using a GuiAdaptor, DocView itself can work with a much wider range
 * of GUI components.
 * <p>
 * $Id: SpinnerIntAdaptor.java,v 1.2 2004/02/17 12:44:06 tcanavan Exp $
 * <p>
 * @author Bob Kemp
 */
public class SpinnerIntAdaptor implements GuiAdaptor, ChangeListener {

    private DocView docView_;

    /** GUI component whose value is to be read/updated */
    private JSpinnerInt spinner_;

    /** Has the GUI component been modified?  Reset by DocView. */
    private boolean modified_;

    /**
     * Constructor
     *
     * @param spinner   GUI component to be linked to
     */
    public SpinnerIntAdaptor(JSpinnerInt spinner) {
        spinner_ = spinner;

        // The change must be attached to the model because the model
        spinner.addChangeListener(this);
    }

    /**
     * Set the background colour of the underlying GUI component
     *
     * @param bg        new value of background colour
     */
    public void setForeground(Color bg) {
        spinner_.setForeground(bg);
    }

    /**
     * Enable the underlying GUI component
     *
     * @param enabled   true if component should be enabled, false otherwise
     */
    public void setEnabled(boolean enabled) {
        spinner_.setEnabled(enabled);
    }

    /**
     * Is the underlying GUI component enabled?
     *
     * @return  true if component should be enabled, false otherwise
     */
    public boolean isEnabled() {
        return spinner_.isEnabled();
    }

    /**
     * Set a new value for the GUI components value
     *
     * @param newValue  new value represented as a String
     */
    public void setValue(String newValue) {
        if (newValue != null) {
            spinner_.setValue(new Integer(newValue).intValue());
        }
    }

    /**
     * Get the current value of the GUI component
     *
     * @return  current value represented as a String
     */
    public String getValue() {
        return String.valueOf(spinner_.getValue());
    }

    /**
     * Flag the component as having been modified (or not modified)
     *
     * @param   new value of the flag
     */
    public void setModified(boolean modified) {
        modified_ = modified;
    }

    /**
     * Has the GUI component been modified?
     *
     * @return  current value of the flag
     */
    public boolean isModified() {
        return modified_;
    }

    /** A listener for changes to the underlying GUI component */
    public void stateChanged(ChangeEvent ev) {
        setModified(true);
        docView_.updateObjectFromGui();
    }

    /**
     * This is an outrageous hack!
     * @param fieldRank_
     */
    public void addDocView(DocView fieldRank_) {
        docView_ = fieldRank_;        
    }
}

