package org.eso.ohs.core.gui.models;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;

/**
 * A Document model to only permit the input of a valid Minimum Strehl Ratio (0.0..1.0).
 * 
 */

public class StrehlRatioModel extends InputMaskModel {

    private final float SR_MAX = (float)100.0; // must be less than 100.0
    private final float SR_MIN = (float)0.0; // must be greater than or equal to 0.0

    /**
     * The regular expression describing legal buffer values.
     */
    private final static String legalValuesPattern_ =
	"\\d{0,2}(\\.\\d{0,2})?";

    /**
     * Constructor for the model.
     */
    public StrehlRatioModel() {
	super(legalValuesPattern_);
    }

    /**
     * Refuse input that is above or below the limits of the
     * textfield.
     * Note that 0.0 is accepted even though it is meaningless - this is because
     * the Strehl Ratio is not used by all instruments.
     */
    public void insertString(int offset, String str, AttributeSet a)
      throws BadLocationException {

        super.insertString(offset, str, a);
        try {
            String buffer_val = buffer_.toString();
            if (buffer_val.equals(".")){
                remove(offset, str.length());
            }
            else{
                Float f = Float.valueOf(buffer_val);
                if(f.floatValue() < SR_MIN) {
                    remove(offset, str.length());
                }
                else if(f.floatValue() >= SR_MAX) {
                    remove(offset, str.length());
                }
            }
        } catch (NumberFormatException e) {
            // nothing, the string is not complete...
        }
    }
}