package org.eso.ohs.core.gui.widgets;

import javax.swing.JFormattedTextField;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import org.apache.log4j.Logger;

/**
 * Spinner class to display integer values
 * <br>
 * $Id: JSpinnerInt.java,v 1.2 2004/04/08 12:09:51 tcanavan Exp $
 * <br>
 * @author C. Boarotto, ESO 13-Jan-2000
 * @version $Revision: 1.2 $
 */
public class JSpinnerInt extends JSpinner {

    private static Logger stdlog_ = Logger.getLogger(JSpinnerInt.class);

    public static final String rcsid =
        "$Id: JSpinnerInt.java,v 1.2 2004/04/08 12:09:51 tcanavan Exp $";

    /** Action event fired when changing the displayed value */
    // protected ActionEvent actionEvent_ = null;

    protected Integer default_;

    private SpinnerNumberModel snm_;


    /**
     * Create a JSpinnerInt widget
     *
     * @param defVal    Starting value
     * @param extent    Extension
     * @param min       Minimum value
     * @param max       Maximum value
     * @param wrap      Not currently supported
     *                  This used to allow cycling around
     */
    public JSpinnerInt(int defVal, int extent, int min, int max, boolean wrap) {
        setNewParams(defVal, extent, min, max, wrap);
    }


    /**
     * Create a new model for the JSpinnerInt widget
     *
     * @param defVal    Starting value
     * @param extent    Extension
     * @param min       Minimum value
     * @param max       Maximum value
     * @param wrap      Not currently supported
     *                  This used to allow cycling around
     */
    public void setNewParams(int defVal, int extent, int min, int max, boolean wrap) {
        snm_ = new SpinnerNumberModel(defVal, min, max, extent);
        setModel(snm_);
        default_ = new Integer(defVal);
    }


    /** @return the currently displayed integer value. */
    public Object getValue() {
        if (snm_ == null) {
            return null;
        }
        return snm_.getValue();
    }


    /**
     * Set the value to be displayed
     * @param val Integer value
     */
    public void setValue(int val) {
        snm_.setValue(new Integer(val));
    }


    /**
     * Set the value to be displayed
     * @param val String value (to be converted to integer)
     */
    public void setValue(String val) {
        snm_.setValue(new Integer(val));
    }


    /** Reset the widget and display its empty default value */
    public void clear() {
        snm_.setValue(default_);
    }


    /**
     * Check if the displayed value is the empty default value
     * @return The boolean result of the check
     *         With the SpinnerNumberModel, this is always false
     */
    public boolean isEmpty() {
        return (snm_.getValue() == null || snm_.getValue().equals(""));
    }


    /**
     * set background color of this widget
     *
     * @param bgCol new background color
     */
    public void setBackground(java.awt.Color bgCol) {
        stdlog_.debug("setting JSpinner bg color to: " + bgCol);
        JSpinner.NumberEditor numEd = (JSpinner.NumberEditor)getEditor();
        JFormattedTextField tf = numEd.getTextField();
        tf.setBackground(bgCol);
    }
}

