package org.eso.ohs.core.gui.widgets;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.PrintJob;
import java.awt.Toolkit;
import java.util.Properties;
import java.util.StringTokenizer;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.core.utilities.TextUtils;

/**
 * This is the basic print class for printing out a string buffer
 * Printing in Java 1.1.8 is awful. So this is the best we could do
 * It is much improved in 1.2
 * <br>
 * @version $Revision: 1.5 $
 */
public class Print {

    public static final String cvsID_ =
        "$Id: Print.java,v 1.5 2005/11/16 15:48:43 tcanavan Exp $";

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(Print.class);

    // the following two variables are not used anywhere (?)
    // private static JTextField text = new JTextField(40);
    // private static JButton button = new JButton("Print...");

    private static Font font = new Font("courier", Font.PLAIN, 10);
    private static FontMetrics fm;
    private static int fontHeight;
    private static int fontAscent;

    private static Properties props = new Properties();

    private static final String errorStr = "Print Error";


    /**
     * set the font
     * @param f the font
     */
    public static void setFont(Font f) {
        font = f;
    }
    
    
    public static String reformatBuffer(String srcString) {
        StringBuffer destBuffer = new StringBuffer();
        StringTokenizer buftoken = new StringTokenizer(srcString,"\n",true);
        while (buftoken.hasMoreElements()) {
            String data = buftoken.nextToken();
            if(data.length() > 80) {
                String [] splitStrings = TextUtils.splitString(data,80);
                for(int i=0;i<splitStrings.length;i++) {
                    destBuffer.append(splitStrings[i]);
                    if(i<splitStrings.length-1)
                         destBuffer.append("\n");    
                }
            } else {
                destBuffer.append(data);    
            }    
        }
        return destBuffer.toString();        
    }
    
    /**
     * set portrait or landscape mode
     * @param p true = portrait, false = landscape
     */
    public static void setPortrait(boolean p) {
        props.put("awt.print.orientation", (p ? "portrait" : "landscape"));
    }

    /**
     * Print a text buffer
     * @param frame
     * @param textBuffer to be printed
     */
    public static void print(JFrame frame, String textBuffer) {

        // port to java 1.4 fm = Toolkit.getDefaultToolkit().getFontMetrics(font);
        fm = frame.getFontMetrics(font);
        fontHeight = fm.getHeight();
        fontAscent = fm.getAscent();
        String message = "Printing file ...";
        stdlog_.debug(message);
        // text.setText(message);

        OHSConfig config = OHSConfig.getInstance();
        String defaultPrinter = config.getValue("DEFAULT.PRINTER");
        // stdlog_.debug("defaultPrinter=" + defaultPrinter);
        props.put("awt.print.printer", defaultPrinter);
        props.put("awt.print.paperSize", "a4");

        Toolkit tk = Toolkit.getDefaultToolkit();
        PrintJob pj;
        try {
            pj = tk.getPrintJob(frame, "Print File", props);
            stdlog_.debug("print job props: " + props);
        } catch (NullPointerException e) {
            message = "Printing failed";
            stdlog_.error(message + ": " + e.getMessage());
            // text.setText(message);
            JOptionPane.showMessageDialog(
                frame, message, errorStr, JOptionPane.ERROR_MESSAGE);
            return;
        }

        if (pj == null) {
            // text.setText("Printing cancelled");
            return;
        }

        int screenResolution = tk.getScreenResolution();
        int topMargin = screenResolution;       // 1 inch
        int leftMargin = screenResolution/2;    // 0.5 inches

        Dimension pageSize = new Dimension(pj.getPageDimension());
        pageSize.width -= 2 * leftMargin;
        pageSize.height -= 2 * topMargin;

        // Use try/finally to release PrintJob even if we get an exception
        try {
            Graphics g = null;
            int line = fontAscent;      // Pixel line on page
            int maxLine = pageSize.height - fontHeight;
            // int tabGap = 8 * fm.stringWidth(" ");
            // text.setText("Printing started");
            //String newTextBuffer = reformatBuffer(textBuffer);
            StringTokenizer buftoken = new StringTokenizer(textBuffer,"\n",true);
            while (buftoken.hasMoreElements()) {
                String data = buftoken.nextToken();
                if (data.charAt(0) == '\n') {
                    line += (data.length()) * fontAscent;
                    continue;
                }
                if (line > maxLine) {
                    stdlog_.debug("Next page");
                    g.dispose();
                    g = null;
                }
                if (g == null) {
                    g = pj.getGraphics();
                    if (g == null) {
                        message = "Cannot Print";
                        // text.setText(message);
                        message += "/Print to file\n" +
                            "No access to Printer/PrintGraphics";
                        stdlog_.error(message);
                        JOptionPane.showMessageDialog(
                            frame, message, errorStr, JOptionPane.ERROR_MESSAGE);
                        return;
                    }
                    g.translate(leftMargin, topMargin); // Skip margins
                    g.clipRect(0, 0, pageSize.width, pageSize.height);
                    g.setFont(font);
                    line = fontAscent;
                }
                g.drawString(data, 0, line);
                // line = line + fontAscent;
                // End of line: go to next line
                // int column = pageSize.width;
            }

            if (g != null) {
                g.dispose();    // Print the last page
            }

        } catch (Exception e) {
            message = "An error occured during Printing";
            stdlog_.error(message + ": " + e.getMessage());
            // text.setText("Printing failed");
            JOptionPane.showMessageDialog(
                frame, message, errorStr, JOptionPane.ERROR_MESSAGE);
        } finally {
            // Always terminate the print job
            pj.end();
            message = "Printing completed";
            stdlog_.debug(message);
            // text.setText(message);
        }
    }
}

