package org.eso.ohs.core.utilities;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.StarcatDate;

/**
 * 
 * Class DateConverter
 *
 * Utility class used to convert date from starcat format (tipically used in our database)
 * to a more readable formata (and viceversa).
 *
 * @version $Revision: 1.8 $
 * @author fsogni, May 4, 2005
 */
/**
 * Class DateConverter
 * 
 * TODO Javadoc
 * 
 * @version $Revision: 1.8 $
 * @author ddorigo, Jun 15, 2005
 */

// $Id: DateConverter.java,v 1.8 2005/06/15 16:50:56 fsogni Exp $
public class DateConverter {
    private static final String rcsid = "$Id: DateConverter.java,v 1.8 2005/06/15 16:50:56 fsogni Exp $";

    private static Logger stdlog_ = Logger.getLogger(DateConverter.class);

    static final long STARCAT_TO_UNIX_OFFSET = 315532800;

    static DateFormat dateFmtMedium_ = DateFormat.getDateInstance(DateFormat.MEDIUM);

    private final static String DEFAULT_ = "----------";

    public static Date longToDate(long val) {
        long time = (val + STARCAT_TO_UNIX_OFFSET) * 1000;
        Date d = new Date(time);
        return d;
    }

    public static long dateToLong(Date val) {
        long dateLong = val.getTime();
        long unixDate = dateLong / 1000;
        long time = unixDate - STARCAT_TO_UNIX_OFFSET;
        return time;
    }

    private static String[] MONTHS_LIST =
    { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct",
            "Nov", "Dec" };

    private static int[] MONTHS_LIST_INT =
    { Calendar.JANUARY, Calendar.FEBRUARY, Calendar.MARCH, Calendar.APRIL,
            Calendar.MAY, Calendar.JUNE, Calendar.JULY, Calendar.AUGUST,
            Calendar.SEPTEMBER, Calendar.OCTOBER, Calendar.NOVEMBER,
            Calendar.DECEMBER };

    private static DateFormat dateFmt_;

    /** Return months */
    public static String[] getMonths() {
        return MONTHS_LIST;
    }

    public static String longToString(long date) {
        String ret = dateFmtMedium_.format(longToDate(date));
        return ret;
    }

    /**
     * Convert a date to a starcat format.
     * 
     * @param value
     * @return
     */
    public static long stringToLong(String value) {

        long val = 0;

        if (value != null) {
            value = value.trim();
            try {
                Date d = dateFmtMedium_.parse(value);
                long time = d.getTime();
                val = time / 1000 - STARCAT_TO_UNIX_OFFSET;
            } catch (ParseException e) {
                if (!value.equals("true") && !value.equals("false")) {
                    stdlog_.warn("DATE CONV.ERR. while converting " + value, e);
                }
            }
        }

        return val;
    }

    /**
     * Get value for reqTime property converted to a float value
     * 
     * @return the float value of the property
     */
    public static float convertStringReqTimeToFloat(String reqTime) {
        String tmp_val = "";
        float time_requested = 0;

        // Set the index to the first occurence of '=' or the end of
        // the string
        int idx = reqTime.indexOf('=');
        if (idx == -1)
            idx = reqTime.length();

        tmp_val = reqTime.substring(0, idx - 1);
        Float f = new Float(tmp_val);
        return f.floatValue();
    }

    /**
     * 
     * Return the calender month from its sequence number Eg. 5 --> May
     * 
     * @param calendarMonth
     * @return String month
     */
    public static String getMonth(int calendarMonth) {
        String month = "";
        if (calendarMonth < MONTHS_LIST_INT.length) {
            for (int i = 0; i < MONTHS_LIST_INT.length; i++) {
                if (calendarMonth == MONTHS_LIST_INT[i]) {
                    month = MONTHS_LIST[i];
                    break;
                }
            }
        }
        return month;
    }

    
    /**
     * @param month
     * @return int index of month
     */
    public static int monthToCalendarMonth(String month) {
        int calendarMonth = Calendar.JANUARY;
        for (int i = 0; i < MONTHS_LIST.length; i++) {
            if (MONTHS_LIST[i].equals(month)) {
                calendarMonth = MONTHS_LIST_INT[i];
                break;
            }
        }
        return calendarMonth;
    }

    /**
     * Convert a Date object to a String
     * 
     * @param dateObj
     * @return String date
     */
    public static String dateToString(Date dateObj) {
        return dateFmtMedium_.format(dateObj);
    }

    /**
     * Converts the Date to a db Date
     * 
     * @param date
     * @return
     */
    public long formatDateToDbValue(String date) {
        StarcatDate catDate = new StarcatDate();
        Object obj = catDate.toDatabaseData(date);
        String s = obj.toString();
        Long longValue = new Long(s);
        long longVal = longValue.longValue();
        return longVal;
    }

    /**
     * Return a date format for format "day month year" As a separator,
     * "/:,-". Month 3 letters case insensitive. year full
     * 
     * @param date
     * @return boolean
     */
    public static Date getDateFormatDMY(String date) {
        Date d = null;
        if (dateFmt_ == null) {
            dateFmt_ = new SimpleDateFormat("d MMM yy");
        }
        StringBuffer value = new StringBuffer();
        if (date != null){    
        StringTokenizer tokens = new StringTokenizer(date,"[/:,- ]");
        if (tokens.countTokens() == 3) {
           String day = tokens.nextToken();
           String month = tokens.nextToken();
           String year = tokens.nextToken();
           String start = month.substring(0,1).toUpperCase();
           String end = month.substring(1).toLowerCase();
           value.append(day).append(" ").append(start).append(end).append(" ").append(year); 
        }
        }
        try {
            d = dateFmt_.parse(value.toString());
        } catch (ParseException e) {
            stdlog_.error("DATE CONV.ERR. while converting " + date, e);
        }
        return d;
    }
    
    /**
     * Validates a date format for format "day month year" As a separator,
     * "/:,-". Month 3 letters case insensitive. year full
     * 
     * @param date
     * @return boolean
     */
    public static boolean validateDateFormatDMY(String date) {
        boolean result = true;
        if (getDateFormatDMY(date) == null) {
            result = false;
        }
        return result;
    }

    /**
     * 
     * Return the list of months in lower case.
     * 
     * @return String lowercase
     */
    public static String[] getLowerCaseMonths() {
        String[] months = getMonths();
        String[] lowerCaseMonths = new String[months.length];
        for (int i = 0; i < lowerCaseMonths.length; i++) {
            lowerCaseMonths[i] = months[i].toLowerCase();
        }
        return lowerCaseMonths;
    }
}