/*
 * Created by amchavan on Jun 6, 2005
 */
package org.eso.ohs.core.utilities;

import java.io.IOException;
import java.io.Reader;
import java.util.StringTokenizer;

/**
 * Class ExtendedExtendedParamfile
 * 
 * TODO Javadoc
 * 
 * @version $Revision: 1.1 $
 * @author amchavan, Jun 6, 2005
 */

// $Id: ExtendedParamfile.java,v 1.1 2005/06/09 06:58:53 amchavan Exp $
public class ExtendedParamfile extends Paramfile {

    /**
     * Default constructor -- not visible. Creates an empty param file object.
     */
    private ExtendedParamfile() {
        super();
    }

    /**
     * Parse the first lines of an input file and extract all the keyword/value
     * pairs we can recognize. Will use the default max number of
     * ExtendedParamfile lines.
     * 
     * @param filename
     *            Pathname of the file to parse
     * @see #defaultMaxNumLines_
     * 
     * @throws IOException
     *             If reading the file generated some I/O error.
     * @throws NotASCIIException
     *             If file contains non-ASCII data, including any 8-bit
     *             character, or another syntax error.
     */
    public ExtendedParamfile( String filename ) throws IOException,
            NotASCIIException {
        this( filename, defaultMaxNumLines_ );
    }

    /**
     * Parse the first lines of an input file and extract all the keyword/value
     * pairs we can recognize. Will use the default max number of
     * ExtendedParamfile lines.
     * 
     * @param Reader
     *            text reader with input to be parsed
     * @see #defaultMaxNumLines_
     * 
     * @throws IOException
     *             If reading the file generated some I/O error.
     * @throws NotASCIIException
     *             If file contains non-ASCII data, including any 8-bit
     *             character, or another syntax error.
     */
    public ExtendedParamfile( Reader rdr ) throws IOException,
            NotASCIIException {
        this( rdr, defaultMaxNumLines_ );
    }

    /**
     * Parse the first lines of an input file and extract all the keyword/value
     * pairs we can recognize.
     * 
     * @param rdr
     *            A reader for the file to parse
     * @param maxNumLines
     *            Max number of ExtendedParamfile lines to be parsed.
     * 
     * @throws IOException
     *             If reading the file generated some I/O error.
     * @throws NotASCIIException
     *             If file contains non-ASCII data, including any 8-bit
     *             character, or another syntax error.
     */
    public ExtendedParamfile( Reader rdr, int maxlines ) throws IOException,
            NotASCIIException {

        this();

        // Initialize this class' fields
        maxNumLines_ = maxlines;

        // Read in and parse input stream
        String inputtext = FileIOUtils.fileToString( rdr );
        matchLines( inputtext );
    }

    /**
     * Parse the first lines of an input file and extract all the keyword/value
     * pairs we can recognize.
     * 
     * @param filename
     *            Pathname of the file to parse
     * @param maxNumLines
     *            Max number of ExtendedParamfile lines to be parsed.
     * 
     * @throws IOException
     *             If reading the file generated some I/O error.
     * @throws NotASCIIException
     *             If file contains non-ASCII data, including any 8-bit
     *             character, or another syntax error.
     */
    public ExtendedParamfile( String filename, int maxNumLines )
            throws IOException, NotASCIIException {
        this();

        // Initialize this class' fields
        maxNumLines_ = maxNumLines;
        pathname_ = filename;

        // Read in and parse input file
        String inputtext = FileIOUtils.fileToString( filename );
        matchLines( inputtext );
    }

    /** 
     * TODO 
     * @throws NotASCIIException 
     */
    protected void matchLines( String text ) throws NotASCIIException {
        
        // Convert newlines to Unix-style -- i.e. make sure
        // there are no "\r" chars.
        String newline = System.getProperty( "line.separator" );
        text = text.replaceAll( newline, "\n" );
        
        int numLines = 0;
        StringBuffer lines = new StringBuffer();
        StringTokenizer st = new StringTokenizer( text, "\n", false );

        while( st.hasMoreTokens() ) {
            String l = st.nextToken();
            while( l.endsWith( "\\" )) {
                int n = l.length();
                l = l.substring( 0, n-1 );
                lines.append( l ).append( "\r" );
                l = st.nextToken();
            }
            lines.append( l ).append( "\n" );
            numLines++;
            if( numLines >= maxNumLines_ )
                break;
        }
    
        super.matchLines( lines.toString() );
        
        String[] keywords = getKeywords();
        for( int i = 0; i < keywords.length; i++ ) {
            String keyword = keywords[i];
            String value   = getValue( keyword ).replaceAll( "\r", newline );
            put( keyword, value );
        }
    }
}
