package org.eso.ohs.dfs.docviewdatatrans;



import org.eso.ohs.core.utilities.Convert;

/**
 *  A transducer to convert set/get of String values into set/get
 *  on an int property.  The property name is given to the
 *  constructor.
 *
 *  <p>
 *  For example, users will call it's
 *  setValue/getValue methods.  Those calls will be translated into
 *  calls to setX/getX on the object given to the constructor.
 *
 * @version $Id: DecAdaptor.java,v 1.2 2004/02/17 14:31:57 tcanavan Exp $
 * @author Bob Kemp
 */
public class DecAdaptor extends DegreeAdaptor {

    /**
     * Construct a transducer.
     */
    public DecAdaptor() {
	super();
    }

    /**
     * Construct a transducer to convert set/get of String values into
     * setX/getX on an int property of <code>object</code>.
     */
    public DecAdaptor(Object obj, String fieldName) {
	super(obj, fieldName);
    }

    /**
     * @see DegreeAdaptor#convertToMilliArcSeconds(String)
     * @see Convert#DDMMSSToMilliarcsec(String)
     */
    protected int convertToMilliArcSeconds( String value ) {
	return( Convert.DDMMSSToMilliarcsec( value ));
    }

    /**
     * @see DegreeAdaptor#convertToHMS(int)
     * @see Convert#milliarcsecToDDMMSS(int)
     *
     * <br>ChangeLog: A M Chavan, 04-Jul-2001 -- fixed 1448
     */
    protected String convertToHMS( int value ) {	
	return( Convert.milliarcsecToDDMMSS( value ));
    }
}

