package org.eso.ohs.dfs.docviewdatatrans;

import org.apache.log4j.Logger;
import org.eso.ohs.core.docview.datatrans.ObjAdaptor;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.dfs.TimeInterval;

/**
 * A transducer to convert set/get of String values into set/get
 * on a long integer property.  The target property's name is given
 * to the constructor.
 *
 * <p>
 * For example, users will call it's
 * setValue/getValue methods.  Those calls will be translated into
 * calls to setX/getX on the object given to the constructor.
 *
 * TimeInterval = int ":" int
 * TimeIntervalArray = TimeInterval ";" TimeInterval
 * 
 * <p>
 * $Id: STTimeIntervalsAdaptor.java,v 1.4 2005/01/20 13:35:48 pnunes Exp $
 */

public class STTimeIntervalsAdaptor extends ObjAdaptor {
    
    
    private static Logger stdlog_ = Logger.getLogger(STTimeIntervalsAdaptor.class);
    
       
    private static final TimeInterval[] tiArray_ = new TimeInterval[0];

    /**
     * Construct a transducer.
     */
    public STTimeIntervalsAdaptor() {
        super(tiArray_.getClass());
    }

    /**
     * Construct a transducer to convert set/get of String values into
     * setX/getX on an int property of <code>object</code>.
     */
    public STTimeIntervalsAdaptor(Object obj, String fieldName) {
        super(tiArray_.getClass(), obj, fieldName);
    }

    /**
     * Converts data from a String to a meta-level equivalent of the
     * actual property's type, i.e. to a java.lang.Long
     * representing a long int.  Used with reflection to set
     * the new property value.
     *
     * @param value the new value of the property.
     */
    public Object toMetaLevelObject(String value) {
        TimeInterval[] tis = TimeIntervalsAdaptor.parseTimeIntervalArrayString(value);
        for(int i=0;i<tis.length;i++) {
            if(tis[i].getEndTime() < tis[i].getStartTime()) {
                tis[i].setEndTime( tis[i].getEndTime() + (Convert.SEC_PER_DAY));
            }
        }
        return tis;
    }

    
    /**
     * Converts data to a String from the meta-level equivalent of the
     * actual property's type, i.e. from a java.lang.Long
     * to a long int.  Used with reflection to get
     * the current property value.
     *
     * @param value	value of the property.
     */
    public String fromMetaLevelObject(Object obj) {
        if (obj == null) {
            return "";
        }

        TimeInterval[] timeInts = (TimeInterval[]) obj;

        StringBuffer sb = new StringBuffer();

        for (int j = 0; j < timeInts.length; j++) {
            TimeInterval ti = timeInts[j];

            sb.append(ti.getStartTime());
            sb.append(':');
            sb.append(ti.getEndTime());
            sb.append(';');
        }

        return sb.toString();
    }

}
