package org.eso.ohs.instruments;

import java.io.IOException;
import java.text.DecimalFormat;

import org.eso.ohs.core.utilities.PafData;
import org.eso.ohs.core.utilities.PafFile;
import org.eso.ohs.core.utilities.TextUtils;

/**
 * This class 
 * @author tcanavan
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */


public class PixelToMM {


      private PafFile pafFile_;


      private double [] maskToCCDX_ = new double[3]; 

      private double [] maskToCCDY_ = new double[3];
   
      private double [] [] yMatrix_ = new double[4] [4];

      private double [] [] xMatrix_ = new double[4] [4]; 

   
      private StringBuffer buf_  = new StringBuffer();
     
      // This should be externalised later

      private double T_COEFF = 2.0e-05;

      private double scale_;

      private int quad_;

      private int maskNo_; 

      private int refNoVal_;

      private int arcNoVal_;

      private int slitNoVal_;

      private int rectNoVal_;

      private String dateTime_;

      private boolean alreadyInMilli_ = false;


      /* Apply X co-cordinate conversion */
     double convToMM(double matrix [] [], double  x, double y) {
        double totalValue=0;
        double element=0;
        int yPower=0;
        for(int i =0;i<matrix.length;i++) {
              for(int j=0;j<matrix.length;j++) {   
                  element = matrix[i][j] * Math.pow(y,yPower) * Math.pow(x,j);
                  totalValue = totalValue + element;                    
              }
              yPower++;
        }                  
        return totalValue;
    }


    public double convY(double Y0[],double matrix [] [], double  x, double y) {
       double linearTransform=Y0[0]+Y0[1]*y+Y0[2]*x;
       return (linearTransform+
               convToMM(matrix,x,y))*scale_;         
    }
    
    public double convX(double X0[],double matrix [] [], double  x, double y) {
       double linearTransform=X0[0]+X0[1]*x+X0[2]*y;
       return (linearTransform
               +convToMM(matrix,x,y))*scale_; 
    }


    private String quoteStringNewLine(String s) {
        return "\"" + s + "\"\n";
    }
 
    public double [] [] getMatrix(PafFile f,int matrixWidth,int matrixHeight,String identifier) {
        PafData [] idList = f.getIdentifierList();
        double [] [] matrix = new double[matrixWidth][matrixHeight];
        int index = 0;
        int k=0;
        int j=0;
        for(int i = 0;i<idList.length;i++) {
            if(idList[i].getIdentifier().equals(identifier + index + "_" + j )) { 
                matrix[k][j] = new Double(idList[i].getValue()).doubleValue();
                j++;
            }
            if(j==4) {
                index++;
                k++;
                j=0; 
            }               
        }
        return matrix;

    }

    public void setYMatrix() {
        yMatrix_ = getMatrix(pafFile_,4,4,"PRO.CCD.MASK.Y_");        
    }

    public void setXMatrix() {
        xMatrix_ = getMatrix(pafFile_,4,4,"PRO.CCD.MASK.X_");        
    }

    public void setMaskCCDForX() {
        maskToCCDX_[0] = new Double(pafFile_.getValueForKeyword("PRO.CCD.MASK.X0")).doubleValue();
        maskToCCDX_[1] = new Double(pafFile_.getValueForKeyword("PRO.CCD.MASK.XX")).doubleValue();
        maskToCCDX_[2] = new Double(pafFile_.getValueForKeyword("PRO.CCD.MASK.XY")).doubleValue();       
    }

    public void setMaskCCDForY() {
        maskToCCDY_[0] = new Double(pafFile_.getValueForKeyword("PRO.CCD.MASK.Y0")).doubleValue();
        maskToCCDY_[1] = new Double(pafFile_.getValueForKeyword("PRO.CCD.MASK.YY")).doubleValue();
        maskToCCDY_[2] = new Double(pafFile_.getValueForKeyword("PRO.CCD.MASK.YX")).doubleValue(); 
       
    }
    public void setQuadrantValue() {
         quad_ = new Integer(pafFile_.getValueForKeyword("OCS.CON.QUAD")).intValue();       
    }


    public void setScale() {
        double tRef=0;       
        double focusTemp = new Double(pafFile_.getValueForKeyword("INS.FOCU" + quad_ + ".TEMP")).doubleValue();
        if(pafFile_.keyExists("PRO.CCD.MASK.TEMP"))
            tRef = new Double(pafFile_.getValueForKeyword("PRO.CCD.MASK.TEMP")).doubleValue();
        scale_=(1+T_COEFF*(focusTemp-tRef));

    }

    public double getScale() {
        return scale_;
    }
   

    public void setSlitData() {
        double xVal,yVal,dimXVal,dimYVal;   
        double bezierDimYVal,bezierX0Val,bezierAxVal,bezierBxVal,bezierCxVal;
        double bezierY0Val,bezierAyVal,bezierByVal,bezierCyVal;
        double mmBezierDimYVal,mmBezierX0Val,mmBezierAxVal,mmBezierBxVal,mmBezierCxVal;
        double mmBezierY0Val,mmBezierAyVal,mmBezierByVal,mmBezierCyVal;
        double X0,X1,X2,X3,Y0,Y1,Y2,Y3,mmX0,mmX1,mmX2,mmX3,mmY0,mmY1,mmY2,mmY3;
        DecimalFormat dformat = new DecimalFormat("0.000");
        DecimalFormat dcformat = new DecimalFormat("0.00000");
        String keyNoValues =  pafFile_.getValueForKeyword("INS.SLIT.NO");
        int numKeyValues = new Integer(keyNoValues).intValue();
        for(int i=1;i<=numKeyValues;i++) {
            String type = pafFile_.getValueForKeyword("INS.SLIT" + i + ".TYPE");         
            String val = pafFile_.getValueForKeyword("INS.SLIT" + i + ".X");             
            xVal=new Double(val).doubleValue();
            val = pafFile_.getValueForKeyword("INS.SLIT" + i + ".Y");             
            yVal=new Double(val).doubleValue();
            val = pafFile_.getValueForKeyword("INS.SLIT" + i + ".DIMX");
            dimXVal= new Double(val).doubleValue();
            val = pafFile_.getValueForKeyword("INS.SLIT" + i + ".DIMY");
            dimYVal= new Double(val).doubleValue();
            if(!alreadyInMilli_) { 
                setMM("INS.SLIT",xVal,yVal,dimXVal,dimYVal,i);
            }
            else {
                 pafFile_.setKeywordValue("INS.SLIT" + i + ".X",TextUtils.padLeft(dformat.format(xVal),8,' '));
                 pafFile_.setKeywordValue("INS.SLIT" + i + ".Y",TextUtils.padLeft(dformat.format(yVal),8,' '));
                 pafFile_.setKeywordValue("INS.SLIT" + i + ".DIMX",TextUtils.padLeft(dformat.format(dimXVal),8,' '));
                 pafFile_.setKeywordValue("INS.SLIT" + i + ".DIMY",TextUtils.padLeft(dformat.format(dimYVal),8,' '));
             
            }
            if(type.equals("CURVE")) {
                 val = pafFile_.getValueForKeyword("INS.SLIT" + i + ".BEZIER.DY");
                 bezierDimYVal= new Double(val).doubleValue(); 
                 val = pafFile_.getValueForKeyword("INS.SLIT" + i + ".BEZIER.AX");                
                 bezierAxVal=new Double(val).doubleValue();
                 val = pafFile_.getValueForKeyword("INS.SLIT" + i + ".BEZIER.BX");             
                 bezierBxVal=new Double(val).doubleValue();
                 val = pafFile_.getValueForKeyword("INS.SLIT" + i + ".BEZIER.CX");               
                 bezierCxVal=new Double(val).doubleValue();                      
                 val = pafFile_.getValueForKeyword("INS.SLIT" + i + ".BEZIER.XX");                
                 bezierX0Val=new Double(val).doubleValue();
                 val = pafFile_.getValueForKeyword("INS.SLIT" + i + ".BEZIER.AY");                
                 bezierAyVal=new Double(val).doubleValue();
                 val = pafFile_.getValueForKeyword("INS.SLIT" + i + ".BEZIER.BY");               
                 bezierByVal=new Double(val).doubleValue();
                 val = pafFile_.getValueForKeyword("INS.SLIT" + i + ".BEZIER.CY");               
                 bezierCyVal=new Double(val).doubleValue();
                 val = pafFile_.getValueForKeyword("INS.SLIT" + i + ".BEZIER.YY");
                 bezierY0Val=new Double(val).doubleValue();
                 X0=bezierX0Val;
                 X1=X0+bezierCxVal/3.0;
                 X2=X1+(bezierCxVal+bezierBxVal)/3.0;
                 X3=X0+bezierCxVal+bezierBxVal+bezierAxVal;
                 Y0=bezierY0Val;
                 Y1=Y0+bezierCyVal/3.0;
                 Y2=Y1+(bezierCyVal+bezierByVal)/3.0;
                 Y3=Y0+bezierCyVal+bezierByVal+bezierAyVal;
                 mmX0=convX(maskToCCDX_,xMatrix_,X0,Y0);
                 mmY0=convY(maskToCCDY_,yMatrix_,X0,Y0);
                 mmX1=convX(maskToCCDX_,xMatrix_,X1,Y1);
                 mmY1=convY(maskToCCDY_,yMatrix_,X1,Y1);
                 mmX2=convX(maskToCCDX_,xMatrix_,X2,Y2);
                 mmY2=convY(maskToCCDY_,yMatrix_,X2,Y2);
                 mmX3=convX(maskToCCDX_,xMatrix_,X3,Y3);
                 mmY3=convY(maskToCCDY_,yMatrix_,X3,Y3);
                 mmBezierDimYVal=Math.abs(convY(maskToCCDY_,yMatrix_,X0,Y0+bezierDimYVal/2.0)-convY(maskToCCDY_,yMatrix_,X0,Y0-bezierDimYVal/2.0));
                 mmBezierX0Val=mmX0;
                 mmBezierCxVal=3*(mmX1-mmX0);
                 mmBezierBxVal=3*(mmX2-mmX1)-mmBezierCxVal;
                 mmBezierAxVal=mmX3-mmX0-mmBezierCxVal-mmBezierBxVal;
                 mmBezierY0Val=mmY0;
                 mmBezierCyVal=3*(mmY1-mmY0);
                 mmBezierByVal=3*(mmY2-mmY1)-mmBezierCyVal;
                 mmBezierAyVal=mmY3-mmY0-mmBezierCyVal-mmBezierByVal;
                 if(!alreadyInMilli_) {
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.DY",TextUtils.padLeft(dcformat.format(mmBezierDimYVal),8,' '));
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.AX",TextUtils.padLeft(dcformat.format(mmBezierAxVal),8,' '));
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.BX",TextUtils.padLeft(dcformat.format(mmBezierBxVal),8,' '));
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.CX",TextUtils.padLeft(dcformat.format(mmBezierCxVal),8,' '));
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.XX",TextUtils.padLeft(dcformat.format(mmBezierX0Val),8,' '));
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.AY",TextUtils.padLeft(dcformat.format(mmBezierAyVal),8,' '));
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.BY",TextUtils.padLeft(dcformat.format(mmBezierByVal),8,' '));
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.CY",TextUtils.padLeft(dcformat.format(mmBezierCyVal),8,' '));
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.YY",TextUtils.padLeft(dcformat.format(mmBezierY0Val),8,' '));
                 }
                 else {
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.DY",TextUtils.padLeft(dcformat.format(bezierDimYVal),8,' '));
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.AX",TextUtils.padLeft(dcformat.format(bezierAxVal),8,' '));
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.BX",TextUtils.padLeft(dcformat.format(bezierBxVal),8,' '));
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.CX",TextUtils.padLeft(dcformat.format(bezierCxVal),8,' '));
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.XX",TextUtils.padLeft(dcformat.format(bezierX0Val),8,' '));
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.AY",TextUtils.padLeft(dcformat.format(bezierAyVal),8,' '));
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.BY",TextUtils.padLeft(dcformat.format(bezierByVal),8,' '));
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.CY",TextUtils.padLeft(dcformat.format(bezierCyVal),8,' '));
                     pafFile_.setKeywordValue("INS.SLIT" + i + ".BEZIER.YY",TextUtils.padLeft(dcformat.format(bezierY0Val),8,' '));
                 }
            }
        }
    }

/*if(id.equals("INS.SLIT") + j + "DIMX")) {
                buf.append(paf.toPafString());
            }
            if(id.equals("INS.SLIT") + j + "Y"))
                buf.append(paf.toPafString());            
            if(id.equals("INS.SLIT") + j + "DIMX"))
                buf.append(paf.toPafString());
            if(id.equals("INS.SLIT") + j + "DIMY"))
            buf.append(paf.toPafString());*/

//private class 

    public void appendMatrix() {
        for(int i=0;i<xMatrix_.length;i++) {
            for(int j=0;j<4;j++)
                buf_.append("PRO.CCD.MASK.X_" + i + "_" + j  + " \"" +
                xMatrix_[i][j] + "\"\n");
        }
        for(int i=0;i<xMatrix_.length;i++) {
            for(int j=0;j<4;j++)
                buf_.append("PRO.CCD.MASK.Y_" + i + "_" + j  + " \"" +
                yMatrix_[i][j] + "\"\n");
        }
    }


    public void setPAFValues() {              
        pafFile_.setKeywordValue("PAF.NAME",dateTime_ + "-101-" + quad_ + ".adm");
        pafFile_.setKeywordValue("PAF.DESC","ADM: Aperture Definition File in mm");         
    }
   

    public void setADFUnit() {
        pafFile_.setKeywordValue("INS.ADF.UNIT","MILLIMETER");  
    }


     public String getADFUnit() {
        return pafFile_.getValueForKeyword("INS.ADF.UNIT");  
    }
  

    public void setMaskId() {
        String maskNumStr =  new Integer(maskNo_).toString();
        pafFile_.setKeywordValue("INS.MASK" + quad_  + ".ID",maskNumStr);  
    }



     public void initRefData() {
        String val = pafFile_.getValueForKeyword("INS.REF.NO");
        refNoVal_=new Integer(val).intValue();
        val =  pafFile_.getValueForKeyword("INS.ARC.NO");
        arcNoVal_ = new Integer(val).intValue();
        val  = pafFile_.getValueForKeyword("INS.SLIT.NO");
        slitNoVal_= new Integer(val).intValue();
        rectNoVal_=slitNoVal_+refNoVal_-arcNoVal_;
     }


     public void setMM(String keyword,double xVal,double yVal,double dimXVal,double dimYVal,int index) {
         double mmXVal,mmYVal,mmXVal1,mmYVal1,mmXVal2,mmYVal2;
         double mmDimXVal,mmDimYVal;
         DecimalFormat dformat = new DecimalFormat("0.000");
         mmXVal=convX(maskToCCDX_,xMatrix_,xVal,yVal);
         mmYVal=convY(maskToCCDY_,yMatrix_,xVal,yVal);
         mmXVal1=convX(maskToCCDX_,xMatrix_,xVal-dimXVal/2.0,yVal);
         mmXVal2=convX(maskToCCDX_,xMatrix_,xVal+dimXVal/2.0,yVal);
         mmYVal1=convY(maskToCCDY_,yMatrix_,xVal,yVal-dimYVal/2.0);
         mmYVal2=convY(maskToCCDY_,yMatrix_,xVal,yVal+dimYVal/2.0);
         mmDimXVal=Math.abs(mmXVal2-mmXVal1);
         mmDimYVal=Math.abs(mmYVal2-mmYVal1);

             //pafFile_.setKeywordValue("INS.REF" + n + ".X",mew
             //String(new Integer(mmXVal));
         pafFile_.setKeywordValue(keyword + index + ".X",TextUtils.padLeft(dformat.format(mmXVal),8,' '));
         pafFile_.setKeywordValue(keyword + index + ".Y",TextUtils.padLeft(dformat.format(mmYVal),8,' '));
         pafFile_.setKeywordValue(keyword + index + ".DIMX",TextUtils.padLeft(dformat.format(mmDimXVal),8,' '));
         pafFile_.setKeywordValue(keyword + index + ".DIMY",TextUtils.padLeft(dformat.format(mmDimYVal),8,' '));
     }


     public void setRefData() {
         double xVal,yVal,dimXVal,dimYVal;                
          DecimalFormat dformat = new DecimalFormat("0.000");
         for(int n=1;n<=refNoVal_;n++) {
             String val = pafFile_.getValueForKeyword("INS.REF" + n + ".X");             
             xVal=new Double(val).doubleValue();
             val = pafFile_.getValueForKeyword("INS.REF" + n + ".Y");             
             yVal=new Double(val).doubleValue();
             val = pafFile_.getValueForKeyword("INS.REF" + n + ".DIMX");
             dimXVal= new Double(val).doubleValue();
             val = pafFile_.getValueForKeyword("INS.REF" + n + ".DIMY");
             dimYVal= new Double(val).doubleValue();
             if(!alreadyInMilli_)           
                 setMM("INS.REF",xVal,yVal,dimXVal,dimYVal,n);
             else {
                 pafFile_.setKeywordValue("INS.REF" + n + ".X",TextUtils.padLeft(dformat.format(xVal),8,' '));
                 pafFile_.setKeywordValue("INS.REF" + n + ".Y",TextUtils.padLeft(dformat.format(yVal),8,' '));
                 pafFile_.setKeywordValue("INS.REF" + n + ".DIMX",TextUtils.padLeft(dformat.format(dimXVal),8,' '));
                 pafFile_.setKeywordValue("INS.REF" + n + ".DIMY",TextUtils.padLeft(dformat.format(dimYVal),8,' '));
             }          
         }
    }


    public String createAdmString() {
        System.out.println("TEMP COEFF " + T_COEFF);
        setPAFValues();
        if(!alreadyInMilli_){
            setADFUnit();           
        }
        setMaskId();
        setRefData();        
        setSlitData();
        return pafFile_.toFileString();
  
    }



    //public float getMaskTemperature() {
    //}

    public PixelToMM(PafFile f,int maskNo,String dateTime) {
        pafFile_ = new PafFile(f);
        MaskInstrument inst = (MaskInstrument)InstrumentList.getInstance().getInstrument("VIMOS");
        T_COEFF = inst.getTempCoeff();
        setQuadrantValue();
        maskNo_ = maskNo;
        dateTime_ = dateTime;
        System.out.println("ADP FILE " + f.getPafName());
        if(!getADFUnit().equals("MILLIMETER")) {
            setMaskCCDForX();     
            setMaskCCDForY();
            setXMatrix();
            setYMatrix();
            setScale();  
            initRefData();
        }
        else {
            alreadyInMilli_ = true;
        }
    }


    public void covertPafHeader() {
        
    }



    public static void main(String args[]) {
        PafData [] idList=null;
        try {
            PafFile f = new PafFile(args[0]);
                   
            PixelToMM pix = new PixelToMM(f,new Integer(args[1]).intValue(),TextUtils.currentISODateTime());
            System.out.println(pix.createAdmString());
            //String header = pix.createHeader();
            //pix.displayMatrix();
            //pix.transformCoordinates();
        }
        catch(IOException e) {
            e.printStackTrace();
        }
        //catch(FileNotFoundException e) {
        //}
        // PafData [] idList = pafFile_.getIdentifierList();

        /*for(int i=0;i<idList.length;i++) {
            double x,y;
            int j=1;
            if(idList[i].getValue().indexOf(idList[i].getIdentifier() + j + ".X")!=-1) {
                x = new Double(idList[i].getValue()).doubleValue();
            }
            if(idList[i].getValue().indexOf(idList[i].getIdentifier() + j + ".Y")!=-1) {
                y = new Double(idList[i].getValue()).doubleValue();
            }
            j++;
            convXCoordinate(maskToCCDX_,x,y,);
       }*/


    }


}
