// -*- Java -*-
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//.IDENT     $Id: TemplateSignatureSummary.java,v 1.5 2003/07/03 08:49:49 ddorigo Exp $
//.LANGUAGE  Java
//.AUTHOR    T. Canavan, ESO
//.PURPOSE   Record in index file fo the tsf's associated with an instrrument.
//-----------------------------------------------------------------------------

package org.eso.ohs.instruments;

import java.io.Serializable;


/**
 * This class contains the summary information an instrument requires for each
 * of its template signature files. This class inherits from Serializable 
 * because it gets read from disk at runtime
 * <p>
 * $Id: TemplateSignatureSummary.java,v 1.5 2003/07/03 08:49:49 ddorigo Exp $
 * @author Tim Canavan
 *
 */


public class TemplateSignatureSummary implements Serializable {

    /**
     * Version number for serialisation.  Update after significant
     * changes to the class but <em>beware</em> changing the
     * version number will break deserialization of the previous
     * version!
     */
    static final long serialVersionUID = -6802629687274226374L;


    ////////////////////////////////////////////////////////////////
    // Constants                                                   /
    ////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////
    // Variables                                                   /
    ////////////////////////////////////////////////////////////////

     /**
      * Data from  PAF.NAME in .tsf file
      */
     private String templateName_;

     /**
      * Data from PAF.DESC in .tsf file
      */
     private String templateDesc_;

     /**
      * Data from TPL.TYPE in .tsf file
      */
     private String templateType_;

     /**
      * Data from TPL.MODE in .tsf file
      */
     private String templateMode_;



    ////////////////////////////////////////////////////////////////
    // Methods                                                     /
    ////////////////////////////////////////////////////////////////

     /**
      * Data from PAF.NAME in .tsf file
      *
      * @return templateName_  Name of template e.g. EMMI
      */
      
     public String getTemplateName() {return templateName_;}
     
     /**
      * Date from PAF.DESC in .tsf file
      *
      * @return templateDesc_ Description of Template
      */
         
     public String getTemplateDesc() {return templateDesc_;}
     
     /**
      * Data from TPL.TYPE in .tsf file
      * 
      */     
      
     public String getTemplateType() {return templateType_;}
     
     /**
      * Data from TPL.MODE in .tsf file
      *
      * @return  templateType__ e,g Template Type = RILD
      * 
      */     
               
     public String getTemplateMode() {return templateMode_;}

     /**
      * Initialize template name from PAF.NAME in .tsf file
      *
      * @return templateName_ name of instrument e.g. EMMI
      */     

     public void setTemplateName(String templateName) {templateName_ = templateName;}
     
     /**
      * Initialize template description from PAF.DESC in .tsf file
      * 
      * @param templateDesc description of instrument 
      */     
     
     public void setTemplateDesc(String templateDesc) {templateDesc_ = templateDesc;}
     
     /**
      * Initialize template type  from TPL.TYPE in .tsf file
      */     
     
     public void setTemplateType(String templateType) {templateType_ = templateType;}
     
     /**
      * Initialize template mode from TPL.MODE in .tsf file
      *
      * @param templateMode template Mode e.g. RILD
      */     
     
     public void setTemplateMode(String templateMode) {templateMode_ = templateMode;}

   
     public String toString() {
         StringBuffer sb = new StringBuffer();
         sb.append("Class TemplateSignatureSummary");
         sb.append("\n");
         sb.append("Template name=");
         sb.append(templateName_);
         sb.append("\n");
         sb.append("Template Type=");
         sb.append(templateType_);
         sb.append("\n");
         sb.append("Template Mode=");
         sb.append(templateMode_);
         sb.append("\n");
         sb.append("Template Descriprion=");
         sb.append(templateDesc_);
         sb.append("\n");
         return sb.toString();           
     }
}
