/* $Id: DbaseHandlerCycle.java,v 1.15 2005/08/18 14:12:27 gdonino Exp $ */

package org.eso.ohs.persistence.dbase.phase1;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;
import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.Cycle;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;

/**
 * Sql handler to retrieve, to get and to update Cycle objects on the database.
 * <br>
 * 
 * @see Cycle
 * @author Carlo Boarotto, ESO 21-Mar-2000
 */
public class DbaseHandlerCycle extends DbaseIOHandler {

    public static int PROPOSAL_SUBMISSION_PHASE = 1;

    private final static int MAX_PHASE1_CYCLES = 100000;

    /**
     * Managed class
     * 
     * @return Cycle class
     */
    public Class getManagedClass() {
        return Cycle.class;
    }

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(DbaseHandlerCycle.class);

    //Internal properties
    private Hashtable cycleProperties_ = null;

    private Hashtable baselines_ = null;

    private Hashtable lambdacouples_ = null;

    private Hashtable telescopesInstruments_ = null;

    // cycle properties from the cycles table
    public static final String CYCLE_LABEL = "label";

    public static final String CYCLE_ID = "id";

    public static final String PERIOD = "period";

    public static final String DESCRIPTION = "description";

    public static final String STEP_ID = "step_id";

    //cycles properties from the cycle_prop table
    public static final String CYCLE_PROP_ID = "cycle_id";

    public static final String CYCLE_PROP_KEY = "cycle_key";

    public static final String CYCLE_PROP_VALUES = "cycle_values";

    public static final String COUNTRIES = "countries";

    public static final String RUN_CODES = "runCodeValues";

    public static final String ALT_RUN_CODES = "altRunCodeValues";

    public static final String PERIOD_LIST = "periodList";

    public static final String OBS_MODE_VALUES = "obsModeValues";

    public static final String MOON_LIST = "moonList";

    public static final String SEEING_VALUES = "seeingValues";

    public static final String SKY_TRANSPARENCY_VALUES = "skyTransparencyValues";

    public static final String SUB_CATEGORY_CODES = "subCategoryCodes";

    public static final String EQUINOX = "equinox";

    public static final String VALID_MONTHS = "validMonths";

    public static final String SPECIAL_INSTRUMENTS = "specialInstruments";

    public static final String INTERF_INSTRUMENTS = "interf_instruments";

    public static final String BASELINES_ROOTNAME = "baselines.";

    public static final String LAMBDACOUPLES_ROOTNAME = "lambdacouples.";

    //Properties for the instrument/telescopes
    public static final String OFFERED_ID = "cycle_id";

    public static final String OFFERED_INSTRUMENT = "instrument_code";

    public static final String OFFERED_TELESCOPE = "telescope_code";

    public static final String INSTRUMENTS_CODE = "code";

    public static final String INSTRUMENTS_TEL = "tel";

    public static final String INSTRUMENTS_ID = "id";

    public static final String TELESCOPES_CODE = "code";

    public static final String TELESCOPES_ITEM_NAME = "item_name";
    
    public static final String PROPRIETARY_TIME = "proprietaryTimes";
    
    public static final String MOON_REQUIREMENTS = "tableMoonRequirements";
    
    public static final String TRANSPARENCY_REQUIREMENTS = "tableTransparencyRequirements";
    
    public static final String NATIONS = "tableNations";
    
   

    //private String[] propertyList_ =
    // {COUNTRIES,RUN_CODES,ALT_RUN_CODES,OBS_MODE_VALUES,MOON_LIST,
    //		SEEING_VALUES,SKY_TRANSPARENCY_VALUES,SUB_CATEGORY_CODES,EQUINOX,SPECIAL_INSTRUMENTS,INTERF_INSTRUMENTS};

    //cycles properties from the opc

    /** Sql select statement to retrieve cycle properties from the cycle table */

    protected static String cycle_fields = null;

    protected  static String cycle_select_sql = null;

    /** Sql select statement to retrieve Cycle entries. */
    protected  static String cycle_sql = null;

    /**
     * Sql select statement to retrieve cycle properties from the cycle_prop
     * table
     */

    protected  static String cycle_properties_fields = null;

    protected static String cycle_properties_select_sql = null;

    /** Sql select statement to retrieve Cycle entries. */
    protected static String cycle_properties_sql = null;

    /**
     * Private constructor: it is a singleton class.
     */
    public DbaseHandlerCycle() {
        super();
        cycle_fields = Phase1SelectStmt.cyclesTable
        .get()
        + "."
        + CYCLE_LABEL
        + ","
        + Phase1SelectStmt.cyclesTable.get()
        + "."
        + CYCLE_ID
        + ","
        + Phase1SelectStmt.cyclesTable.get()
        + "."
        + PERIOD
        + ","
        + Phase1SelectStmt.cyclesTable.get()
        + "."
        + DESCRIPTION
        + ","
        + Phase1SelectStmt.cyclesTable.get()
        + "."
        + STEP_ID;
        
        cycle_select_sql = "SELECT  "
            + cycle_fields;
        cycle_sql = cycle_select_sql + " FROM "
        + Phase1SelectStmt.cyclesTable.get();
        cycle_properties_fields = Phase1SelectStmt.cyclePropertiesTable
        .get()
        + "."
        + CYCLE_PROP_ID
        + ","
        + Phase1SelectStmt.cyclePropertiesTable.get()
        + "."
        + CYCLE_PROP_KEY
        + ","
        + Phase1SelectStmt.cyclePropertiesTable.get()
        + "."
        + CYCLE_PROP_VALUES;
        
        cycle_properties_select_sql = "SELECT  "
            + cycle_properties_fields;
        cycle_properties_sql = cycle_properties_select_sql
        + " FROM " + Phase1SelectStmt.cyclePropertiesTable.get();
    }

    /**
     * Read opcReferee data
     * 
     * @param sql
     *            statement
     * @param Object
     *            id to be read
     * @exception SQLException
     *                an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *                the requested object was not found in the database
     * @exception ObjectIOException
     *                IO Eror occured while reading from the database
     */

    public UnpackedStorableObject read(Statement stmt, long objId)
            throws ObjectIOException, ObjectNotFoundException, SQLException {

        Cycle cycle = (Cycle) newInstance();
        //long cycleId = Config.getCfg().uniqueToTableId(objId);
        long cycleId = objId;

        // Retrieve cycle properties from table cycle
        String cyclesSql = cycle_sql + " WHERE " + CYCLE_ID + " = " + cycleId;
        ResultSet cyclesSqlRs = getResultSet(stmt, cyclesSql);
        cyclesSqlRs.next();
        cycle.setId(Config.getCfg().tableToUniqueId(objId, Cycle.class));
        cycle.setCycleId( cyclesSqlRs.getInt(CYCLE_ID));
        setCyclesFields(cycle, cyclesSqlRs);
        cyclesSqlRs.close();

        retrieveCycleProperties(stmt, cycleId);
        retrieveBaselines(stmt, cycleId);
        retrieveLambdacouples(stmt, cycleId);
        retrieveTelescopesInstruments(stmt, cycleId);
        setCyclePropertiesFields(cycle);

        UnpackedStorableObject uso = new UnpackedStorableObject(cycle);
        return uso;
    }

    
    /**
     * @throws ObjectNotFoundException
     * @throws SQLException
     *  
     */
    private void retrieveCycleProperties(Statement stmt, long cycleId)
            throws SQLException, ObjectNotFoundException {

        cycleProperties_ = new Hashtable();
        String cyclePropertiesSql = cycle_properties_sql + " WHERE "
                + CYCLE_PROP_ID + " = " + cycleId;
        ResultSet cyclePropRs = getResultSet(stmt, cyclePropertiesSql);
        while (cyclePropRs.next()) {
            String key = cyclePropRs.getString(CYCLE_PROP_KEY);
            String values = cyclePropRs.getString(CYCLE_PROP_VALUES);

            //Don't insert baselines or lambdacouples in the property list
            if (key.indexOf(".") == -1) {
                cycleProperties_.put(key, values);
            }
        }
        cyclePropRs.close();
    }

    /**
     * @throws ObjectNotFoundException
     * @throws SQLException
     *  
     */
    private void retrieveBaselines(Statement stmt, long cycleId)
            throws SQLException, ObjectNotFoundException {

        baselines_ = new Hashtable();
        String cycleBaselinesSql = "SELECT " + CYCLE_PROP_KEY + ","
                + CYCLE_PROP_VALUES + " FROM "
                + Phase1SelectStmt.cyclePropertiesTable.get() + " WHERE "
                + CYCLE_PROP_ID + " = " + cycleId + " AND " + CYCLE_PROP_KEY
                + " like '" + BASELINES_ROOTNAME + "%'";

        stdlog_.info(" Baselines: " + cycleBaselinesSql);
        ResultSet cycleBaselinesRs = getResultSet(stmt, cycleBaselinesSql);
        while (cycleBaselinesRs.next()) {
            String key = cycleBaselinesRs.getString(CYCLE_PROP_KEY);
            String trimmedKey = key.substring(BASELINES_ROOTNAME.length());
            String[] values = TextUtils.stringToStringArray(cycleBaselinesRs
                    .getString(CYCLE_PROP_VALUES), " ", true);
            baselines_.put(trimmedKey, values);
            stdlog_.debug("baseline Key: " + trimmedKey + " gets value: "
                    + values);
        }
        cycleBaselinesRs.close();
    }

    /**
     * @throws ObjectNotFoundException
     * @throws SQLException
     *  
     */
    private void retrieveLambdacouples(Statement stmt, long cycleId)
            throws SQLException, ObjectNotFoundException {

        lambdacouples_ = new Hashtable();
        String cycleLambdacouplesSql = "SELECT " + CYCLE_PROP_KEY + ","
                + CYCLE_PROP_VALUES + " FROM "
                + Phase1SelectStmt.cyclePropertiesTable.get() + " WHERE "
                + CYCLE_PROP_ID + " = " + cycleId + " AND " + CYCLE_PROP_KEY
                + " like '" + LAMBDACOUPLES_ROOTNAME + "%'";

        ResultSet cycleLambdacouplesRs = getResultSet(
                stmt, cycleLambdacouplesSql);
        while (cycleLambdacouplesRs.next()) {
            String key = cycleLambdacouplesRs.getString(CYCLE_PROP_KEY);
            String trimmedKey = key.substring(LAMBDACOUPLES_ROOTNAME.length());
            String[] values = TextUtils.stringToStringArray(
                    cycleLambdacouplesRs.getString(CYCLE_PROP_VALUES), " ",
                    true);
            lambdacouples_.put(trimmedKey, values);
            stdlog_.debug("lambdacouples Key: " + trimmedKey + " gets value: "
                    + values);
        }
        cycleLambdacouplesRs.close();
    }

    private void retrieveTelescopesInstruments(Statement stmt, long cycleId)
            throws SQLException, ObjectNotFoundException {

        //String telescopesTable = Phase1SelectStmt.telescopeTable.get();
        //String instrumentsTable = Phase1SelectStmt.instrumentTable.get();
        String offeredTable = Phase1SelectStmt.offeredTable.get();
        String instrumentColumn = offeredTable + "." + OFFERED_INSTRUMENT;
        String telescopeColumn = offeredTable + "." + OFFERED_TELESCOPE;
     

        telescopesInstruments_ = new Hashtable();
        //	    String instrumentsTelescopesSql = "SELECT " + telescopesTable + "." +
        // TELESCOPES_CODE +","
        //		+ "'instrument' ="+ instrumentsTable + "." + INSTRUMENTS_CODE + "
        // FROM " + telescopesTable +","+
        //		instrumentsTable + "," + offeredTable
        //		+ " WHERE "
        //		+ offeredTable +"." + OFFERED_ID +" = " + cycleId
        //		+ " AND " + instrumentsTable + "."+ INSTRUMENTS_ID + "= " +
        // offeredTable + "."+ OFFERED_INSTRUMENT
        //		+ " AND " + instrumentsTable + "."+ INSTRUMENTS_TEL + " = " +
        // telescopesTable + "." + TELESCOPES_CODE;
        //		//+ " ORDER BY " + telescopesTable + "." + TELESCOPES_CODE;

        String instrumentsTelescopesSql = "SELECT " + instrumentColumn + ","
                + telescopeColumn + " FROM " + offeredTable + " WHERE "
                + offeredTable + "." + OFFERED_ID + " = " + cycleId;
        //+ " ORDER BY " + telescopeColumn;

        stdlog_.info("Telescopes: " + instrumentsTelescopesSql);

        ResultSet instTelescopesRs = getResultSet(
                stmt, instrumentsTelescopesSql);
        while (instTelescopesRs.next()) {
        	String[] instrumentArray = {};
            String telescope = instTelescopesRs
                    .getString(OFFERED_TELESCOPE)
                    .trim();
            String instrument = instTelescopesRs
                    .getString(OFFERED_INSTRUMENT)
                    .trim();
            stdlog_.debug("Telescope in the vector: " + telescope);
            stdlog_.debug("Inserting the instruments in the vector: "
                    + instrument);
            if (telescopesInstruments_.containsKey(telescope)) {
                instrumentArray = (String[]) telescopesInstruments_
                        .get(telescope);               
            }
            String[] arrayInst = {instrument};
            instrumentArray = 
                TextUtils.concatenateStringArrays(instrumentArray,arrayInst);
            telescopesInstruments_.put(telescope, instrumentArray);
        }

        instTelescopesRs.close();
    }

    
    /**
     * @param valuesCollection
     * @return String[]
     */
    private String[] getArrayFromCollection(Collection valuesCollection) {

        Vector valueVector = new Vector();
        Iterator valuesIterator = valuesCollection.iterator();
        while (valuesIterator.hasNext()) {
            //i--;

            String[] tmpValues = (String[]) valuesIterator.next();
            for (int j = 0; j < tmpValues.length; j++) {

                String stringValue = tmpValues[j];
                valueVector.add(stringValue);

            }
        }

        String[] values = (String[]) valueVector.toArray(new String[valueVector
                .size()]);

        return values;
    }

    /**
     * @param stmt
     * @param cycleSql
     * @return
     * @throws SQLException
     * @throws ObjectNotFoundException
     */
    private ResultSet getResultSet(Statement stmt, String cycleSql)
            throws SQLException, ObjectNotFoundException {
        stdlog_.debug(cycleSql);

        boolean isThereAnObject = stmt.execute(cycleSql);

        if (isThereAnObject) {
            ResultSet rs = stmt.executeQuery(cycleSql);
            return rs;
        } else {
            throw new ObjectNotFoundException("Object not found in database");
        }

    }

    /**
     * Read data
     * 
     * @param sql
     *            statement
     * @param Object
     *            id to be read
     * @exception SQLException
     *                an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *                the requested object was not found in the database
     * @exception ObjectIOException
     *                IO Eror occured while reading from the database
     */

    public UnpackedStorableObject[] read(Statement stmt, String whereClause,
            String otherTables) throws ObjectIOException,
            ObjectNotFoundException, SQLException {

        UnpackedStorableObject[] cycle_array = null;
        Cycle cycle = null;
        if (otherTables == null) {
            otherTables = "";
        }
        String sql = cycle_sql + otherTables + " WHERE " + whereClause;

        stdlog_.info(sql);
        ResultSet rs = stmt.executeQuery(sql);

        try {
            Vector cycle_vector = new Vector();

            while (rs.next()) {

                cycle = (Cycle) newInstance();
                // New element
                cycle_vector.addElement(cycle);

                // Set only when attaching the object as a new
                // element
                setCyclesFields(cycle, rs);
                cycle.setId(Config.getCfg().tableToUniqueId(
                        rs.getInt(CYCLE_ID), Cycle.class));
                cycle.setCycleId(
                        rs.getInt(CYCLE_ID));
            }

            rs.close();

            cycle_array = new UnpackedStorableObject[cycle_vector.size()];
            // add the affiliation to the user and insert them in array
            for (int i = 0; i < cycle_vector.size(); i++) {
                cycle = (Cycle) cycle_vector.elementAt(i);
                long objId = cycle.getCycleId();
                retrieveCycleProperties(stmt, objId);
                retrieveBaselines(stmt, objId);
                retrieveLambdacouples(stmt, objId);
                retrieveTelescopesInstruments(stmt, objId);
                setCyclePropertiesFields(cycle);
                cycle_array[i] = new UnpackedStorableObject(cycle);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return cycle_array;
    }

    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be changed
     * from within P2PP.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @param obj
     *            an array: the first element is the BO to be stored.
     * 
     * @exception ObjectIOException
     *                operation is not permitted exception
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject obj)
            throws ObjectIOException {
        throw new ObjectIOException("no such method implemented");
    }

    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be changed
     * from within P2PP.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @param obj
     *            an array: the first element is the BO to be stored.
     * 
     * @exception ObjectIOException
     *                operation is not permitted exception
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject obj)
            throws ObjectIOException {
    }

    //  Update both rated and comments tables.
    public void update(Statement stmt, StorableObject obj)
            throws ObjectIOException, SQLException {

        Cycle cycle = (Cycle) obj;

        //long id = Config.getCfg().uniqueToTableId(cycle.getId());

        updateCycleFields(stmt, cycle);
        updateCycleProperties(stmt, cycle);
        updateBaselines(stmt, cycle);
        updateLambdacouples(stmt, cycle);
        updateTelescopesInstruments(stmt, cycle);
    }

    /**
     * @param stmt
     * @param cycle
     * @throws SQLException
     */
    private void updateCycleFields(Statement stmt, Cycle cycle)
            throws SQLException {
        StringBuffer sb = new StringBuffer();
        sb.append(CYCLE_LABEL);
        sb.append(" = \'");
        sb.append(cycle.getLabel());
        sb.append("\',");
        sb.append(PERIOD);
        sb.append(" = ");
        sb.append(cycle.getPeriod());
        sb.append(",");
        sb.append(DESCRIPTION);
        sb.append(" = \'");
        sb.append(cycle.getDescription());
        sb.append("\',");
        sb.append(STEP_ID);
        sb.append(" = ");
        sb.append(cycle.getStepId());

        String sql = Phase1SelectStmt.beginTransaction + " \n " + "UPDATE  "
                + Phase1SelectStmt.cyclesTable.get() + " SET " + sb.toString()
                + " WHERE " + CYCLE_ID + " = "
                + Config.getCfg().uniqueToTableId(cycle.getId());

        stdlog_.debug(sql);
        stmt.executeUpdate(sql);
    }

    private void updateCycleProperties(Statement stmt, Cycle cycle)
            throws SQLException {

        long cycleId = cycle.getCycleId();

        refreshCyclePropertiesTable(cycle);
        StringBuffer sb = new StringBuffer();
        sb.append(Phase1SelectStmt.beginTransaction);
        sb.append("\n");
        for (Enumeration propertyKeys = cycleProperties_.keys(); propertyKeys
                .hasMoreElements();) {
            //get the key and the value
            String key = (String) propertyKeys.nextElement();
            
            //Don't save these proprierties to the database.
            if (key.equals("tableTransparencyRequirements") || 
				key.equals("tableMoonRequirements") ||
				key.equals("tableNations") ||
				key.equals("instruments")){
				continue;
			}
            
            
            Object tmpObj =  cycleProperties_.get(key);

            String[] values = (String[]) cycleProperties_.get(key);
            String value =  TextUtils.arrayToString(values, " ", true);

            //build the update line
            sb.append("UPDATE ");
            sb.append(Phase1SelectStmt.cyclePropertiesTable.get());
            sb.append(" SET ");
            sb.append(CYCLE_PROP_VALUES);
            sb.append(" = '");
            sb.append(value);
            sb.append("' WHERE ");
            sb.append(CYCLE_PROP_ID);
            sb.append(" = ");
            sb.append(cycleId);
            sb.append(" AND ");
            sb.append(CYCLE_PROP_KEY);
            sb.append(" = '");
            sb.append(key);
            sb.append("'");
            sb.append("\n");

        }

        String sql = sb.toString();
        stdlog_.debug(sql);
        stmt.executeUpdate(sql);

    }

    private void refreshCyclePropertiesTable(Cycle cycle) {

        cycleProperties_.put(COUNTRIES, cycle.getCountries());
        cycleProperties_.put(RUN_CODES, cycle.getRuncodeValues());
        cycleProperties_.put(ALT_RUN_CODES, cycle.getAltcodeValues());
        cycleProperties_.put(OBS_MODE_VALUES, cycle.getObsModeValues());
        cycleProperties_.put(PERIOD_LIST, cycle.getPeriodListString());
        cycleProperties_.put(MOON_LIST, cycle.getMoonList());
        cycleProperties_.put(SEEING_VALUES, cycle.getSeeingValues());
        cycleProperties_.put(SKY_TRANSPARENCY_VALUES, cycle
                .getSkyTransparencyValues());
        cycleProperties_.put(SUB_CATEGORY_CODES, cycle.getSubCategoryCodes());
        cycleProperties_.put(EQUINOX, cycle.getEquinox());
        cycleProperties_.put(VALID_MONTHS, cycle.getValidMonths());
        cycleProperties_
                .put(SPECIAL_INSTRUMENTS, cycle.getSpecialInstruments());
        cycleProperties_.put(INTERF_INSTRUMENTS, cycle
                .getInterferometricInstruments());
        cycleProperties_.put(PROPRIETARY_TIME, cycle.getProprietaryTimes());
        //cycleProperties_.put(INSTRUMENTS, cycle.getInstruments());
        
        
    }

    /**
     * @param stmt
     * @param cycle
     * @throws SQLException
     */
    private void updateBaselines(Statement stmt, Cycle cycle)
            throws SQLException {

        //refresh the baselines table
        baselines_ = cycle.getBaselinesTable();
        int cycleId = cycle.getCycleId();

        StringBuffer sb = new StringBuffer();
        sb.append(Phase1SelectStmt.beginTransaction);
        sb.append("\n");

        //First delete all the baselines
        sb.append("DELETE FROM ");
        sb.append(Phase1SelectStmt.cyclePropertiesTable.get());
        sb.append(" WHERE ");
        sb.append(CYCLE_PROP_ID);
        sb.append(" = ");
        sb.append(cycleId);
        sb.append(" AND ");
        sb.append(CYCLE_PROP_KEY);
        sb.append(" like '");
        sb.append(BASELINES_ROOTNAME);
        sb.append("%' \n");

        Enumeration baselinesEnum = baselines_.keys();

        StringBuffer baselinesQuery = buildUpdateQuery(
                cycleId, baselinesEnum, baselines_, BASELINES_ROOTNAME);
        String sql = sb.toString() + baselinesQuery.toString();
        stdlog_.debug(sql);
        stmt.executeUpdate(sql);
    }

    /**
     * @param stmt
     * @param cycle
     * @throws SQLException
     */
    private void updateLambdacouples(Statement stmt, Cycle cycle)
            throws SQLException {

        //refresh the baselines table
        lambdacouples_ = cycle.getLambdacouplesTable();
        int cycleId = cycle.getCycleId();

        StringBuffer sb = new StringBuffer();
        sb.append(Phase1SelectStmt.beginTransaction);
        sb.append("\n");

        //First delete all the baselines
        sb.append("DELETE FROM ");
        sb.append(Phase1SelectStmt.cyclePropertiesTable.get());
        sb.append(" WHERE ");
        sb.append(CYCLE_PROP_ID);
        sb.append(" = ");
        sb.append(cycleId);
        sb.append(" AND ");
        sb.append(CYCLE_PROP_KEY);
        sb.append(" like '");
        sb.append(LAMBDACOUPLES_ROOTNAME);
        sb.append("%' \n");

        Enumeration lambdacouplesEnum = lambdacouples_.keys();
        StringBuffer lambdacouplesQuery = buildUpdateQuery(
                cycleId, lambdacouplesEnum, lambdacouples_,
                LAMBDACOUPLES_ROOTNAME);
        String sql = sb.toString() + lambdacouplesQuery.toString();
        stdlog_.debug(sql);
        stmt.executeUpdate(sql);
    }

    /**
     * @param stmt
     * @param cycle
     * @throws SQLException
     */
    private void updateTelescopesInstruments(Statement stmt, Cycle cycle)
            throws SQLException {

        //		refresh the offered table
        telescopesInstruments_ = cycle.getTelescopesInstrumentsTable();
        int cycleId = cycle.getCycleId();

        StringBuffer sb = new StringBuffer();
        sb.append(Phase1SelectStmt.beginTransaction);
        sb.append("\n");

        //First delete all the baselines
        sb.append("DELETE FROM ");
        sb.append(Phase1SelectStmt.offeredTable.get());
        sb.append(" WHERE ");
        sb.append(OFFERED_ID);
        sb.append(" = ");
        sb.append(cycleId);
        sb.append(" \n");

        Enumeration telInstEnum = telescopesInstruments_.keys();
        StringBuffer telInstQuery = buildUpdateTelInstQuery(
                cycleId, telInstEnum, telescopesInstruments_);
        String sql = sb.toString() + telInstQuery.toString();
        stdlog_.debug(sql);
        stmt.executeUpdate(sql);

    }

    /**
     * @param cycleId
     * @param sb
     */
    private StringBuffer buildUpdateQuery(int cycleId, Enumeration enum,
            Hashtable valuesTable, String rootname) {

        StringBuffer query = new StringBuffer();
        for (Enumeration propertyKeys = enum; propertyKeys.hasMoreElements();) {
            //get the key and the value
            String key = (String) propertyKeys.nextElement();

            Object tmpObj = valuesTable.get(key);
            String[] values = (String[]) tmpObj;
            String value = TextUtils.arrayToString(values, " ", true);

            //build the insert line
            query.append("INSERT INTO ");
            query.append(Phase1SelectStmt.cyclePropertiesTable.get());
            query.append("(");
            query.append(CYCLE_PROP_ID);
            query.append(",");
            query.append(CYCLE_PROP_KEY);
            query.append(",");
            query.append(CYCLE_PROP_VALUES);
            query.append(")");
            query.append(" VALUES(");
            query.append(cycleId);
            query.append(",'");
            query.append(rootname);
            query.append(key);
            query.append("','");
            query.append(value);
            query.append("') \n ");
        }

        return query;
    }

    /**
     * @param cycleId
     * @param enum
     * @param valuesTable
     * @return StringBuffer
     */
    private StringBuffer buildUpdateTelInstQuery(int cycleId, Enumeration enum,
            Hashtable valuesTable) {

        StringBuffer query = new StringBuffer();
        for (Enumeration propertyKeys = enum; propertyKeys.hasMoreElements();) {
            //get the key and the value
            String key = (String) propertyKeys.nextElement();

            Object tmpObj = valuesTable.get(key);
            String[] values = (String[]) tmpObj;
            //String value = TextUtils.arrayToString(values, " ", true);

            for (int i = 0; i < values.length; i++) {
                String instrument = values[i];
                //build the insert line
                query.append("INSERT INTO ");
                query.append(Phase1SelectStmt.offeredTable.get());
                query.append("(");
                query.append(OFFERED_ID);
                query.append(",");
                query.append(OFFERED_TELESCOPE);
                query.append(",");
                query.append(OFFERED_INSTRUMENT);
                query.append(")");
                query.append(" VALUES(");
                query.append(cycleId);
                query.append(",'");
                query.append(key);
                query.append("','");
                query.append(instrument);
                query.append("') \n ");
            }

        }

        return query;
    }

    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be changed
     * from within P2PP.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * 
     * @exception ObjectIOException
     *                operation is not permitted exception
     */
    public void delete(Statement stmt, long id) throws ObjectIOException {

    }

    /**
     * Set the user fields from the result set. Only for avoiding duplicated
     * code
     *  
     */

    protected void setCyclesFields(Cycle cycle, ResultSet rs)
            throws SQLException {

        cycle.setLabel(rs.getString(CYCLE_LABEL));
        cycle.setPeriod(rs.getInt(PERIOD));
        cycle.setDescription(rs.getString(DESCRIPTION));
        cycle.setStepId(rs.getInt(STEP_ID));
    }

    /**
     * Set the user fields from the result set. Only for avoiding duplicated
     * code
     *  
     */

    protected void setCyclePropertiesFields(Cycle cycle) throws SQLException {

        cycle.setCountries(TextUtils.stringToStringArray(
                (String) cycleProperties_.get(COUNTRIES), " ", true));
        cycle.setRuncodeValues(TextUtils.stringToStringArray(
                (String) cycleProperties_.get(RUN_CODES), " ", true));
        cycle.setAltcodeValues(TextUtils.stringToStringArray(
                (String) cycleProperties_.get(ALT_RUN_CODES), " ", true));
        cycle.setObsModeValues(TextUtils.stringToStringArray(
                (String) cycleProperties_.get(OBS_MODE_VALUES), " ", true));
        cycle.setPeriodList(TextUtils.stringToIntObjArray(
                (String) cycleProperties_.get(PERIOD_LIST), " "));
        cycle.setMoonList(TextUtils.stringToStringArray(
                (String) cycleProperties_.get(MOON_LIST), " ", true));
        cycle.setSeeingValues(TextUtils.stringToStringArray(
                (String) cycleProperties_.get(SEEING_VALUES), " ", true));
        cycle.setSkyTransparencyValues(TextUtils.stringToStringArray(
                (String) cycleProperties_.get(SKY_TRANSPARENCY_VALUES), " ",
                true));
        cycle.setSubCategoryCodes(TextUtils.stringToStringArray(
                (String) cycleProperties_.get(SUB_CATEGORY_CODES), " ", true));
        cycle.setEquinox(TextUtils.stringToStringArray(
                (String) cycleProperties_.get(EQUINOX), " ", true));
        cycle.setValidMonths(TextUtils.stringToStringArray(
                (String) cycleProperties_.get(VALID_MONTHS), " ", true));
        cycle.setSpecialInstruments(TextUtils.stringToStringArray(
                (String) cycleProperties_.get(SPECIAL_INSTRUMENTS), " ", true));
        cycle.setInterferometricInstruments(TextUtils.stringToStringArray(
                (String) cycleProperties_.get(INTERF_INSTRUMENTS), " ", true));
        cycle.setBaselinesTable(baselines_);
        cycle.setLambdaCouplesTable(lambdacouples_);
        cycle.setTelescopesInstrumentsTable(telescopesInstruments_);
        
        cycle.setProprietaryTimes((TextUtils.stringToStringArray(
        		(String) cycleProperties_.get(PROPRIETARY_TIME), " ", true)));
        
        cycle.setMoonRequirements((String) cycleProperties_.get(MOON_REQUIREMENTS));
        cycle.setTransparencyRequirements((String) cycleProperties_.get(TRANSPARENCY_REQUIREMENTS));
        cycle.setNations((String) cycleProperties_.get(NATIONS));
        
        //this method requires that both InterferometricInstruments and baselines are set
        //if (cycle.getPeriod())
        cycle.setInterfBaselines();
    }

}