/* $Id: DbaseHandlerOpcPanel.java,v 1.1 2005/03/30 09:56:11 tcanavan Exp $ */

package org.eso.ohs.persistence.dbase.phase1;

import java.math.BigInteger;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.OpcPanel;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;



/**
 * Sql handler to retrieve, to get and to update OpcPanel objects
 * on the database.  
 * <br>
 *
 * @see OpcPanel
 * @author Carlo Boarotto, ESO 21-Mar-2000 
 */
public class DbaseHandlerOpcPanel  extends DbaseIOHandler {
 
  public static final int RADIX = 10;
  public static final int SKIP_PANEL_FIELD = 100;
    private Map panelMapping_;
    private Map	intMapping_ ;
    /**
     * Managed class
     * @return OpcPanel class
     */
    public Class getManagedClass() {
	return OpcPanel.class;
    }

   /** Standard log channel */ 
    private static Logger stdlog_ = 
        Logger.getLogger( DbaseHandlerOpcPanel.class );

    /** A reference to the singleton OpcPanelSql instance. */
    private static DbaseHandlerOpcPanel singleton_ = new DbaseHandlerOpcPanel();


    public static final String PANEL =  "panel"; // from panels table
    public static final String CYCLE_ID = "cycle_id";
    public static final String OPC_SUBPANEL = "opc_subpanel";
  
    /** Sql select statement to retrieve OpcPanel entries. */
   

    protected final static String panel_fields = 
	Phase1SelectStmt.refereesTable.get()+"."+ PANEL + "," +
	Phase1SelectStmt.refereesTable.get()+"."+ CYCLE_ID + "," +
	Phase1SelectStmt.refereesTable.get()+"."+ OPC_SUBPANEL;
    
    protected final static String panel_select_sql =  "SELECT DISTINCT " 
	+ panel_fields;
 

	
   /** Sql select statement to retrieve OpcPanel entries. */
    protected final static String panel_sql = panel_select_sql +
	" FROM " + Phase1SelectStmt.refereesTable.get();
    
    /** 
     * Private constructor: it is a singleton class.
     */
    public DbaseHandlerOpcPanel () {
	super();
	panelMapping_ = new HashMap();
	panelMapping_.put(new Integer(1),"A"); 
	panelMapping_.put(new Integer(2),"B"); 
	panelMapping_.put(new Integer(3),"C"); 
	panelMapping_.put(new Integer(4),"D");
	panelMapping_.put(new Integer(5),"E");
	panelMapping_.put(new Integer(6),"F");
	panelMapping_.put(new Integer(7),"G");
	panelMapping_.put(new Integer(8),"H");
	intMapping_ = new HashMap();
	intMapping_.put("A",new Integer(1)); 
	intMapping_.put("B",new Integer(2)); 
	intMapping_.put("C",new Integer(3)); 
	intMapping_.put("D",new Integer(4));
	intMapping_.put("E",new Integer(5));
	intMapping_.put("F",new Integer(6));
	intMapping_.put("G",new Integer(7));
	intMapping_.put("H",new Integer(8));
    }
    
    /**
     * Read  opcReferee data
     * @param sql statement
     * @param Object id to be read
     * @exception SQLException
     *		    an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *		    the requested object was not found in the database   
     * @exception ObjectIOException IO Eror occured while reading from the database
     */
    
    public UnpackedStorableObject read(Statement stmt,long objId)
	throws ObjectIOException, ObjectNotFoundException,
	       SQLException {
	
 	OpcPanel panel = (OpcPanel)newInstance();
	
	String panelChar = panelFromId(objId);
	int subpanel = subpanelFromId(objId);
	
	String sql = panel_sql + " WHERE "+ PANEL + " = '" + panelChar+ "'"+
	    " AND "+ OPC_SUBPANEL + " = " + subpanel; 
	
	//	stdlog_.debug(" Sql: " + sql);
        ResultSet rs = stmt.executeQuery(sql);
	
	if ( ! rs.next()) { // Only one with this id !!
	    throw new ObjectNotFoundException("Object not found in database");
	}
	
	setOpcPanelFields(panel, rs);
	rs.close();
	
	UnpackedStorableObject uso = new UnpackedStorableObject(panel);
	return uso;
    }


   /**
     * Read  data
     * @param sql statement
     * @param Object id to be read
     * @exception SQLException
     *		    an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *		    the requested object was not found in the database   
     * @exception ObjectIOException IO Eror occured while reading from the database
     */


    public UnpackedStorableObject[] read(Statement stmt,String whereClause, String otherTables)
	throws ObjectIOException, ObjectNotFoundException, SQLException {
	    
	    
	    UnpackedStorableObject[] panel_array = null;
	    OpcPanel panel = null;
	    if (otherTables == null) {
		otherTables = "";
	    }
	String sql = panel_sql  + otherTables + " WHERE "+ whereClause;
	stdlog_.debug("read with whereClause SELECT: "+ sql);

	ResultSet rs = stmt.executeQuery(sql);

	try {
	    Vector panel_vector = new Vector();      
	
  	    while (rs.next()) {

		panel = (OpcPanel)newInstance();
		    // New element
		    panel_vector.addElement(panel);

		    // Set only when attaching the object as a new
		    // element
		    setOpcPanelFields(panel, rs);
 stdlog_.debug(panel.getPanel()+" panel data  "+panel.getOpcSubpanel());
		
		    long panelId = convertPanelDataToId(panel.getPanel(), panel.getOpcSubpanel());
		    stdlog_.debug(Config.getCfg().tableToUniqueId(
								  panelId, OpcPanel.class)+" panel "+panelId);
		    panel.setId(Config.getCfg().tableToUniqueId(
					   panelId, OpcPanel.class));
	    }
	    
	    rs.close();
	    panel_array =  new UnpackedStorableObject[panel_vector.size()];
	    // add the affiliation to the user and insert them in array
	    for (int i = 0 ; i< panel_vector.size(); i++) {
		panel = (OpcPanel) panel_vector.elementAt(i);
		panel_array[i] = new UnpackedStorableObject(panel);
	    }
	}
	catch (SQLException e) {
	    e.printStackTrace();
	}
	
	return panel_array;
    }

    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be
     * changed from within P2PP.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * 
     * @exception ObjectIOException	operation is not permitted exception
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject obj)
			throws ObjectIOException {
    }


    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be
     * changed from within P2PP.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * 
     * @exception ObjectIOException	operation is not permitted exception
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject obj)
			throws ObjectIOException {

	
    }

    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be
     * changed from within P2PP.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * 
     * @exception ObjectIOException	operation is not permitted exception
     */
    public void delete(Statement stmt, long id)
			throws ObjectIOException {
   
    }


    /**
     * Set the user fields from the result set.
     * Only for avoiding duplicated code
     *
     */ 
    
    protected void setOpcPanelFields (OpcPanel panel ,ResultSet rs) 
	throws SQLException {
	
	panel.setPanel(rs.getString(PANEL));
	panel.setCycleId(rs.getInt(CYCLE_ID));
	panel.setOpcSubpanel(rs.getInt(OPC_SUBPANEL));
    }

     private String panelFromId(long objId) {
	 BigInteger skip =  BigInteger.valueOf(SKIP_PANEL_FIELD);
	 int panelId = BigInteger.valueOf(objId).divide(skip).intValue();
	 String panel = (String)panelMapping_.get(new Integer(panelId));
	 return panel;
     };

    private int subpanelFromId(long objId) {
	 BigInteger skip = BigInteger.valueOf(SKIP_PANEL_FIELD);
	 int subpanel = BigInteger.valueOf(objId).remainder(skip).intValue();
	return subpanel;
    };
    
    private long convertPanelDataToId(String panel , int opcSubpanel) {

	Integer panelId = (Integer)intMapping_.get(panel);
	long objId = -1;

	if (panelId != null) {
	    objId =  panelId.intValue()*SKIP_PANEL_FIELD + opcSubpanel;
	}
	return objId;
    };


}
