package org.eso.ohs.persistence.dbase.phase2;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import javax.swing.event.EventListenerList;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.Mask;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.QueueItem;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.MaskInstrument;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.OBEvent;
import org.eso.ohs.persistence.OBListener;
import org.eso.ohs.persistence.OHSApplication;
/**
 * Doc TBD
 * 
 * @version $Revision: 1.9 $
 * @author ?
 */

public class QueueManager {

    private static final String rcsid =
        "$Id: QueueManager.java,v 1.9 2005/12/12 16:04:04 ddorigo Exp $";

    /**
     * To be used for the OT ONLY. It has a single execution sequence
     */
    
    
    
    public static String online_execution_sequence = "online_execution_sequence";
    
    public static String exec_tmp = "#tmp_exec_sequences";


    protected static String obrepDbName_ =
        Config.getCfg().getDbName(ObservationBlock.getSuffix());

    private static EventListenerList listenerList = new EventListenerList();
    private static Logger stdlog_ = Logger.getLogger(QueueManager.class);

    /** Adds a listener for an OBEvent */
    public static void addOBListener(OBListener l) {
        listenerList.add(OBListener.class, l);
    }

    /** Remove a listener for an OBEvent */
    public static void removeOBListener(OBListener l) {
        listenerList.remove(OBListener.class, l);
    }

    /** Notify all listeners that are registered for a OBEvent */

    protected static void fireObPropertyChange(OBEvent e) {
        Object[] listeners = listenerList.getListenerList();
        OBListener lnr = null;
        for (int i = 0; i < listeners.length; i += 2) {
            lnr = (OBListener) listeners[i + 1];
            lnr.obPropertyChange(e);
        }
    }

    /**
     *Updates the sequence field for a set of QueueItems that belongs to a 
    
     *specific queue
     *@param obsId a set of IDs of OBs to be updated
     *@param queueId specifies the Id of a queue
     *@param tableName specifies the table in which the SQL command 
     *@param statement the sql statement
     *       will be performed
     *@throws an SQLException if an SQL error occurs
     */

    private static void updateOBsSequences(
        Statement stmt,
        long[] obsId,
        long queueId,
        String tableName)
        throws SQLException {
        for (int i = 0; i < obsId.length; i++) {
            if (obsId[i] != -1) {
                //stdlog_.debug("!!!!!!!!! Updating OB Sequence");
                updateOBSequence(stmt, obsId[i], i, queueId, tableName);
            }
        }
    }

    /**
      *Updates the following properties for a set of OBs
      * status, obsComment,obsGrade
      *@param obId  the ID of an OB to be updated
      *@param newStatus the new value for the status parameter
      *@param newObComments the new value for OBcomment parameter 
      *@param newObGrade the new value for the OBGrade parameter
      *@throws an SQLException if an error occurs
      */
    public static void updateOBStatus(long obId, String newStatus)
        throws ObjectIOException {

        Connection con = null;
        Statement stmt = null;
        Config cfg = Config.getCfg();
        SQLException error = null;
        try {
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());

            stmt = con.createStatement();

            StringBuffer sb = new StringBuffer();

            String sqlText = "UPDATE " + obrepDbName_ + "..obs_blocks SET";
            if (newStatus != null) {
                sqlText = sqlText + " status = '" + newStatus + "' ";
            }

            sqlText =
                sqlText
                    + "WHERE "
                    + obrepDbName_
                    + "..obs_blocks.ob_id = "
                    + Config.getCfg().uniqueToTableId(obId);
            stdlog_.info(sqlText);

            stmt.executeUpdate(sqlText);

            ConnectionPool.getPool().releaseConnection(con);
        } catch (SQLException e) {
            e.printStackTrace();
            error = e;
            try {
                con.rollback();
            } catch (SQLException ex) {
                // nothing here...already have an error recorded!
            }
        } finally {
            closeConnection(con, stmt, error);
            long[] obIds = new long[1];
            obIds[0] = obId;
            SQLException logError=null;
            //stdlog_.debug("!!!!!!!!!!!!!! Fireing Event" + newStatus);
            fireObPropertyChange(
                new OBEvent("QueueManager", obIds, "Status", newStatus));
            try {
                OBHistory.addEvent(
                    Config.getCfg().uniqueToTableId(obId),
                    null,
                    OHSApplication.getShortDesc(),
                    new Integer(Config.getCfg().getUserName()).toString(),
                    newStatus);
            } catch (SQLException e) {
                logError = e;
            }
            if(logError != null)
                 throw new ObjectIOException(logError);

        }

    }

    /**
     *Updates the sequence field for a QueueItem that belongs to a 
     *specific queue
     *@param obId specifies the Id of an OB to be updated
     *@param queueId specifies the Id of a queue
     *@param tableName specifies the table in which the SQL command 
     *@param statement the sql statement
     *       will be performed
     *@throws an SQLException if an SQL error occurs
     */
    private static void updateOBSequence(
        Statement stmt,
        long obId,
        int sequenceId,
        long queueId,
        String tableName)
        throws SQLException {
        Config cfg = Config.getCfg();

        StringBuffer sb = new StringBuffer();

        String sqlText =
            "UPDATE "
                + tableName
                + " SET"
                + " sequence = "
                + sequenceId
                + " WHERE ob_id = "
                + cfg.uniqueToTableId(obId)
                + " AND sch_id = "
                + cfg.uniqueToTableId(queueId);

        stdlog_.info(sqlText);
        stmt.addBatch(sqlText);
    }

    /**
     *Updates the sequence field for a set of QueueItems that belongs to a 
     *specific queue
     *@param obsId a set of IDs of OBs to be updated
     *@param queueId specifies the Id of a queue
     *@param tableName specifies the table in which the SQL command 
     *       will be performed
     *@throws an SQLException if an SQL error occurs
     */

    public static void updateExecOBsSequences(
        long[] obIds
        )
        throws ObjectIOException {
        Config cfg = Config.getCfg();
        Connection con = null;
        Statement stmt = null;
        SQLException error = null;
        try {
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());
            stmt = con.createStatement();
            for(int i=0;i<obIds.length;i++) {
                String sqlText =
                    "UPDATE "
                        + obrepDbName_ + ".." +  online_execution_sequence
                        + " SET"
                        + " sequence = "
                        + i
                        + " WHERE ob_id = "
                        + cfg.uniqueToTableId(obIds[i]);
                stmt.executeUpdate(sqlText);                        
                
            }
        } catch (SQLException e) {
            e.printStackTrace();
            error = e;
            try {
                con.rollback();
            } catch (SQLException ex) {
                // nothing here...already have an error recorded!
            }
        } finally {
            closeConnection(con, stmt, error);
        }
    }
    
    
    /**
     * Update the sequence field for the single execution
     * 
     * @param obsId
     * @throws ObjectIOException
     */
    
    
    public static void updateOBsSequences(
        long[] obsId
        )
        throws ObjectIOException {
        Config cfg = Config.getCfg();
        Connection con = null;
        Statement stmt = null;
        SQLException error = null;
        try {
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());
            stmt = con.createStatement();
        } catch (SQLException e) {
            e.printStackTrace();
            error = e;
            try {
                con.rollback();
            } catch (SQLException ex) {
                // nothing here...already have an error recorded!
            }
        } finally {
            closeConnection(con, stmt, error);
        }
    }

    
    
    /**
     *Updates the sequence field for a set of QueueItems that belongs to a 
     *specific queue
     *@param obsId a set of IDs of OBs to be updated
     *@param queueId specifies the Id of a queue
     *@param tableName specifies the table in which the SQL command 
     *       will be performed
     *@throws an SQLException if an SQL error occurs
     */

    public static void updateOBsSequences(
        long[] obsId,
        long queueId,
        String tableName)
        throws ObjectIOException {
        Config cfg = Config.getCfg();
        Connection con = null;
        Statement stmt = null;
        SQLException error = null;
        try {
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());
            stmt = con.createStatement();

            updateOBsSequences(stmt, obsId, queueId, tableName);
            stmt.executeBatch();
        } catch (SQLException e) {
            e.printStackTrace();
            error = e;
            try {
                con.rollback();
            } catch (SQLException ex) {
                // nothing here...already have an error recorded!
            }
        } finally {
            closeConnection(con, stmt, error);
        }
    }
    
    
    
    

    public static void deleteOBsFromQueueTable(
        long[] OBIDs,
        long queueId,
        String tableName)
        throws ObjectIOException {
        Connection con = null;
        Statement stmt = null;
        SQLException error = null;
        stdlog_.info("Requested to remove: " + OBIDs.length + " OBs");
        try {

            Config cfg = Config.getCfg();
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());

            con.setAutoCommit(false);
            stmt = con.createStatement();
            delete(OBIDs, queueId, tableName, stmt);
            delete(OBIDs, queueId, exec_tmp, stmt);
            con.commit();
        } catch (SQLException e) {
            e.printStackTrace();
            error = e;
            try {
                con.rollback();
            } catch (SQLException ex) {
                // nothing here...already have an error recorded!
            }
        } finally {
            stdlog_.debug("Running Close COnnection");
            closeConnection(con, stmt, error);
        }
    }

    public static void deleteOBsFromTable(
        long[] OBIDs,
        long queueId,
        String tableName)
        throws ObjectIOException {
        Connection con = null;
        Statement stmt = null;
        SQLException error = null;
        stdlog_.info("Requested to remove: " + OBIDs.length + " OBs");
        try {

            Config cfg = Config.getCfg();
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());

            con.setAutoCommit(false);
            stmt = con.createStatement();
            delete(OBIDs, queueId, tableName, stmt);
            con.commit();
        } catch (SQLException e) {
            e.printStackTrace();
            error = e;
            try {
                con.rollback();
            } catch (SQLException ex) {
                // nothing here...already have an error recorded!
            }
        } finally {
            closeConnection(con, stmt, error);
        }
    }

    public static void closeConnection(
        Connection con,
        Statement stmt,
        SQLException error)
        throws ObjectIOException {
        if (stmt != null) {
            try {
                stmt.close();
            } catch (SQLException e) {
                if (e == null) {
                    error = e;
                }
            }
        }
        if (con != null) {
            try {
                 con.setAutoCommit(true);
            } catch (SQLException e) {
                e.printStackTrace();
            }
            ConnectionPool.getPool().releaseConnection(con);
        }
        // throw an exception if there was an error encountered.
        if (error != null) {
            throw new ObjectIOException(error);
        }
    }

    public static void delete(
        long[] OBIDs,
        long queueId,
        String tableName,
        Statement stmt)
        throws SQLException {
        String sqlText =
            " DELETE "
                + obrepDbName_ + ".." + tableName
                + " WHERE sch_id  = "
                + Config.getCfg().uniqueToTableId(queueId)
                + " AND ob_id IN ( ";

        deleteOBs(OBIDs, stmt, sqlText);
    }

    private static void deleteOBs(long[] OBIDs, Statement stmt, String sqlText)
        throws SQLException {
        int i = 0;
        while (i < OBIDs.length) {
            StringBuffer inExpr = new StringBuffer();
            int nextChunk = i + 50;
            if (nextChunk > OBIDs.length) {
                nextChunk = OBIDs.length;
            }
            for (; i < nextChunk; i++) {
                if (inExpr.length() > 0) {
                    inExpr.append(", ");
                }
                inExpr.append(Config.getCfg().uniqueToTableId(OBIDs[i]));
            }
            String sql = sqlText + inExpr.toString() + " )";
            stdlog_.info(sql);
            stmt.executeUpdate(sql);
        }
    }
    
    
    
    
    
    
    public static void deleteOBFromOTExecSequence(long [] OBIDs)
        throws ObjectIOException {
        Connection con = null;
        Statement stmt = null;
        SQLException error = null;
        try {
            String sqlText =
                " DELETE "
                    + obrepDbName_ + ".." + online_execution_sequence
                    + " where ob_id IN ( ";
                                        
                    

            Config cfg = Config.getCfg();
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());

            stmt = con.createStatement();
            deleteOBs(OBIDs, stmt, sqlText);
        } catch (SQLException e) {
            e = error;           
        } finally {
            closeConnection(con, stmt, error);
            fireObPropertyChange(
                new OBEvent("QueueManager",OBIDs, "Status", "Delete"));
            
        }

    }


    public static void deleteOBFromExecSequence(long OBID)
        throws ObjectIOException {
        Connection con = null;
        Statement stmt = null;
        SQLException error = null;
        try {
            String sqlText =
                " DELETE "
                    + exec_tmp
                    + " WHERE sch_id  = 0 "
                    + " AND ob_id IN ( ";

            Config cfg = Config.getCfg();
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());

            stmt = con.createStatement();

            sqlText += Config.getCfg().uniqueToTableId(OBID);
            sqlText += " )";

            stdlog_.info(sqlText);
            stmt.executeUpdate(sqlText);
        } catch (SQLException e) {
            e = error;           
        } finally {
            closeConnection(con, stmt, error);
            long[] obIds = new long[1];
            obIds[0] = OBID;
            fireObPropertyChange(
                new OBEvent("QueueManager", obIds, "Status", "Delete"));

        }

    }

    /**
      *Insert a set of new rows into a schedule_items table:
      *@param OBIDs a set of ID of OBs to be inserted into the table
      *@param queueId specifies the id of the queue
      *@param startSequenceValue specifies the first index of 
      *       schedule item within the schedule
      *@param tableName specifies the table in which the SQL
      *       command will be performed
      *@throws an SQLExcception if an SQL error occurs 
    
      */
    public static void appendOBsToExecSequence(
        long[] OBIDs,
        int startSequenceValue,String hostname)
        throws ObjectIOException {
        Connection con = null;
        Statement stmt = null;
        SQLException error = null;
        try {
            Config cfg = Config.getCfg();
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());
            stmt = con.createStatement();
            StringBuffer sb = new StringBuffer();

            for (int i = 0; i < OBIDs.length; i++) {
                sb.setLength(0);
                sb.append(cfg.uniqueToTableId(OBIDs[i]));
                sb.append(",");
                sb.append(startSequenceValue + i);
                sb.append(",");
                sb.append(SqlUtils.sqlEscapedString(hostname));                
                String sqlText =
                    "INSERT INTO " + obrepDbName_ 
                        + ".." + online_execution_sequence 
                        + " (ob_id, sequence,hostname)"
                        + " VALUES ("
                        + sb
                        + ")";
                stdlog_.info(sqlText);
                stmt.executeUpdate(sqlText);
            }
        } catch (SQLException e) {        
            error = e;
        }
        finally {
            closeConnection(con, stmt, error);
        }
    }
    
    
    /**
      *Insert a set of new rows into a schedule_items table:
      *@param OBIDs a set of ID of OBs to be inserted into the table
      *@param queueId specifies the id of the queue
      *@param startSequenceValue specifies the first index of 
      *       schedule item within the schedule
      *@param tableName specifies the table in which the SQL
      *       command will be performed
      *@throws an SQLExcception if an SQL error occurs 
    
      */
    public static void appendOBsToMaskExecSequence(
        long [] OBIDs,
        long queueId,
        int startSequenceValue)
        throws ObjectIOException {
        Connection con = null;
        Statement stmt = null;
        SQLException error = null;
        try {
            Config cfg = Config.getCfg();
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());
            stmt = con.createStatement();
            StringBuffer sb = new StringBuffer();

            for (int i = 0; i < OBIDs.length; i++) {
                sb.setLength(0);
                sb.append(queueId);
                sb.append(",");                                
                sb.append(cfg.uniqueToTableId(OBIDs[i]));
                sb.append(",");
                sb.append(startSequenceValue + i);
                String sqlText =
                    "INSERT INTO "
                        + exec_tmp 
                        + " (sch_id,ob_id, sequence)"
                        + " VALUES ("
                        + sb
                        + ")";
                stdlog_.info(sqlText);
                stmt.executeUpdate(sqlText);
            }
        } catch (SQLException e) {        
            error = e;
        }
        finally {
            closeConnection(con, stmt, error);
        }
    }


    public static void appendObsToQueue(
        Queue queue,
        long[] obsIds,
        long[] appendIds,
        int startSequenceValue,
        boolean isOutOfOrder)
        throws ObjectIOException {
        Config cfg = Config.getCfg();
        Connection con = null;
        Statement stmt = null;
        SQLException error = null;
        try {
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());
            stmt = con.createStatement();
            if (isOutOfOrder) {
                updateOBsSequences(
                    stmt,
                    obsIds,
                    queue.getId(),
                    DbaseHandlerQueue.TMP_SCHEDULE_ITEMS);
            }
            StringBuffer sb = new StringBuffer();

            for (int i = 0; i < appendIds.length; i++) {
                sb.setLength(0);
                sb.append(cfg.uniqueToTableId(queue.getId()));
                sb.append(",");
                sb.append(cfg.uniqueToTableId(appendIds[i]));
                sb.append(",");
                sb.append(startSequenceValue + i);
                sb.append(",");
                sb.append(0);
                sb.append(",");
                sb.append(0);
                sb.append(",");
//TODO How should this be handled properly.                
                sb.append("\'\'");
                String sqlText =
                    "INSERT INTO "
                        + DbaseHandlerQueue.TMP_SCHEDULE_ITEMS
                        + " (sch_id, ob_id, sequence, start_time,end_time,rank)"
                        + " VALUES ("
                        + sb
                        + ")";
                stdlog_.info(sqlText);
                stmt.executeUpdate(sqlText);
            }
        } catch (SQLException e) {
            error = e;
            
        }
        finally {
            closeConnection(con, stmt, error);
        }
    }

    /**
    *Deletes a set of OBs from the DB using the stored procedure
    *@param obsId a set of ob's id to be removed
    *@throws an SQLExcception if an SQL error occurs
    */

    public static void removeOBs(long[] obsId) throws ObjectIOException {
        Connection con = null;
        Statement stmt = null;
        SQLException error = null;
        try {
            Config cfg = Config.getCfg();
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());

            stmt = con.createStatement();

            for (int i = 0; i < obsId.length; i++) {
                removeOB(obsId[i], stmt);
            }
        } catch (SQLException e) {
            error = e;
        }
        finally {
            closeConnection(con, stmt, error);
        }
    }

    /**
    *Deletes an  OB from the DB using the stored procedure
    *@param obsId a set of ob's id to be removed
    *@throws an SQLExcception if an SQL error occurs
    */

    private static void removeOB(long obId, Statement stmt)
        throws SQLException {
        String sqlText =
            obrepDbName_
                + "..proc_uss_delete_ob "
                + Config.getCfg().uniqueToTableId(obId);

        stdlog_.info(sqlText);
        stmt.executeUpdate(sqlText);
    }

    /**
     *Deletes a set of OBs from all queues which reference them 
     *@param obsId a set of obsId to be  removed 
     * (id are in the ObjetManager format
     *@throws an SQLException if an SQL error occurs
     */

    public static void removeOBsFromAllQueues(long[] obsId)
        throws ObjectIOException {
        Connection con = null;
        Statement stmt = null;
        SQLException error = null;
        try {

            Config cfg = Config.getCfg();
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());
            stmt = con.createStatement();
            con.setAutoCommit(false);
            deleteFromAll(obsId, stmt, "schedule_items");
            deleteFromAll(obsId, stmt, "exec_sequences");
            con.commit();

        } catch (SQLException e) {
            e.printStackTrace();
            error = e;
            try {
                con.rollback();
            } catch (SQLException ex) {
                // nothing here...already have an error recorded!
            }
        } finally {
            closeConnection(con, stmt, error);
        }
    }

    public static void deleteFromAll(
        long[] obsId,
        Statement stmt,
        String tableName)
        throws SQLException {
        String sqlText =
            "DELETE FROM "
                + obrepDbName_
                + ".." + tableName + " WHERE ob_id  in ( ";
        for (int i = 0; i < obsId.length - 1; i++) {
            sqlText =
                sqlText + Config.getCfg().uniqueToTableId(obsId[i]) + " , ";
        }
        sqlText =
            sqlText
                + Config.getCfg().uniqueToTableId(obsId[obsId.length - 1])
                + " ) ";

        stdlog_.info(sqlText);
        stmt.executeUpdate(sqlText);
    }

    /**
       *Returns a string  that contains the following information
       *:obsBlockId, QueueId and QueueName for each queue that 
       *reference the specified observation block 
       *@param a set of obsId  (the ObjectManager ID format)
       *@throws an SQLExcception if an SQL error occurs
       */
    public static String reportReferencingQueues(long[] obsId)
        throws ObjectIOException {

        Statement stmt = null;
        Connection con = null;
        SQLException error = null;
        try {
            Config cfg = Config.getCfg();
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());
            stmt = con.createStatement();

            String text =
                "obsBlock  \t ProgrammeId \t\t QueueId \t QueueName"
                    + "\n-----------------------------------------------------"
                    + "----------------\n";

            Vector v = new Vector();

            for (int i = 0; i < obsId.length; i++) {
                text
                    += reportReferencingQueues(
                        obsId[i],
                        obrepDbName_ + ".." + DbaseHandlerQueue.SCHEDULE_ITEMS,
                        stmt,
                        v);
                //	  text += reportReferencingQueues(obsId[i],DbaseHandlerQueue.TMP_SCHEDULE_ITEMS,stmt,v);
            }
            return text;
        } catch (SQLException e) {
            error  =e;     
        }
        finally {
            closeConnection(con, stmt, error);
        }
        return "";

    }

    /**
     *Returns a string  that contains the following information
     *:obsBlockId, QueueId and QueueName for each queue that 
     *reference the specified observation block 
     *@param the obId  (the ObjectManager ID )
     *@throws an SQLExcception if an SQL error occurs
     */

    private static String reportReferencingQueues(
        long obId,
        String tableName,
        Statement stmt,
        Vector v)
        throws SQLException {
        ResultSet rs = null;
        String phase1db = Config.getCfg().getDbName(ObservingRun.getSuffix());
        String sqlText =
            "SELECT DISTINCT "
                + tableName
                + ".ob_id, "
                + phase1db
                + ".dbo.sched_rep.progid, "
                + tableName
                + ".sch_id, "
                + obrepDbName_
                + "..schedules.item_name FROM "
                + obrepDbName_
                + "..obs_blocks, "
                + phase1db
                + ".dbo.sched_rep,"
                + tableName
                + " , "
                + obrepDbName_
                + "..schedules "
                + " WHERE "
                + obrepDbName_
                + "..obs_blocks.run_id = "
                + phase1db
                + ".dbo.sched_rep.run_id "
                + " AND "
                + obrepDbName_
                + "..obs_blocks.ob_id = "
                + tableName
                + ".ob_id  "
                + " AND "
                + obrepDbName_
                + "..schedules.sch_id = "
                + tableName
                + ".sch_id "
                + " AND "
                + obrepDbName_
                + "..obs_blocks.ob_id = "
                + Config.getCfg().uniqueToTableId(obId);

        stdlog_.info(sqlText);
        rs = stmt.executeQuery(sqlText);

        String txt = "";

        while (rs.next()) {
            long obID = Config.getCfg().uniqueToTableId(obId);
            String progID = rs.getString("progid");
            long queueID = rs.getInt("sch_id");
            String itemName = rs.getString("item_name");

            if (!contains(v, obID, queueID)) {
                //stdlog_.debug("added " + obID + " " + queueID);
                //    v.addElement( new Long (obID));
                v.addElement(new Long(queueID));

                txt += obID
                    + "\t"
                    + progID
                    + "\t \t"
                    + queueID
                    + "\t"
                    + itemName
                    + "\n";
            }
        }

        return txt;
    }

    static private boolean contains(Vector v, long nObID, long nQueueID) {
        for (int i = 0; i < v.size() - 1; i = i + 2) {
            Long obId = (Long) v.elementAt(i);
            Long queueId = (Long) v.elementAt(i + 1);

            //stdlog_.debug("compare " + obId + " " + queueId + " " + nObID + " " +
            //                    nQueueID);

            if ((obId.longValue() == nObID)
                && (queueId.longValue() == nQueueID)) {
                //stdlog_.debug("true");
                return true;
            }
        }

        //stdlog_.debug("false");
        return false;
    }

    /**
    *Updates the following properties for a set of OBs
    * status, obsComment,obsGrade
    *@param obIds a set of IDs of OBs to be updated
    *@param newStatus the new value for the status parameter
    *@param newObComments the new value for OBcomment parameter 
    *@param newObGrade the new value for the OBGrade parameter
    *@throws an SQLException if an error occurs
    */
    public static void updateOBs(
        long[] obIds,
        String newStatus,
        String newObComments,
        String newObGrade)
        throws ObjectIOException {
        Connection con = null;
        Statement stmt = null;
        SQLException error = null;
        try {

            Config cfg = Config.getCfg();
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());

            con.setAutoCommit(false);
            stmt = con.createStatement();
            for (int i = 0; i < obIds.length; i++) {
                updateOB(obIds[i], newStatus, newObComments, newObGrade, stmt);
            }
            con.commit();
            con.setAutoCommit(true);
        } catch (SQLException e) {
            e.printStackTrace();
            error = e;
            try {
                con.rollback();
            } catch (SQLException ex) {
                // nothing here...already have an error recorded!
            }

        } finally {
            closeConnection(con, stmt, error);
            fireObPropertyChange(
                new OBEvent("QueueManager", obIds, "ALL", newStatus));
            try {
                for(int i=0;i<obIds.length;i++) {
                    OBHistory.addEvent(
                    Config.getCfg().uniqueToTableId(obIds[i]),
                    null,
                    OHSApplication.getShortDesc(),
                    new Integer(Config.getCfg().getUserName()).toString(),                    
                    newStatus);
                }
            } catch (SQLException e) {
                throw new ObjectIOException(e);
            }
        }
    }

    /**
     *Updates the following properties for a set of OBs
     * status, obsComment,obsGrade
     *@param obId  the ID of an OB to be updated
     *@param newStatus the new value for the status parameter
     *@param newObComments the new value for OBcomment parameter 
     *@param newObGrade the new value for the OBGrade parameter
     *@throws an SQLException if an error occurs
     */
    private static void updateOB(
        long obId,
        String newStatus,
        String newObComments,
        String newObGrade,
        Statement stmt)
        throws SQLException {
        StringBuffer sb = new StringBuffer();

        String sqlText = "UPDATE " + obrepDbName_ + "..obs_blocks SET";

        if (newObComments != null) {
            //sqlText = sqlText + " obsComments = '" + TextUtils.escapedString(newObComments) + "' ,";
            sqlText =
                sqlText
                    + " obsComments = "
                    + SqlUtils.sqlEscapedString(newObComments)
                    + " ,";
        }
        if (newObGrade != null) {
            sqlText = sqlText + " obs_grade = '" + newObGrade + "' ,";
        }
        if (newStatus != null) {
            sqlText = sqlText + " status = '" + newStatus + "' ";
        }

        sqlText =
            sqlText + "WHERE ob_id = " + Config.getCfg().uniqueToTableId(obId);
        stdlog_.info(sqlText);

        stmt.executeUpdate(sqlText);
    }

    /**
     *Create the tmp tables
     *Must be improved the check if tables already exsit
     */
    public static void createTmpTables() {
        try {
            stdlog_.debug(
                "Creating... tables "
                    + DbaseHandlerQueue.TMP_SCHEDULE_ITEMS
                    + " "
                    + exec_tmp);
            createExecSequenceTmpTable();
            createScheduleItemsTmpTable();

            stdlog_.debug(
                "tables "
                    + DbaseHandlerQueue.TMP_SCHEDULE_ITEMS
                    + " "
                    + exec_tmp
                    + "created");
        } catch (SQLException e) {

            stdlog_.debug(
                "....tables "
                    + DbaseHandlerQueue.TMP_SCHEDULE_ITEMS
                    + " "
                    + exec_tmp
                    + " already defined into the DB");
            e.printStackTrace();
        }
    }
    
    /**
     *Drop the tmp tables
     */
    public static void dropTmpTables() {
        try {
            stdlog_.debug(
                "Dropping... tables "
                    + DbaseHandlerQueue.TMP_SCHEDULE_ITEMS
                    + " "
                    + exec_tmp);
            dropExecSequenceTmpTable();
            createScheduleItemsTmpTable();

            stdlog_.debug(
                "tables "
                    + DbaseHandlerQueue.TMP_SCHEDULE_ITEMS
                    + " "
                    + exec_tmp
                    + "created");
        } catch (SQLException e) {

            stdlog_.debug(
                "....tables "
                    + DbaseHandlerQueue.TMP_SCHEDULE_ITEMS
                    + " "
                    + exec_tmp
                    + " not defined into the DB");
         //   e.printStackTrace();
        }
    }

    /**
     *Creates the exec_sequence temporary table
     *@throws an SQLExcception if an SQL error occurs
     */
    public static void createExecSequenceTmpTable() throws SQLException {
        String sqlText =
            "create table "
                + exec_tmp
                + " ( "
                + "sch_id      int not null, "
                + "ob_id       int not null, "
                + "sequence    int not null )";
        execSql(sqlText);
    }
    
    /**
     *Drops the exec_sequence temporary table
     *@throws an SQLExcception if an SQL error occurs
     */
    public static void dropExecSequenceTmpTable() throws SQLException {
        String sqlText =
            "drop table "
                + exec_tmp;
        execSql(sqlText);
    }

    /**
     *Creates the schedule_items temporary table
     *@throws an SQLExcception if an SQL error occurs
     */
    public static void createScheduleItemsTmpTable() throws SQLException {
        String sqlText =
            "create table "
                + DbaseHandlerQueue.TMP_SCHEDULE_ITEMS
                + " ( "
                + " sch_id      int not null, "
                + " ob_id       int not null, "
                + " sequence    int not null, "
                + " start_time  int not null, "
                + " end_time    int not null,  "
                + " rank  varchar(255) default \'\' )";

        execSql(sqlText);

    }
    /**
     *Drops the schedule_items temporary table
     *@throws an SQLExcception if an SQL error occurs
     */
    public static void dropScheduleItemsTmpTable() throws SQLException {
        String sqlText =
            "drop table "
                + DbaseHandlerQueue.TMP_SCHEDULE_ITEMS;

        execSql(sqlText);

    }

    /**
     * @param sqlText
     * @throws SQLException void
     */
    private static void execSql(String sqlText) throws SQLException {
        Connection con;
        Statement stmt;
        Config cfg = Config.getCfg();
        con =
            ConnectionPool.getPool().acquireConnection(
                cfg.getUrl(ObservationBlock.getSuffix()),
                cfg.getDbUserName(),
                cfg.getDbPassword());
        stmt = con.createStatement();
        //stdlog_.info(sqlText);  
        stmt.executeUpdate(sqlText);
        ConnectionPool.getPool().releaseConnection(con);
    }

    /**
      *Updates the following properties for a set of OBs
      * status, obsComment,obsGrade
      *@param obId  the ID of an OB to be updated
      *@param newStatus the new value for the status parameter
      *@param newObComments the new value for OBcomment parameter 
      *@param newObGrade the new value for the OBGrade parameter
      *@throws an SQLException if an error occurs
      */
    public static void updateMaskStatus(long maskId, String newStatus)
        throws ObjectIOException {

        Connection con = null;
        Statement stmt = null;
        Config cfg = Config.getCfg();
        SQLException error = null;
        try {
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());

            stmt = con.createStatement();

            MaskInstrument maskInstrument =
                (MaskInstrument) InstrumentList.getInstance().getInstrument(
                    "VIMOS");
            String dummyAdm = maskInstrument.getDummyAdm();
            String sqlText = null;
            if (newStatus.equals(Mask.NO_MASK)) {
                sqlText =
                    "UPDATE "
                        + obrepDbName_
                        + "..masks SET "
                        + " mask_status = '"
                        + newStatus
                        + "' ,"
                        + " adm_file = \'"
                        + dummyAdm
                        + "\' ,"
                        + " mask_slot_num = -99 ,"
                        + " bar_code = \'-99\' "
                        + " WHERE "
                        + obrepDbName_
                        + "..masks.mask_id = "
                        + Config.getCfg().uniqueToTableId(maskId);

            } else if (newStatus.equals(Mask.MASK_MANUFACTURED)) {
                sqlText =
                    "UPDATE "
                        + obrepDbName_
                        + "..masks SET "
                        + " mask_status = '"
                        + newStatus
                        + "' ,"
                        + " mask_slot_num = -99 "
                        + " WHERE "
                        + obrepDbName_
                        + "..masks.mask_id = "
                        + Config.getCfg().uniqueToTableId(maskId);

            } else
                sqlText =
                    "UPDATE "
                        + obrepDbName_
                        + "..masks SET "
                        + " mask_status = '"
                        + newStatus
                        + "' "
                        + " WHERE "
                        + obrepDbName_
                        + "..masks.mask_id = "
                        + Config.getCfg().uniqueToTableId(maskId);

            System.out.println(sqlText);

            stmt.executeUpdate(sqlText);

            //ConnectionPool.getPool().releaseConnection(con);
        } catch (SQLException e) {
            error = e;        
        }
        finally {
            closeConnection(con, stmt, error);
        }
    }

    public static void appendQueueItemsToQueue (Queue queue) throws ObjectIOException {


      //  Queue queue,
     //   long[] obsIds,
      //  long[] appendIds,
      //  int startSequenceValue,
     //   boolean isOutOfOrder
        Config cfg = Config.getCfg();
        Connection con = null;
        Statement stmt = null;
        SQLException error = null;
        try {
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());
            stmt = con.createStatement();
           
            StringBuffer sb = new StringBuffer();
            QueueItem[] queueList = queue.getQueueList();
            for (int i = 0; i < queueList.length; i++) {
                QueueItem currQueueItem = queueList[i];
                sb.setLength(0);
                sb.append(cfg.uniqueToTableId(queue.getId()));
                sb.append(",");
                sb.append(cfg.uniqueToTableId(currQueueItem.getObId()));
                sb.append(",");
                sb.append(i);
                sb.append(",");
                sb.append(0);
                sb.append(",");
                sb.append(0);
                sb.append(",\'");
                sb.append(TextUtils.quoteSqlString(currQueueItem.getRank()));
//TODO How should this be handled properly.                
                sb.append("\'");
                String sqlText =
                    "INSERT INTO "
                        + DbaseHandlerQueue.TMP_SCHEDULE_ITEMS
                        + " (sch_id, ob_id, sequence, start_time,end_time,rank)"
                        + " VALUES ("
                        + sb
                        + ")";
                stdlog_.info(sqlText);
                stmt.executeUpdate(sqlText);
            }
        } catch (SQLException e) {
            error = e;
            
        }
        finally {
            closeConnection(con, stmt, error);
        }
    }
}
