package org.eso.ohs.phase2.actions;

import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.phase2.orang.OBScheduleInfo;
import org.eso.ohs.phase2.orang.ObDataContainer;
import org.eso.ohs.phase2.visibility.VisicalcOptions;
import org.eso.ohs.phase2.visiplot.ChartFrame;
import org.eso.ohs.phase2.visiplot.OBVisiplotInfo;
import org.eso.ohs.phase2.visiplot.VisiPlotOptions;
import org.eso.ohs.phase2.visiplot.VisiplotUtils;


/**
 * This class loads an array of Observation Blocks using the object manager 
 * 
 * @author pnunes 25.10.2005
 *
 */
public class IncrementalOBVisiplot implements IncrementalAction {
	
	private Long[] idLongs_;
	private ObDataContainer [][] groupedObs_;	
	private VisiPlotOptions visiplotOptions_;
	private OBScheduleInfo[] computedOBs_;
	private ChartFrame frame_;
	private int upperLimit_;
	
	
	public IncrementalOBVisiplot(Long[] ids,
			VisiPlotOptions options, OBScheduleInfo[] obs,
			ChartFrame frame) {
		idLongs_ = ids;
		computedOBs_ = obs;
		frame_ = frame;
		visiplotOptions_ = options;
		upperLimit_= ids.length;
	}
	
	
	
	/**
	 * @param  int counter - will be passed to the iteration method
	 * @return number of steps remaining
	 */
	
	public int getNumStepsRemaining(int counter) {
		return ObjUtils.howManyNotNull(idLongs_, counter);
	}
	
	
	public void startOfAction() {
		upperLimit_= groupOBs();
	}
	
	public void endOfAction() {
		frame_.displayPlots();
		frame_.pack();
		frame_.setVisible(true);
	}
	
    /**
     * Groups the OBs by target depending on the visiplots options
     */
    private int groupOBs() {
    	
    	ObDataContainer [] containers = VisiplotUtils.getOBContainers(idLongs_,computedOBs_,visiplotOptions_);
    	    	
    	
    	groupedObs_ =
            VisiplotUtils.groupDuplicates(
            		containers,
                visiplotOptions_.isEnabled(VisiPlotOptions.groupPlotKey));
    	groupedObs_ = VisiplotUtils.sort(groupedObs_, visiplotOptions_);

        return (groupedObs_.length);
    }
	
    public void performStep(int counter) {
        if (counter < upperLimit_) {
            doStep(counter);
        }
    }
    
    
	/**
	 * @param int counter - will be passed to the iteration method
	 */
	public void doStep(int counter) {
		//		 build the chart
		String report[] = VisiplotUtils.getTextReport(groupedObs_[counter], visiplotOptions_);    				    				
		
		// find the correct OBScheduleInfo object
		OBScheduleInfo obInfo = VisiplotUtils.find(groupedObs_[counter][0].getObInfo().getId(), computedOBs_,visiplotOptions_);    				    		
		OBVisiplotInfo visi = new OBVisiplotInfo(obInfo, visiplotOptions_);
		
		frame_.addReport(report);
		frame_.addVisiplot(visi); 
	}
	

	
	

	
	
}