package org.eso.ohs.phase2.actions;

import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.OverwriteOptionPane;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.FindingChart;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSPersistence;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectSelector;

/**
 * @author tcanavan
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class SaveFCAction extends ActionSuperclass {
    
    
    private Media device_;
    
    
    private ObjectContainer folderView_;
    
    
    private boolean allOBs_;
    
    
    private JFrame frm_;
    
    
    private int returnValue_;
    
    
    private ObjectSelector objSelector_;
    
    
    /**
     * The fileChooser_ to export the finding charts.
     */    
    
    private static SwingFileChooser fileChooser_ = new SwingFileChooser();
    static {
        File f1 = new File(AppConfig.getAppConfig().getFcDir());
        fileChooser_.setCurrentDirectory(new File(f1.getParent()));
        fileChooser_.setSelectedFile(f1);
    }

    
    
    
    /**
     * Construct an action for finding charts
     * @param device
     * @param folderView
     * @param name
     * @param allOBs
     */


    public SaveFCAction(
        Media device,
        ObjectContainer folderView,
        String name,
        boolean allOBs) {
        super((JComponent) folderView, name);
        device_ = device;
        folderView_ = folderView;
        allOBs_ = allOBs;
        frm_ = (JFrame)folderView_.getTopLevelAncestor();
    }
    
    public SaveFCAction(
        Media device,
        ObjectSelector objSelector,
        String name
        ) {
        super((JFrame)objSelector, name);
        device_ = device;
        objSelector_ = objSelector;
        frm_ = (JFrame)objSelector;
    }


    /**
     * @see org.eso.ohs.core.gui.baseaction.ActionSuperclass#actionPerformedImpl(java.awt.event.ActionEvent)
     */
    public void actionPerformedImpl(ActionEvent ev) {
        Long[] idLongs;
        if(objSelector_!=null) {
            idLongs = new Long[1];
            idLongs[0] = new Long(objSelector_.getDisplayedObject().getId());            
        } else {
            if (allOBs_)
                idLongs = folderView_.getAll();
            else
                idLongs = folderView_.getAllSelected();         
        }
        idLongs = (Long[]) ObjUtils.uniqueOccurrences(idLongs);
        if (idLongs.length == 0) {
            JOptionPane.showMessageDialog(
                folderView_.getTopLevelAncestor(),
                new Object[] {
                    "There are no OB's Selected",
                    "Please select an OB ",
                    },
                "Nothing selected",
                JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        fileChooser_.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        fileChooser_.setCurrentDirectory(fileChooser_.getCurrentDirectory());
        
        int response =
            fileChooser_.showDialog(null, "Select FindingCharts Directory ");
        if (response == JOptionPane.CLOSED_OPTION
            || response == JOptionPane.CANCEL_OPTION)
            return;
        if(folderView_ != null)
            frm_ = (JFrame)folderView_.getTopLevelAncestor();
        if (response == JFileChooser.APPROVE_OPTION) {
            File f = fileChooser_.getSelectedFile();
            if (!f.canWrite()) {
                JOptionPane.showMessageDialog(
                    null,
                    new Object[] { "No write permissions to write Finding Charts to this directory" },
                    "No write permisssions",
                    JOptionPane.ERROR_MESSAGE);
            } else {
                new IncrementalActionAutomaton(
                    new IncrementalSave(
                        device_,
                        f.getAbsolutePath(),
                        idLongs),
                    frm_,
                    "Saving...");

            }
        }        
    }
    
    /**
     * Utility class to export a sequence of Business Objects, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalSave implements IncrementalAction {


        private Long[] idLongs_;

        private Media device_;
        
        
        private String absolutePath_;


        /**
         * Constructor.
         * 
         * @param device    media containing the objects to be duplicated
         * @param idLongs   array of ids of objects to be duplicated
         */
        public IncrementalSave(Media device,String absolutePath, Long[] idLongs) {
            device_ = device;
            idLongs_ = idLongs;
            absolutePath_ = absolutePath;

        }

        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(idLongs_, counter);
        }

        public void startOfAction() {
            returnValue_ = -1;
        }

        public void endOfAction() {

        }

        /**
         * Method to print finding charts for OB's
         * 
         * @param counter   counter to show which element of obs to check in.
         * @return      true if the duplicating should continue
         *          and false otherwise.
         */
        public void performStep(int counter) {
            if (idLongs_[counter] != null) {
                // Main body of the "loop" + fall thru
                doSave(idLongs_[counter].longValue());
            }
        }

        /**
         * Utility method to duplicate a given business object
         * 
         * @param id        id of business object to duplicate
         */
        public  void doSave(long id) {
            try {
                Class c = Config.getCfg().getClassFromId(id);
                if (c.equals(ObservationBlock.class)) {                    
                    OHSPersistence persistence =
                        (OHSPersistence) OHSPersistence.getObjectManager();
                    FindingChart[] fcs = null;
                    if (device_.equals(Media.DBASE))
                        fcs = persistence.populateFindingCharts(Media.DBASE,id);
                    for(int i=0;i<fcs.length;i++) {
                        String name = new String(absolutePath_ + File.separator + fcs[i].getFcName());
                        name = name.replace(' ', '_');                        
                        File outFile  = new File(name);                        
                        if (outFile.exists()
                            && returnValue_ != OverwriteOptionPane.YES_TO_ALL_OPTION) {
                            OverwriteOptionPane pane = new OverwriteOptionPane();
                            returnValue_ =
                                OverwriteOptionPane.showOptionDialog(
                                    frm_,
                                    "Do you wish to overwrite the following finding chart "
                                        + name,
                                    "Overwrite files");
                                    
                            if(returnValue_ == OverwriteOptionPane.YES_OPTION ||
                               returnValue_ == OverwriteOptionPane.YES_TO_ALL_OPTION) {                                                
                                FileIOUtils.writeBinaryFile(fcs[i].getFcData(),outFile);
                            }
                                    
                                    
                        } else {
                            FileIOUtils.writeBinaryFile(fcs[i].getFcData(),outFile);                            
                        }                        
                    }                    
                }

            } catch (ObjectIOException e) {
                ErrorMessages.announceIOError(
                    frm_,
                    e);
            } catch (RuntimeException e) {
                e.printStackTrace();
                throw e;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

}
