package org.eso.ohs.phase2.actions;

import java.awt.event.ActionEvent;
import java.io.IOException;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.JPEGViewer;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSPersistence;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.ObjectSelector;
import org.eso.ohs.phase2.common.SimpleJpegViewer;

/**
 * Action class to display the finding charts for an ObservationBlock.
 *
 * <p>
 * $Id: ShowFindingChartsAction.java,v 1.26 2005/04/25 11:49:31 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */

public class ShowFindingChartsAction extends ActionSuperclass {


    private ObjectSelector objSelector_;

    private Media media;

    private JPEGViewer jpegViewer;

    private FindingChartViewer simpleJpegViewer;
    
    private SimpleJpegViewer p2ppFCViewer;

    
    private JFrame frm_;
    
    
    private JComponent parentComp;

    private boolean simple = true;

    private boolean folderView = true;


    /**
     * Constructs a ShowDetailsAction action.
     */
    public ShowFindingChartsAction(
        Media media,
        ObjectSelector objSelector,
        JFrame frm,
        boolean simple) {
        super(frm, "Finding Charts View");
        objSelector_ = objSelector;
        this.media = media;
        this.simple = simple;
        folderView = false;
        frm_ = frm;

    }
    
    
    /**
     * Constructs a ShowDetailsAction action.
     */
    public ShowFindingChartsAction(
        Media media,
        ObjectSelector objSelector,
        boolean simple) {
        super((JComponent)objSelector, "Finding Charts View");
        objSelector_ = objSelector;
        this.media = media;
        this.simple = simple;
        folderView = false;
        JComponent comp = (JComponent)objSelector;
        frm_ = (JFrame)comp.getTopLevelAncestor();

    }


    /**
     * Event handler for the ShowDetailsAction action.
     * 
     * @param ev       user-initiated ActionEvent to quit.
     */
    public void actionPerformedImpl(ActionEvent evt) {
        Long id = null;
        OHSPersistence objMgr =
            (OHSPersistence) OHSPersistence.getObjectManager();
        try {
            ObservationBlock ob=null;
            CalibrationBlock cb =
                    (CalibrationBlock) objSelector_.getDisplayedObject();
            if(cb == null || cb.getType().equalsIgnoreCase("C")) {
                JOptionPane.showMessageDialog(
                                    frm_,
                                    new Object[] { "No OB Selected." },
                                    "No OB Selected",
                                    JOptionPane.ERROR_MESSAGE);
                                return;         
            } else {
                ob = (ObservationBlock)cb;
            }
            
            
                    
            if (ob.getFindingCharts().length <= 0) {
                JOptionPane.showMessageDialog(
                    frm_,
                    new Object[] { "There are no finding charts for this ob." },
                    "Finding Charts not found",
                    JOptionPane.ERROR_MESSAGE);
                return;
            }                    
            if (!simple) {
                //TODO for the OT
                simpleJpegViewer = FindingChartViewer.getInstance();
                simpleJpegViewer.init(media,ob);

            } else {
                p2ppFCViewer  = SimpleJpegViewer.getInstance();
                p2ppFCViewer.init(ob);

            }
        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(frm_, e);
        } catch (ObjectNotFoundException e) {
            e.printStackTrace();
            ErrorMessages.announceIOError(frm_, e);
        } catch (IOException e) {
            ErrorMessages.announceIOError(frm_, e);            
        }
    }

    /**
     * Returns the media.
     * @return Media
     */
    public Media getMedia() {
        return media;
    }

    /**
     * Sets the media.
     * @param media The media to set
     */
    public void setMedia(Media media) {
        this.media = media;
    }

}
