package org.eso.ohs.phase2.actions;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;


/**
 * Action class to display the selected ObservationBlock in PAF form.
 *
 * <p>
 * @version $Id: ShowPAFDescrAction.java,v 1.14 2005/03/30 09:56:13 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * <br>ChangeLog: cmavros 1501/OT Change to allow user to select more than
 *    one OB.
 * <br>ChangeLog: tcanavan 2002/25/02, Initialisation of the PafTextDisplayWidget
 */

public class ShowPAFDescrAction extends ActionSuperclass {

    private ObjectContainer folderView_;
  /** Boolean to turn off the header after first OB is read */
    protected boolean getHDR_;
    protected boolean folder_;
    protected static PafTextDisplayWidget log_ = new PafTextDisplayWidget(); 
    private CalibrationBlock [] cbList_;
  
    /**
     * Constructs a ShowPAFDescrAction action.
     */
    public ShowPAFDescrAction(ObjectContainer folderView, boolean folder, 
             String name) {
        super((JComponent)folderView, name);
        folderView_ = folderView;
        folder_ = folder;
    }

    /**
     * Event handler for the ShowPAFDescrAction action.
     * 
     * @param ev       user-initiated ActionEvent to quit.
     */
    public void actionPerformedImpl(ActionEvent evt) {
        Long [] idLongs = null;
        getHDR_ = true;
        if ( !folder_ ) {
            idLongs = folderView_.getAllSelected();
        }
        else {
            idLongs = folderView_.getAll();
        }

        idLongs = (Long[])ObjUtils.uniqueOccurrences(idLongs);

        if(idLongs == null || idLongs.length <= 0) {
            JOptionPane.showMessageDialog(
                                          folderView_.getTopLevelAncestor(),
                                          new Object[] {
                                              "There are no selected business objects",
                                              " ",
                                              "You must first select business objects to report on."
                                          },
                                          "Nothing selected",
                                          JOptionPane.ERROR_MESSAGE
                                          );
            return;
        }
        new IncrementalActionAutomaton(
                                       new IncrementalReport( idLongs),
                                       (JFrame)folderView_.getTopLevelAncestor(),"Reports...");
    }

    /**
     * Utility class to export a sequence of Business Objects, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalReport implements IncrementalAction {

        /** Array of ids of objects to be duplicated */
        private Long[] idLongs_;
        StringBuffer buf = new StringBuffer();

        Vector cbList = new Vector();

        /**
         * Constructor.
       	 *
         * @param device    media containing the objects to be duplicated
         * @param objType   type of object to be duplicated
         * @param idLongs   array of ids of objects to be duplicated
       	 */
        public IncrementalReport(Long[] idLongs) {
            idLongs_  = idLongs;
        }

        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(idLongs_,counter);
        }

        public void startOfAction() { 
            log_.setVisible(false);
            log_.clearTextArea();
            getHDR_ = true;
        }

        public void endOfAction() {
            CalibrationBlock [] cbs = new CalibrationBlock[cbList.size()];
            cbList.copyInto(cbs);
            log_.setCBs(cbs);
            if((buf.toString()).length() > 0){
                log_.setTextArea(buf.toString());
                log_.setSize(new Dimension(600,400));
                log_.setVisible(true);
            }
            else {
                JOptionPane.showMessageDialog(
                                              folderView_.getTopLevelAncestor(),
                                              new Object[] {
                     " None of the selections cannot be displayed in OBD (PAF) format." ,
                     "", "" }, "OBD (PAF) Format", JOptionPane.ERROR_MESSAGE);
            } 
        }


        /**
         * Method to duplicate the n'th element of obs, where n = counter.
       	 *
         * @param counter counter to show which element of obs to check in.
         * @return true if the duplicating should continue
         *   and false otherwise.
       	 */
        public void performStep(int counter) {
            if (idLongs_[counter] != null) {
                doPAF(idLongs_[counter].longValue());
                //cbList.addElement(cb);
            }
         }

        /**
         * ObservationBlock in PAF form.
       	 */
        public void doPAF(long id){
            try {
                ObjectManager objMgr = ObjectManager.getObjectManager();
                Class objType = Config.getCfg().getClassFromId(id);
                if(folderView_.getMedia().equals(Media.DBASE))
                    objMgr.refreshBusObjRecursive(folderView_.getMedia(), id, objType);                
                CalibrationBlock cb = (CalibrationBlock)objMgr.getBusObj(folderView_.getMedia(), id, objType);

                String errs = cb.verify();
                if ( ! errs.equals("") ) {
                    JOptionPane.showMessageDialog(
                                                  folderView_.getTopLevelAncestor(),
                                                  new Object[] { errs, " ",
                                                                 "'" + cb.getName() + "'" +
                                                                 " cannot be displayed in OBD (PAF) format." },
                                                  "Error",
                                                  JOptionPane.ERROR_MESSAGE);
                    return;
                }               
                buf.append(cb.toOBD(getHDR_,true,objMgr.getDbaseId(cb),OHSApplication.getShortDesc()));
                cbList.addElement(cb);
                getHDR_ = false;
            }              
            catch (ObjectNotFoundException e) {
                e.printStackTrace();
                announceNoObject(e);
                return;
            }
            catch (ObjectIOException e) {
                announceIOError(e);
                return;
            }
        }
    }

    public static class PafTextDisplayWidget extends TextDisplayWidget {

        private CalibrationBlock [] cbList;

        PafTextDisplayWidget() {
            super();
        } 


        public void setCBs(CalibrationBlock [] cbList) {
            this.cbList = cbList;
        }

        private JMenu getFileMenu() {
            return fileMenu;  
        }

        private JTextArea getTextArea() {
            return textArea_;
        }       

        public void createSaveActionListener() {
           saveReport_.addActionListener(new SaveActionListener());
        }

        public class SaveActionListener implements ActionListener {
    	    public void actionPerformed(ActionEvent ae) {
                JFileChooser chooser = new SwingFileChooser();
                int option = chooser.showSaveDialog(getFileMenu());
                ObjectManager objMgr = ObjectManager.getObjectManager();
                if(option == JFileChooser.APPROVE_OPTION) {
                    try {
                        StringBuffer buf = new StringBuffer();
                        for(int i=0;i<cbList.length;i++) {
                             if(i==0)
                                 buf.append(cbList[i].toOBD(true,false,objMgr.getDbaseId(cbList[i]),OHSApplication.getShortDesc()));
                             else
                                 buf.append(cbList[i].toOBD(false,false,objMgr.getDbaseId(cbList[i]),OHSApplication.getShortDesc())); 
                        }
                        FileIOUtils.stringToFile(buf.toString(),chooser.getSelectedFile().getAbsolutePath());
                    }
                    catch (Throwable t) {
                        getTextArea().setText("Failed to write file contents.");
                    }
                }
            }
        }
    }
   
}



 
