package org.eso.ohs.phase2.apps.masktracker.actions;

import java.awt.event.ActionEvent;
import java.sql.SQLException;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.phase2.apps.masktracker.MaskDiscarding;

/**
 * Action class to display the selected ObservationBlock in PAF form.
 *
 * <p>
 * $Id: DiscardingReportAction.java,v 1.12 2005/02/11 10:31:49 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * @author cmavros 1501/OT Change to allow user to select more than
 *    one OB.
 */

public class DiscardingReportAction extends ActionSuperclass {

    private ObjectContainer folderView_;
  /** Boolean to turn off the header after first OB is read */
    protected boolean getHDR_;
    protected boolean folder_;
    private boolean  report_;
    protected static TextDisplayWidget log_ = new TextDisplayWidget(); 
    private CalibrationBlock [] cbList_;
  
    /**
     * Constructs a ShowPAFDescrAction action.
     */
    public DiscardingReportAction(ObjectContainer folderView, boolean folder, 
             String name,boolean report) {
        super((JComponent)folderView, name);
        folderView_ = folderView;
        folder_ = folder;
        report_ = report;
    }

    /**
     * Event handler for the ShowPAFDescrAction action.
     * 
     * @param ev       user-initiated ActionEvent to quit.
     */
    public void actionPerformedImpl(ActionEvent evt) {
        Long [] idLongs = null;
        getHDR_ = true;
        if ( !folder_ ) {
            idLongs = folderView_.getAllSelected();
        }
        else {
            idLongs = folderView_.getAll();
        }

        idLongs = (Long[])ObjUtils.uniqueOccurrences(idLongs);

        if(idLongs == null || idLongs.length <= 0) {
            JOptionPane.showMessageDialog(
                                          folderView_.getTopLevelAncestor(),
                                          new Object[] {
                                              "There are no selected business objects",
                                              " ",
                                              "You must first select business objects to report on."
                                          },
                                          "Nothing selected",
                                          JOptionPane.ERROR_MESSAGE
                                          );
            return;
        }
        new IncrementalActionAutomaton(
                                       new IncrementalReport( idLongs),
                                       (JFrame)folderView_.getTopLevelAncestor(),"Reports...");
    }

    /**
     * Utility class to export a sequence of Business Objects, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalReport implements IncrementalAction {

        /** Array of ids of objects to be duplicated */
        private Long[] idLongs_;
        StringBuffer buf = new StringBuffer();

        Vector cbList = new Vector();

        /**
         * Constructor.
       	 *
         * @param device    media containing the objects to be duplicated
         * @param objType   type of object to be duplicated
         * @param idLongs   array of ids of objects to be duplicated
       	 */
        public IncrementalReport(Long[] idLongs) {
            idLongs_  = idLongs;
        }

        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(idLongs_,counter);
        }

        public void startOfAction() {
            log_.setVisible(false);
            log_.clearTextArea();
            getHDR_ = true;
            //Updater.propagateMaskInfo();
        }

        public void endOfAction() {
            try {
                StringBuffer buf = new StringBuffer();
                String reportStr = null;
                long [] ids = new long [idLongs_.length];
                for(int i=0;i<ids.length;i++) {
                    ids[i] = idLongs_[i].longValue();
                }
                if(report_)
                     reportStr = MaskDiscarding.getInstance().getDiscardingText(ids);
                else
                     reportStr = MaskDiscarding.getInstance().getDiscardingReport(ids);              
                log_.setTextArea(reportStr);
                log_.setVisible(true);
            }
            catch(SQLException e) {
            }
        }


        /**
         * Method to duplicate the n'th element of obs, where n = counter.
       	 *
         * @param counter counter to show which element of obs to check in.
         * @return true if the duplicating should continue
         *   and false otherwise.
       	 */
        public void performStep(int counter) {
        }
    }

 
   
}



 
