/**
 * @version $Id: MenuGroups.java,v 1.31 2005/09/28 14:32:31 pnunes Exp $
 * @author cmavros 2002/02/11, 1510/OT Add OBD Format
 * @author cmavros 2002/19/02, 1640/OT Add Refresh Menu option
 * @author tcanavan 2002/25/02, Initialisation of the PafTextDisplayWidget
 */

package org.eso.ohs.phase2.apps.ot.gui;

import java.awt.Cursor;

import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.phase2.actions.ExpTimeReportAction;
import org.eso.ohs.phase2.actions.MissingTemplatesReport;
import org.eso.ohs.phase2.actions.OBReportAction;
import org.eso.ohs.phase2.actions.PrintFindingCharts;
import org.eso.ohs.phase2.actions.ReadmeStatusHistoryAction;
import org.eso.ohs.phase2.actions.SaveFCAction;
import org.eso.ohs.phase2.actions.ShowAcceptedReadmeAction;
import org.eso.ohs.phase2.actions.ShowPAFDescrAction;
import org.eso.ohs.phase2.actions.ShowReadmeFileHistoryAction;
import org.eso.ohs.phase2.actions.Stage2VerifyBOAction;
import org.eso.ohs.phase2.actions.USGEditAction;
import org.eso.ohs.phase2.actions.ViewEphFileAction;
import org.eso.ohs.phase2.actions.VisiplotAction;
import org.eso.ohs.phase2.apps.ot.actions.ChileNotesReadmeAction;
import org.eso.ohs.phase2.apps.ot.actions.ConfigReportAction;
import org.eso.ohs.phase2.apps.ot.actions.OrangStartAction;

public class MenuGroups extends JFrame {

    protected JMenuBar menuBar;

    //file menu items
    protected JMenu fileMenu;
    protected JMenuItem newMenuItem;
    protected JMenuItem openMenuItem;
    protected JMenuItem reloadMenuItem;
    protected JMenuItem refreshMenuItem;
    protected JMenuItem closeMenuItem;

    protected JMenuItem saveMenuItem;
    protected JMenuItem saveAsMenuItem;
    //protected JMenuItem   dumpMenuItem;
    protected JMenuItem deleteMenuItem;

    protected JMenuItem viewMenuItem;
    protected JMenuItem repositoryBrowserMenuItem;
    protected JMenuItem maskBrowserMenuItem;
    protected JMenuItem processReportMenuItem;
    protected JMenuItem quitMenuItem;
    protected JMenu showPAFDescriptionMenu;
    //protected JMenuItem showPAFDescriptionMenuItem_;

    //edit menu items
    protected JMenu editMenu;
    protected JMenuItem clearMenuItem;
    protected JMenuItem renameMenuItem;

    //services menu items
    protected JMenu servicesMenu;
    protected JMenuItem ccsSetupMenuItem;

    protected Action openQueueAction;
    protected Action deleteQueueAction;
    protected Action newQueueAction;
    protected Action closeQueueAction;
    protected Action saveQueueAction;
    protected Action dumpQueueAction;
    protected Action reloadQueueAction;
    protected Action refreshQueueAction;
    protected Action repositoryBrowserAction;
    protected Action saveAsQueueAction;
    protected Action deleteOpenQueueAction;
    protected Action clearQueueAction;

    protected Action deselectAllAction;
    protected Action moveUpAction;
    protected Action moveDownAction;
    protected Action ephViewAction;
    protected JMenu expMenu;
    protected JMenu obMenu;
    protected JMenu verifyMenu;
    protected JMenu configReportMenu;
    protected JMenu missingTplMenu;
    protected JMenu visiplotMenu;
    protected JMenu printFCMenu;
    protected JMenu readmeMenu;
    protected JMenu orangMenu;

    
    protected JMenu saveFCMenu;

    public MenuGroups() {
    }

    /**
         * Retrieves or creates the menu with the given name in the menubar.
         * 
         * @param menuName	    name of the menu
         */
    protected JMenu getMenu(String menuName) {
        JMenuBar mb = getJMenuBar();
        int numMenus = 0;
        JMenu m = null;
        boolean found = false;

        if (mb == null) {
            mb = new JMenuBar();
            setJMenuBar(mb);
        }

        numMenus = mb.getMenuCount();

        for (int i = 0; i < numMenus; i++) {
            m = mb.getMenu(i);
            if (m.getText().equals(menuName)) {
                found = true;
                break;
            }
        }
        if (!found) {
            m = new JMenu(menuName);
            mb.add(m);
        }
        return m;
    }

    public void reinitReportsMenu(ObjectContainer container) {
        if (expMenu != null)
            removeMenuItems(expMenu);
        if (obMenu != null)
            removeMenuItems(obMenu);
        if (verifyMenu != null)
            removeMenuItems(verifyMenu);
        if (configReportMenu != null)
            removeMenuItems(configReportMenu);
        if (missingTplMenu != null)
            removeMenuItems(missingTplMenu);
        if (showPAFDescriptionMenu != null)
            removeMenuItems(showPAFDescriptionMenu);
        if (visiplotMenu != null)
            removeMenuItems(visiplotMenu);
        createReportsMenu(container);
    }

    public void removeMenuItems(JMenu m) {
        for (int i = m.getItemCount() - 1; i >= 0; i--) {
            if (m.getItem(i) == null)
                continue;
            m.remove(m.getItem(i));
        }
    }

    /**
     * Adds a action to the named menu.
     * 
     * @param menuName	    name of the menu
     * @param subMenu	    subMenu to be added
     */
    public JMenuItem addSubMenu(String menuName, JMenuItem subMenu) {
        JMenu m = getMenu(menuName);
        JMenuItem mnItem = m.add(subMenu);
        return mnItem;
    }

    /**
     * Constructor.
     * 
     * @param title	initial title of window
     */

    public MenuGroups(String title) {
        super(title);
    }

    public void buildOpenQueueMenu() {
        JMenu openQueueMenu = new JMenu("Open Queue List");
        reloadMenuItem = openQueueMenu.add(reloadQueueAction);
        closeMenuItem = openQueueMenu.add(closeQueueAction);
        saveMenuItem = openQueueMenu.add(saveQueueAction);
        //dumpMenuItem = openQueueMenu.add(dumpQueueAction);        
        menuBar.add(openQueueMenu);
    }

    public void waitCursor() {
        JComponent glassPane = (JComponent) getGlassPane();

        glassPane.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        glassPane.setVisible(true);
    }

    public void normalCursor() {
        JComponent glassPane = (JComponent) getGlassPane();

        glassPane.setVisible(false);
    }

    public void createReportsMenu(ObjectContainer container) {
        String menuName = "Reports";
        if (expMenu == null)
            expMenu = new JMenu("Execution Time");
        expMenu.add(new ExpTimeReportAction(container, true, "All"));
        expMenu.add(new ExpTimeReportAction(container, false, "Selected"));

        if (obMenu == null)
            obMenu = new JMenu("ObsBlocks Breakdown");
        obMenu.add(new OBReportAction(container, true, "All"));
        obMenu.add(new OBReportAction(container, false, "Selected"));

        if (verifyMenu == null)
            verifyMenu = new JMenu("Verify");
        verifyMenu.add(new Stage2VerifyBOAction(container, true, "All"));
        verifyMenu.add(new Stage2VerifyBOAction(container, false, "Selected"));
        if (configReportMenu == null)
            configReportMenu = new JMenu("Config Reports");
        configReportMenu.add(new ConfigReportAction(container, true, "All"));
        configReportMenu.add(
            new ConfigReportAction(container, false, "Selected"));

        // amchavan, 21-Oct-2001
        if (missingTplMenu == null)
            missingTplMenu = new JMenu("Missing Templates");
        missingTplMenu.add(new MissingTemplatesReport(container, true, "All"));
        missingTplMenu.add(
            new MissingTemplatesReport(container, false, "Selected"));

        if (showPAFDescriptionMenu == null)
            showPAFDescriptionMenu = new JMenu("Display in OBD (PAF) format");
        showPAFDescriptionMenu.add(
            new ShowPAFDescrAction(container, true, "All"));
        showPAFDescriptionMenu.add(
            new ShowPAFDescrAction(container, false, "Selected"));

        createVisiplotMenu(container);

        addSubMenu(menuName, expMenu);
        addSubMenu(menuName, obMenu);
        addSubMenu(menuName, configReportMenu);
        addSubMenu(menuName, verifyMenu);
        addSubMenu(menuName, missingTplMenu); // amchavan, 21-Oct-2001: 
        addSubMenu(menuName, showPAFDescriptionMenu);
        addSubMenu(menuName, visiplotMenu);

    }

	/**
	 * @param container
	 */
	protected void createVisiplotMenu(ObjectContainer container) {
		if (visiplotMenu == null)
            visiplotMenu = new JMenu("Visibility Plots");
        visiplotMenu.add(new VisiplotAction(container, true, "All"));
        visiplotMenu.add(new VisiplotAction(container, false, "Selected"));
	}
  

    public void createReadmeMenu(ObjectContainer container) {
        String menuName = "Readme";
        if (readmeMenu == null)
            readmeMenu = getMenu( menuName);
        if(AppConfig.getAppConfig().getBoolean(OHSKeys.USGEnabledKey)) {
            readmeMenu.add(new ShowReadmeFileHistoryAction(container,this,"View Current Readme"));
            readmeMenu.add(new ShowAcceptedReadmeAction(container,this,"View Readme At The Mountain"));
            readmeMenu.add(new ReadmeStatusHistoryAction(container));
            readmeMenu.add(new USGEditAction(container));
        } else {
            readmeMenu.add(new ShowAcceptedReadmeAction(container,this,"View Readme"));
            readmeMenu.add(new ChileNotesReadmeAction(container,"Readme Notes"));
            //readmeMenu.add(new USGEditAction(container));
        }
    }

    
    public void createOrangMenu(ObjectContainer container) {
        String menuName = "Orang";
          orangMenu = getMenu(menuName);
           orangMenu.add(new OrangStartAction(container,this,"Start"));
        
    }

    
    
    

    public void createFCPrintMenu(ObjectContainer container) {
        String menuName = "Finding Charts";
        if (printFCMenu == null)
            printFCMenu = new JMenu("Print");
        printFCMenu.add(
            new PrintFindingCharts(Media.DBASE, container, "All", true));
        printFCMenu.add(
            new PrintFindingCharts(Media.DBASE, container, "Selected", false));
        addSubMenu(menuName, printFCMenu);
        String saveMenu = "Save As jpg";
        if (saveFCMenu == null)
            saveFCMenu = new JMenu(saveMenu);            
        saveFCMenu.add(new SaveFCAction(Media.DBASE, container, "All",true));        
        saveFCMenu.add(new SaveFCAction(Media.DBASE, container, "Selected", false));    
        addSubMenu(menuName,saveFCMenu);
    }
    
    

    
    protected void createEphFileMenu(ObjectContainer view) {
        final String menuName = "Ephemeris Files";
        ephViewAction = new ViewEphFileAction(view, Media.DBASE);
        JMenu ephMenu = getMenu(menuName);
        ephMenu.add(ephViewAction);
    }

    
    

    /** cmavros 25-10-01 Copy from p2pp*/
    public void setEnabledReports(boolean enabled) {
        getMenu("Reports").setEnabled(enabled);
    }

    /** cmavros 25-10-01 Copy from p2pp*/
    public void setEnabledFCPrint(boolean enabled) {
        getMenu("Finding Charts").setEnabled(enabled);
    }

}
