/*
 * @(#)CalibDataTreePanel.java   $Revision: 1.3 $ $Date: 2003/07/03 08:49:49 $
 *
 * Copyright (C) 2000 European Southern Observatory 
 * License:  GNU General Public License version 2 or later
 *
 */
package org.eso.ohs.phase2.apps.ot.gui;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;





/** 
 *This is a specialised JScrollPane  used to display a tree 
 *rappresentation of an ObservationBlock objects fileds
 *Some fields of an ObservationBlock may not be visible.
 *When the associated ObservationBlock is modified ,
 *changes will be reflected to the view 
 *
 *  @version $Revision: 1.3 $
 *  @author G.Passera, DMD/ESO, <gpassera@eso.org>
 */
public class OBTreeView extends JScrollPane 
    implements PropertyChangeListener {

private JTree tree;
private DefaultTreeModel model;   
private CalibrationBlock cb = null; //OB  associated to the view

    /**
     *Constructor for creating an empty panel
     */
    public OBTreeView()
    {
    super();
    tree = new JTree(new DefaultMutableTreeNode(new String("OB Tree view")));
    tree.getSelectionModel().setSelectionMode(
				 TreeSelectionModel.SINGLE_TREE_SELECTION);
    tree.putClientProperty("JTree.lineStyle", "Angled"); 
    tree.setCellRenderer(new MyRenderer());
    setViewportView(tree);

    }

    /**
     *Constructor for creating a panel in which is displaied  
     *a tree view of ObservationBlock's fields
     *@param ob specifies an ObservationBlock
     */
 
    public OBTreeView(CalibrationBlock cb)
    {
    super();
    this.cb = cb;
    cb.addPropertyChangeListener(this);
    if(cb instanceof ObservationBlock)
         model = new DefaultTreeModel(OBTreeNodeFactory.createOB((ObservationBlock)cb));
    else
         model = new DefaultTreeModel(OBTreeNodeFactory.createCB(cb));
    tree = new JTree(model);
    tree.getSelectionModel().setSelectionMode(
				 TreeSelectionModel.SINGLE_TREE_SELECTION);
    tree.putClientProperty("JTree.lineStyle", "Angled"); 
    tree.setCellRenderer(new MyRenderer());
    setViewportView(tree);
    }

    public void expandTree() {
        // expands all rows in tree
        for (int counter = 0; counter < tree.getRowCount(); counter++) {
            tree.expandRow(counter);
        }
    }

   
    /**
     *Updates the view displaing a new Observation block
     *@param newOb specifies a new observationBlock
     */
    public void updateView(CalibrationBlock newCb) 
    {
    if (newCb != null)
	{
	newCb.addPropertyChangeListener(this);
        if(newCb instanceof ObservationBlock)
	model = new DefaultTreeModel(OBTreeNodeFactory.createOB((ObservationBlock)newCb));
        else
	model = new DefaultTreeModel(OBTreeNodeFactory.createCB(newCb));
	}
    else
	{
	model = new DefaultTreeModel(
		       new DefaultMutableTreeNode("OB Tree view"));
	}

    tree.setModel(model);
    
    if (cb != null)
	{
	cb.removePropertyChangeListener(this);
	}
    cb = newCb;
    expandTree();
    }


    /**Handle changes for ObservationBlock associated to the view*/ 
    public void propertyChange(PropertyChangeEvent ev)
    {
//    stdlog_.debug("---> OB Tree View propagation---");
    updateView((CalibrationBlock) ev.getSource());
    }

    /**
     *Return the OB displayed by this view
     *@return the OB displayed by this view
     */
    public CalibrationBlock getOB() {
	return cb;
    }



    /**Used for rendering the tree*/
    class MyRenderer extends DefaultTreeCellRenderer 
    {
    public java.awt.Component getTreeCellRendererComponent(JTree tree,
                                                      Object value,
                                                      boolean sel,
                                                      boolean expanded,
                                                      boolean leaf,
                                                      int row,
                                                      boolean hasFocus) 
        {
        java.awt.Component component =
                super.getTreeCellRendererComponent(tree,value,sel,
                                                   expanded,leaf,row,
						   hasFocus);
        if (value == null)
                return component;

	if (((DefaultMutableTreeNode)value).isRoot())
	    {
	    component.setForeground(Color.red);
	    }
	
        else if ( ((DefaultMutableTreeNode)value).isLeaf()) 
            {
            component.setForeground(Color.gray);
            } 
        else 
            {
            component.setForeground(Color.blue);
            }
        return component;
        }
    } //class MyTreeRender
   
}


