package org.eso.ohs.phase2.apps.ot.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.sql.SQLException;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.RepaintManager;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DelConfigAction;
import org.eso.ohs.core.dbb.client.SaveConfigAction;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.utilities.GTreeNode;
import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.MaskQueue;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.Readme;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSPersistence;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.ObjectSelector;
import org.eso.ohs.persistence.dbase.phase2.QueueManager;
import org.eso.ohs.phase2.actions.ExportBOAction;
import org.eso.ohs.phase2.actions.ReloadOBAction;
import org.eso.ohs.phase2.apps.ot.actions.CloseQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.ReloadQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.SaveAsQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.SaveQueueAction;

/**
 * This view displays information about Queue objects <br>
 * ChangeLog: A M Chavan, 13-Jun-2001: removed 'rename queue' menu item <br>
 * ChangeLog: cmavros, 07/02/02, 1510/OT and add log4j <br>
 * ChangeLog: cmavros, 12-02-02 1641/OT <br>
 * ChangeLog: tcanavan, 25/02/02 Cleaned up indentation and put in some javadoc
 * <br>
 * ChangeLog: cmavros, 04-07-02 Read option folder keyword before creating the
 * option menu. <br>
 * 
 * @author G.Passera, DMD/ESO, <gpassera@eso.org>
 * @version $Revision: 1.48 $
 */
public class QueueView extends MenuGroups implements PropertyChangeListener,
        QueueContainer, ObjectSelector {

    public static final String cvsID_ = "$Id: QueueView.java,v 1.48 2005/12/12 16:01:39 ddorigo Exp $";

    protected static Logger stdlog_ = Logger.getLogger(QueueView.class);

    protected static final String dirtyString = " ** ";

    /**
     * OBs with these status values are NOT permitted to be members of Queues.
     */
    public static final String[] notQueueableStatus =
    { CalibrationBlock.DEFINED, CalibrationBlock.REJECTED };

    private JMenuBar menuBar;

    protected JMenu fileMenu;

    private JMenu OBsMenu;

    private JMenu editMenu;

    private JMenu optionsMenu_;

    // private JMenu STSMenu;
    // private JMenu viewMenu;
    private JMenu reload;

    private JMenuItem selectAllMenuItem;

    private JMenuItem removeMenuItem;

    private JMenuItem moveUpMenuItem;

    private JMenuItem moveDownMenuItem;

    // OBsMenuItems
    private JMenuItem displayMenuItem;

    private JMenuItem commentMenuItem;

    // private JMenu markSubMenu;
    // private JMenuItem reloadQueueItemsMenuItem;
    private JMenuItem queueForExecutionMenuItem;

    // <br>ChangeLog: amchavan, 28-May-2002: commented out
    // private JMenuItem queueMaskBrowserMenuItem;

    // file menu items
    protected JMenuItem reloadQueueMenuItem;

    protected JMenuItem closeQueueMenuItem;

    // <br>ChangeLog: amchavan, 28-May-2002: commented out
    // private JMenuItem maskQueueViewMenuItem;

    protected JMenuItem saveQueueMenuItem;

    protected JMenuItem saveAsQueueMenuItem;

    protected JMenuItem summaryQueueMenuItem;

    // private JMenuItem dumpQueueMenuItem;
    protected JMenuItem verifyQueueMenuItem;

    private boolean m_noRepaint = false;

    private WindowAdapter wl = null;

    // Options menu items

    ActionSuperclass actionMenuDefault_;

    private static final String CONFIG_KEYWORD = "QUEUEVIEW.CONFIG.CONFIGMENU";

    protected final String DEFAULT = "default";

    protected final String DEFAULT_REQUEST = "Save As Default";

    String userConfigFile_ = null;

    protected OBTreeView obTreeView_ = null;

    protected QueueItemsBView queueItemsView_ = null;

    protected Queue queue_ = null;

    protected QueuePropertiesView queuePropertiesView_ = null;

    protected CommentDialog commentDialog_ = new CommentDialog();

    protected static final int BORDER_WIDTH_ = 30;

    protected static final int BORDER_HEIGHT_ = 30;

    public QueueView(Queue newQueue) {
        super("Queue View: " + newQueue.getName());
        initQueueView(newQueue);
        // focusListener = new MyFocusListener();
        // stdlog_.debug("Added Focus Listener");

    }

    /**
     * @param newQueue
     *            void
     */
    protected void initQueueView(Queue newQueue) {
        newQueue(newQueue);
        setQueuePropertiesView(new QueuePropertiesView(newQueue));
        initQueueItemsBView();
        buildMainMenu();
        buildMenu();
        createView();
        setTitle("Queue view: " + queue_.getName() + " ("
                + queue_.getTelescope() + ")");

        if (actionMenuDefault_ != null) {
            actionMenuDefault_.actionPerformedImpl(new ActionEvent(
                    actionMenuDefault_, 0, ""));
        }

    }

    /**
     * void
     */
    protected void buildMainMenu() {
        menuBar = new JMenuBar();
        fileMenu = new JMenu("Queue");
        menuBar.add(fileMenu);

        editMenu = new JMenu("Edit");
        menuBar.add(editMenu);

        OBsMenu = new JMenu("OBs");
        menuBar.add(OBsMenu);
        setJMenuBar(menuBar);
    }

    /**
     * @param newQueue
     *            void
     */
    protected void newQueue(Queue newQueue) {
        queue_ = newQueue;
        queue_.addPropertyChangeListener(this);
    }

    public QueueView() {
        super();
    }

    protected String getOptionsKeyword() {
        return CONFIG_KEYWORD;
    }

    protected void setQueuePropertiesView(QueuePropertiesView view) {
        queuePropertiesView_ = view;

    }

    protected void initQueueItemsBView() {
        obTreeView_ = new OBTreeView();
        QueueItemsBView queueView = new QueueItemsBView(queue_, obTreeView_);
        queueView.init();
        queueItemsView_ = queueView;
    }

    public QueueItemsBView getQueueItemsBView() {
        return getQueueItemsView();
    }

    protected void forgetQueue(int index) {
        OTViewManager otMgr = OTViewManager.getOTViewMgr();
        otMgr.getQueueViews().removeElementAt(index);
        disposeNoSave();
        ObjectManager objMgr = ObjectManager.getObjectManager();
        Class c = Config.getCfg().getClassFromQueueId(queue_.getId());
        objMgr.forgetBusObj(Media.DBASE, getQueue().getId(), c);
    }

    protected void createView() {
        System.out.println("Creating Queue View");

        // queueView_ = new
        // QueueView(queue_,queueItemsView_,queuePropertiesView_);
        JPanel content = new JPanel();
        content.setLayout(new BorderLayout());

        JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                getQueueItemsView(), obTreeView_);

        content.add(getTitlePanel(), BorderLayout.NORTH);
        content.add(splitPane, BorderLayout.CENTER);
        getContentPane().add(content, BorderLayout.CENTER);

        stdlog_.debug("!!!!!!!!!!!!!! >>>>>>>>>>>>> Preferred Size = "
                + queuePropertiesView_.getMinimumSize());
        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();

        // Dimension window = new Dimension(740,650);
        // queueView_.setSize(window);

        // Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        queuePropertiesView_.setMinimumSize(new Dimension(0, 10));
        int frame_width = (screen.width / 3) * 2;
        int frame_height = (screen.height / 8) * 7;
        Dimension window = new Dimension(frame_width, frame_height);
        splitPane.setDividerLocation((frame_height / 4) * 2);
        setSize(window);
        int x, y;
        x = (5 * screen.width / 7) - (window.width / 2);
        if (x + window.width > screen.width) {
            x = screen.width - window.width;
        }

        if (x < BORDER_WIDTH_) {
            x = BORDER_WIDTH_;
        }
        y = screen.height / 3 - (window.height / 2);
        if (y + window.height > screen.height) {
            y = screen.height - window.height;
        }

        if (y < BORDER_HEIGHT_) {
            y = BORDER_HEIGHT_;
        }
        stdlog_
                .debug("!!!!!!!!!!!!!!!!! >>>>>>>>>>>>>>>> Minimum devider location "
                        + splitPane.getMinimumDividerLocation());
        setLocation(x, y);
        queueItemsView_.addDisplayTextAction(this);
        setVisible(true);
    }

    protected JPanel getTitlePanel() {
        return getQueuePropertiesView();
    }

    public void appendOB(long[] selectedOBIDs) {
        if (getQueueItemsView().appendOBs(selectedOBIDs)) {
            System.out.println("It worked");
            // queueItemsView_.startQuery();
        }
    }

    /**
     * Updates the view
     * 
     * @param newQueue
     *            specifies the new queue to be displayed
     */
    public void updateView(Queue newQueue) {
        if (queue_ != null) {
            queue_.removePropertyChangeListener(this);
        }
        newQueue(newQueue);
        queuePropertiesView_.setObject(newQueue);
        getQueueItemsView().setQueue(queue_);
        setTitle("Queue view: " + queue_.getName() + " ("
                + queue_.getTelescope() + ")");
    }

    public QueuePropertiesView getQueuePropertiesView() {
        return queuePropertiesView_;
    }

    public Queue getQueue() {
        return queue_;
    }

    public JFrame getParentFrame() {
        return this;
    }

    /**
     * This close metod is required to fix the known memory probem in JFrame
     */
    public void close() {
        m_noRepaint = true;
        Dimension d = getSize();
        RepaintManager.currentManager(this).getOffscreenBuffer(this, d.width,
                d.height);
    }

    /**
     * An overridden method which is identical to the base class except that it
     * returns null if we must not repaint
     */
    public Image createImage(int _width, int _height) {
        if (m_noRepaint) {
            return null;
        }
        return super.createImage(_width, _height);
    }

    /**
     * All removeMenuItems calls and a call to this classes close method are
     * needed because of a known memory problem in JFrame.
     */
    public void disposeNoSave() {
        getQueueItemsView().makeComponentsAavailableForGC();
        queue_.removePropertyChangeListener(this);
        close();
        super.dispose();
    }

    /**
     * PropertyChange
     */
    public void propertyChange(PropertyChangeEvent ev) {
        updateTitle();
    }

    public void updateTitle() {
        /** cmavros, 12-02-02 1641/OT */
        String title = getTitle();
        boolean dirty = toBeUpdated();
        if (dirty && !title.endsWith(dirtyString)) {
            title = "Queue view: " + queue_.getName() + " ("
                    + queue_.getTelescope() + ")";
            setTitle(title + dirtyString);
        } else if (!dirty && title.endsWith(dirtyString)) {
            title = "Queue view: " + queue_.getName() + " ("
                    + queue_.getTelescope() + ")";
            setTitle(title);
        } else if (dirty && title.endsWith(dirtyString)) {
            title = "Queue view: " + queue_.getName() + " ("
                    + queue_.getTelescope() + ")";
            setTitle(title + dirtyString);
        }
    }

    public boolean toBeUpdated() {
        if (queue_.isUpdate()) {
            return true;
        }
        return false;
    }

    public long[] getSelectedOpenQueueIds() {
        long[] ids = new long[1];
        ids[0] = queue_.getId();
        return ids;
    }

    /**
     * Builds a menu bar
     */
    protected void buildMenu() {
        try {
            buildFileMenu();
            buildEditMenu();
            buildOBsMenu();
            addSomeMenus();
        } catch (ObjectIOException e) {
            stdlog_.error(e);
        }
    }

    /**
     * @throws ObjectIOException
     *             void
     */
    protected void addSomeMenus() throws ObjectIOException {
        if (!AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
            createReadmeMenu(getQueueItemsView().getView());
            createEphFileMenu(getQueueItemsView().getView());
        }
        createReportsMenu(getQueueItemsView().getView());
        createFCPrintMenu(getQueueItemsView().getView());
        if (optionsSetup()) {
            optionsMenu_ = new JMenu("Options");
            buildOptionsMenu();
            menuBar.add(optionsMenu_);
            setJMenuBar(menuBar);
        }

        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        wl = new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                closeQueue();
            }
        };
        addWindowListener(wl);
    }

    /**
     * Builds the file menu
     */
    public void buildFileMenu() {

        reloadQueueMenuItem = fileMenu
                .add(new ReloadQueueAction(this, "Revert"));
        reloadQueueMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_R, ActionEvent.CTRL_MASK));
        closeQueueMenuItem = fileMenu.add(new CloseQueueAction(this, "Close"));
        fileMenu.addSeparator();
        saveQueueMenuItem = fileMenu.add(new SaveQueueAction(this, "Save"));
        saveAsQueueMenuItem = fileMenu.add(new SaveAsQueueAction(this,
                "Save As"));
        // dumpQueueMenuItem = fileMenu.add(new
        // SaveAsQueueAction(this,"Dump.."));
        fileMenu.add(new ExportBOAction(Media.DBASE, getQueueItemsView()
                .getView(), "Export...", true, true));
        fileMenu.addSeparator();
        summaryQueueMenuItem = fileMenu.add(new SummaryAction());
        summaryQueueMenuItem.setEnabled(false);
        fileMenu.addSeparator();
        JMenuItem printDetailedAreaMenuItem = fileMenu.add(new PrintAction());
        printDetailedAreaMenuItem.setEnabled(false);
        // <br>ChangeLog: amchavan, 28-May-2002: commented out
        // maskQueueViewMenuItem = fileMenu.add(new MaskQueueBrowserAction());
        // fileMenu.add(maskQueueViewMenuItem);
        closeQueueMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W,
                ActionEvent.CTRL_MASK));
        closeQueueMenuItem.setMnemonic(KeyEvent.VK_W + KeyEvent.CTRL_MASK);

        saveQueueMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
                ActionEvent.CTRL_MASK));
        saveQueueMenuItem.setMnemonic(KeyEvent.VK_S + KeyEvent.CTRL_MASK);
    }

    /**
     * Create and populate the "Edit" menu.
     */
    public void buildEditMenu() {
        selectAllMenuItem = editMenu.add(new SelectAllAction());
        selectAllMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A,
                InputEvent.CTRL_MASK, false));
    }

    public void finalize() throws Throwable {
        // This cannot use stdlog_.debug() as it gives the closed appender
        // error message from log4j
        System.out.println("FINALIZE - - ot.gui.QueueView - - FINALIZE");
        super.finalize();
    }

    /** Sets the managed queue */
    public void setQueue(Queue newQueue) throws SQLException {
        queue_ = newQueue;
        getQueueItemsView().updateView(newQueue);
        getQueuePropertiesView().setObject(newQueue);
    }

    public void clearQueue() {
        getQueueItemsView().removeAllOBs();
    }

    public void reinitOBsMenu() {
        if (OBsMenu != null)
            removeMenuItems(OBsMenu);
        if (reload != null)
            removeMenuItems(reload);
        buildOBsMenu();
    }

    public void reinitFCPrintMenu() {
        if (printFCMenu != null)
            removeMenuItems(printFCMenu);
        if (saveFCMenu != null)
            removeMenuItems(saveFCMenu);
        createFCPrintMenu(getQueueItemsView().getView());
    }

    protected void buildOBsMenu() {
        addFirstItems();
        addMoveItems();
        addLastItems();

    }

    protected void buildOBsMenuNoMove() {
        addFirstItems();
        addLastItems();

    }

    /**
     * void
     */
    private void addLastItems() {
        queueForExecutionMenuItem = OBsMenu.add(getQueueItemsView()
                .getExecutionAction());
        queueForExecutionMenuItem.setText("Execution Sequence");

        /**
         * Moved submenu item from OB menu to Reports menu to get consistent
         * with OT Main Menu 1510/OT CMM 07/02/02 showPAFDescriptionMenuItem =
         * OBsMenu.add(queueItemsView_.getShowPAFDescriptionAction());
         */

        OBsMenu.add(getQueueItemsView().getExportAction());
        if (AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
            OBsMenu.add(getQueueItemsView().getDumpAdpAdmDataAction());
        }
        OBsMenu.add(getQueueItemsView().getObHistoryAction());

        removeMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,
                ActionEvent.CTRL_MASK));
        removeMenuItem.setMnemonic(KeyEvent.VK_X + KeyEvent.CTRL_MASK);

        queueForExecutionMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_E, ActionEvent.CTRL_MASK));
        queueForExecutionMenuItem.setMnemonic(KeyEvent.VK_E
                + KeyEvent.CTRL_MASK);
    }

    /**
     * void
     */
    private void addFirstItems() {
        reload = new JMenu("Reload OB");
        reload.add(new ReloadOBAction(getQueueItemsView().getView(), true,
                "All"));
        reload.add(new ReloadOBAction(getQueueItemsView().getView(), false,
                "Selected"));
        OBsMenu.add(reload);
        if (!AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
            displayMenuItem = OBsMenu.add(getQueueItemsView()
                    .getDisplayAction());
        }
        if (!AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
            commentMenuItem = OBsMenu.add(getQueueItemsView()
                    .getOpenCommentsAction());
        }
        removeMenuItem = OBsMenu.add(getQueueItemsView().getRemoveAction());
    }

    /**
     * void
     */
    private void addMoveItems() {
        moveUpMenuItem = OBsMenu.add(getQueueItemsView().getMoveUpAction());
        moveDownMenuItem = OBsMenu.add(getQueueItemsView().getMoveDownAction());
        moveUpMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P,
                ActionEvent.CTRL_MASK));
        moveUpMenuItem.setMnemonic(KeyEvent.VK_P + KeyEvent.CTRL_MASK);

        moveDownMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,
                ActionEvent.CTRL_MASK));
        moveDownMenuItem.setMnemonic(KeyEvent.VK_N + KeyEvent.CTRL_MASK);
    }

    /**
     * Read the config file and the neccessary keyword for creating the
     * "Options" menu in the database browser view.
     * 
     * @return true if no exceptions caught
     */
    protected boolean optionsSetup() {
        OHSConfig config = OHSConfig.getInstance();
        AppConfig appConfig = AppConfig.getAppConfig();
        try {
            String optionDir = appConfig.getOptionDir();
            // stdlog_.info ("Options Dir: " + optionDir);
            if (optionDir != null && optionDir.length() > 0) {
                String fileName = config.getValue(getOptionsKeyword());
                if (fileName != null && fileName.length() > 0) {
                    userConfigFile_ = optionDir + File.separator + fileName;
                    config.addParamfile(userConfigFile_);
                } else {
                    return false;
                }
            } else {
                return false;
            }
        } catch (Exception e) {
            stdlog_.info("File: " + userConfigFile_ + " does not exist.");
            try {
                PrintWriter out = new PrintWriter(new FileWriter(
                        userConfigFile_));
                out.println("");
                out.close();
                stdlog_.info("File: " + userConfigFile_ + " created.");
                config.addParamfile(userConfigFile_);
            } catch (java.io.FileNotFoundException e1) {
                return false;
            } catch (org.eso.ohs.core.utilities.NotASCIIException e2) {
                return false;
            } catch (java.io.IOException e3) {
                return false;
            }
        }
        return true;
    }

    /**
     * 
     * Create and populate the "Options" menu in the database browser view.
     * 
     * @exception ObjectIOException
     *                object IO Exception
     */
    private void buildOptionsMenu() throws ObjectIOException {

        SaveConfigAction confAction = new SaveConfigAction(this, optionsMenu_,
                null, QueueItemsBView.colPanel_, QueueItemsBView.sortPanel_,
                CONFIG_KEYWORD, userConfigFile_, "Save settings", true);
        optionsMenu_.add(confAction);

        confAction = new SaveConfigAction(this, optionsMenu_, null,
                QueueItemsBView.colPanel_, QueueItemsBView.sortPanel_,
                CONFIG_KEYWORD, userConfigFile_, DEFAULT_REQUEST, true);
        optionsMenu_.add(confAction);

        DelConfigAction delAction = new DelConfigAction(this, optionsMenu_,
                CONFIG_KEYWORD, userConfigFile_, "Delete Settings", true);
        delAction.setMenuMap(confAction.getMenuMap());

        optionsMenu_.add(delAction);
        optionsMenu_.addSeparator();
        confAction.addMenus(CONFIG_KEYWORD);
        confAction.applyDefault("QUEUEVIEW.CONFIG.MENUDEFAULT");
        menuBar.add(optionsMenu_);
    }

    /**
     * Get the selection widget values from configuration and save them into an
     * action
     * 
     * @return the action that applies the retrieved configuration
     */
    ActionSuperclass getMenuSetUpFromConfig(GTreeNode menu, String menuName) {
        GTreeNode colWidgets = menu.getChild("cols");
        GTreeNode rowWidgets = menu.getChild("rows");
        GTreeNode sortWidgets = menu.getChild("sort");
        ConfigValuesAction action = new ConfigValuesAction(this, menuName,
                colWidgets, rowWidgets, sortWidgets);
        return action;
    }

    /**
     * @return true if the status index given represents a status that does not
     *         allow the OB to be queued.
     */
    public static boolean isNotQueueableStatus(int idx) {
        String status = CalibrationBlock.getStatusCode(idx);
        return isNotQueueableStatus(status);
    }

    /**
     * @return true if the status given represents a status that does not allow
     *         the OB to be queued.
     */
    public static boolean isNotQueueableStatus(String label) {
        for (int i = 0; i < notQueueableStatus.length; i++) {
            if (label.equals(notQueueableStatus[i])) {
                return true;
            }
        }
        return false;
    }

    public void updateQueue() {
        getQueuePropertiesView().updateQueue();
    }

    /**
     * Mark the status of a set of OBs, including dequeuing the OB if required.
     * This method was moved here as it was duplicated in both
     * RepositoryBrowserView and QueueItemsBView - it is not shared by both.
     * <br>
     * ChangeLog: nkornwei
     */
    public static void markStatus(long[] OBIDs, int i)
            throws ObjectNotFoundException, ObjectIOException {
        ObjectManager objMgr = ObjectManager.getObjectManager();
        boolean obDequeueRequired = isNotQueueableStatus(i);
        if (obDequeueRequired) {
            // must remove the OBs from all queues first.
            deleteQueuesReference(OBIDs);
        }
        QueueManager.updateOBs(OBIDs, CalibrationBlock.getStatusCode(i), null,
                null);
        for (int j = 0; j < OBIDs.length; j++) {
            if (objMgr.isInRegistry(OBIDs[j])) {
                Class c = Config.getCfg().getClassFromId(
                        Config.getCfg().uniqueToTableId(OBIDs[j]));
                CalibrationBlock cb = (CalibrationBlock) objMgr.getBusObj(
                        Media.DBASE, OBIDs[j], c);
                cb.setStatus(CalibrationBlock.getStatusCode(i));
            }
        }
    }

    public static void markReadmeStatus(ObservingRun obsRun, int index)
            throws ObjectIOException, ObjectNotFoundException, IOException {
        String status = Readme.statusValues_.getValueFromIndex(index);
        OHSPersistence persistence = (OHSPersistence) OHSPersistence
                .getObjectManager();
        Readme readme = persistence.populateReadmeFile(Media.DBASE, obsRun);
        obsRun.setReadme(readme);
        persistence.setReadmeFileStatus(Media.DBASE, status, obsRun.getId());
        Vector queueViews = OTViewManager.getOTViewMgr().getQueueViews();
        for (int i = 0; i < queueViews.size(); i++) {
            QueueView view = (QueueView) queueViews.elementAt(i);
            QueueItemsBView bView = view.getQueueItemsBView();
            bView.refreshReadmeStatus(Config.getCfg().uniqueToTableId(
                    obsRun.getId()), status);
        }
    }

    /**
     * Remove queue references of a set of OBs. This method was moved here as it
     * was duplicated in both RepositoryBrowserView and QueueItemsBView - it is
     * not shared by both. <br>
     * ChangeLog: nkornwei
     */
    public static void deleteQueuesReference(long[] selectedOBsId)
            throws ObjectIOException {
        Vector queueViewMgrVector = OTViewManager
                .getOTViewMgr()
                .getQueueViews();
        for (int i = 0; i < queueViewMgrVector.size(); i++) {
            QueueView mgr = (QueueView) queueViewMgrVector.elementAt(i);
            Queue queue = mgr.getQueue();
            boolean update = queue.isUpdate();
            mgr.getQueueItemsBView().removeOBs(selectedOBsId);
            queue.setUpdateAndFire(update);
        }
        // Delete reference into the DB
        QueueManager.removeOBsFromAllQueues(selectedOBsId);
    }

    /**
     * The action that applies the values to the widgets to the panels of the
     * dbbbrowser
     */
    public class ConfigValuesAction extends ActionSuperclass {
        GTreeNode colNames_;

        GTreeNode rowNames_;

        GTreeNode sortNames_;

        public ConfigValuesAction(JFrame repBrowserView, String name,
                GTreeNode colNames, GTreeNode rowNames, GTreeNode sortNames) {
            super(repBrowserView, name);
            colNames_ = colNames;
            rowNames_ = rowNames;
            sortNames_ = sortNames;
        }

        public final void actionPerformedImpl(final ActionEvent ev) {
            QueueItemsBView.colPanel_.setValueWidgets(colNames_);
            QueueItemsBView.sortPanel_.setValueWidgets(sortNames_);
        }
    };

    protected class ExportAction extends AbstractAction {
        ExportAction() {
            super("Export...");
        }

        public void actionPerformed(ActionEvent ev) {
        }
    }

    private class SummaryAction extends AbstractAction {
        SummaryAction() {
            super("Summary...");
        }

        public void actionPerformed(ActionEvent ev) {
        }
    }

    private class PrintAction extends AbstractAction {
        PrintAction() {
            super("Print Detailed area...");
        }

        public void actionPerformed(ActionEvent ev) {
        }
    }

    private class SelectAllAction extends AbstractAction {
        SelectAllAction() {
            super("Select All");
        }

        public void actionPerformed(ActionEvent ev) {
            ListSelectionModel lsm = getQueueItemsView()
                    .getView()
                    .getTable()
                    .getTable()
                    .getSelectionModel();
            lsm.addSelectionInterval(0, getQueueItemsView()
                    .getView()
                    .getTable()
                    .getRowCount() - 1);
        }

    }

    public BusinessObject getDisplayedObject() {
        return queue_;
    }

    /**
     * Close the queue data structures without saving them to the database
     */
    public void closeQueueWithoutSaving() {
        OTViewManager otMgr = OTViewManager.getOTViewMgr();
        int index = otMgr.getQueueViews().indexOf(this);
        forgetQueue(index);
    }

    public Queue reloadQueue() {

        OTViewManager otViewMgr = OTViewManager.getOTViewMgr();
        int index = otViewMgr.getQueueViews().indexOf(this);
        boolean reload = toBeUpdated();
        int oldQueueSize_ = 0;

        if (index != -1) {
            if (reload) {
                String message = "Queue '" + getQueue().getName()
                        + "' was modified. \nAll changes will be lost."
                        + " Are you sure?";
                int n = JOptionPane.showConfirmDialog(null, message, "Revert",
                        JOptionPane.YES_NO_OPTION);
                if (n == JOptionPane.NO_OPTION) {
                    return null;
                }
            } else {
                oldQueueSize_ = queue_.getSize();
                queue_.setUpdate(false);
            }
        }

        try {
            ObjectManager objMgr = ObjectManager.getObjectManager();
            if (queue_ instanceof MaskQueue) {
                objMgr.refreshBusObj(Media.DBASE, queue_.getId(),
                        MaskQueue.class);
            } else {
                objMgr.refreshBusObj(Media.DBASE, queue_.getId(), Queue.class);
            }

            if (!reload && oldQueueSize_ != queue_.getSize()) {
                String message = "Queue '"
                        + getQueue().getName()
                        + "' was modified by another user.\n\n"
                        + "The queue will be reverted with those modifications.";
                JOptionPane.showMessageDialog(null, message, "Warning",
                        JOptionPane.WARNING_MESSAGE);
            } else if (!reload) {
                /**
                 * No changes have been made the toBeUpdated() flag was not set
                 * and the size of the queue is the same as when the queue was
                 * opened.
                 */
                queue_.setUpdate(false);
                return null;
            }

            // setQueue(queue_);
            queue_.fireNameEvent();
            oldQueueSize_ = queue_.getSize();
            queue_.setSize(oldQueueSize_);
            queue_.setUpdate(false);
            setQueue(queue_);

        } catch (ObjectNotFoundException ex) {
            JOptionPane.showMessageDialog(null,
                    "An error occured while loading the Queue", "I/O Error",
                    JOptionPane.WARNING_MESSAGE);
            ex.printStackTrace();

        } catch (ObjectIOException ex) {
            JOptionPane.showMessageDialog(null,
                    "An error occured while loading the Queue", "I/O Error",
                    JOptionPane.WARNING_MESSAGE);
            ex.printStackTrace();

        } catch (SQLException ex) {
            JOptionPane.showMessageDialog(null,
                    "An error occured while loading the Queue", "I/O Error",
                    JOptionPane.WARNING_MESSAGE);
            ex.printStackTrace();
        }

        return queue_;
    }

    public boolean closeQueue() {
    	OTViewManager otMgr = OTViewManager.getOTViewMgr();
    	int index = otMgr.getQueueViews().indexOf(this);
    	updateQueue();
    	if (index != -1) {
    		if (toBeUpdated()) {
    			String message = "Queue "
    				+ queue_.getName()
    				+ " was modified,\nDo you want to save the changes before closing?";
    			int n = JOptionPane.showConfirmDialog(null, message,
    					"Save Changes", JOptionPane.YES_NO_CANCEL_OPTION);
    			ObjectManager objMgr = ObjectManager.getObjectManager();
    			if (n == JOptionPane.YES_OPTION) {
    				try {
    					objMgr.putBusObj(queue_);
    					queue_.setUpdate(false);
    					queue_.setIsNew(false);
    					getQueueItemsView().setOrderDirtyFlag(false);
    					forgetQueue(index);
    				} catch (ObjectNotFoundException e) {
    					JOptionPane.showMessageDialog(this,
    		                    "An error occured while saving the Queue", "I/O Error",
    		                    JOptionPane.ERROR_MESSAGE);
    				} catch (ObjectIOException e1) {
    					JOptionPane.showMessageDialog(this,
    		                    "An error occured while saving the Queue", "I/O Error",
    		                    JOptionPane.ERROR_MESSAGE);
    				}
    			}
    			if (n == JOptionPane.CANCEL_OPTION) {
    				return false;
    			}
    			if (n == JOptionPane.NO_OPTION) {    				   			
    				
    				forgetQueue(index);
    			}
    		} else {
    			forgetQueue(index);
    		}
    	}
    	return true;
    }

    /**
     * @param queueItemsView_
     *            The queueItemsView_ to set.
     */
    protected void setQueueItemsView(QueueItemsBView queueItemsView) {
        this.queueItemsView_ = queueItemsView;
    }

   

    /**
     * @return Returns the queueItemsView_.
     */
    public QueueItemsBView getQueueItemsView() {
        return queueItemsView_;
    }

    

}
