package org.eso.ohs.phase2.apps.p2pp;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.KeyboardFocusManager;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Collections;
import java.util.Enumeration;
import java.util.EventObject;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.CellEditor;
import javax.swing.DefaultCellEditor;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import javax.swing.text.DefaultCaret;
import javax.swing.text.Document;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.AbstractDbbTableModel;
import org.eso.ohs.core.docview.datatrans.AdaptorFactory;
import org.eso.ohs.core.docview.datatrans.ObjectAdaptor;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.SystemStatusEvent;
import org.eso.ohs.core.utilities.SystemStatusListener;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.dfs.Summary;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.ObjectSelectionEvent;
import org.eso.ohs.persistence.ObjectSelectionListener;
import org.eso.ohs.persistence.ObjectSelector;
import org.eso.ohs.persistence.ObjectTableEvent;
import org.eso.ohs.persistence.ObjectTableListener;
import org.eso.ohs.phase2.apps.p2pp.views.ObjectView;
/**
 * GUI class to display (a summary of) the contents of a Folder.
 * The OBs, ODs and Targets in the folder are displayed in separate
 * tables, each contained in a page of a TabbedPane panel.
 * This class listens for changes in the summary and updates the
 * appropriate tabel automatically.
 * 
 * <p>
 * $Id: FolderGridView.java,v 1.30 2005/12/15 19:48:15 tcanavan Exp $
 *
 * @author Bob Kemp
 * <br>ChangeLog: Tim Canavan
 * <br>ChangeLog: cmavros 22May02 Remove obsolete print statments and add log4j
 * 
 */

public class FolderGridView
    extends JPanel
    implements ObjectTableListener, ObjectContainer, ObjectSelector {

    /** Standard log channel */
    private static Logger stdlog_ =
        Logger.getLogger(FolderGridView.class);

    /** Preferred minimum size of a FolderGridView pane */
    private static final Dimension MIN_TABLE_SIZE = new Dimension(600, 300);
    /** Medium on which the folder exists. */
    private Media device_;

    /** Folder that is being displayed. */
    private Folder folder_;

    /** Tabbed pane containing the tables for the OBs, ODs, and Targets. */
    private JTabbedPane tabPane_;

    /** Support for event handling */
    private EventListenerList listenerList_;

    /** Hashtable of PaneInfo objects: one per displayed tab pane. */
    private Hashtable paneInfo_ = new Hashtable();

    private SyncListener syncListener_;

    /**
     * Ancestor control for the JTable
     */

    /** Structure describing the tables, models, etc for a given tab pane */
    private static class PaneInfo {
        private Ancestor ancestor_;
        public String title;
        public SummaryTable table;
        public Class objType;
        public SummaryModel model;
        public ListListener listener;
    }

    /**
     * Adds a pane of the given type to the FolderGridView tabPane,
     * including a JTable, model, etc.
     * 
     * @param title	title for the tabPane tab.
     * @param objType	the type of object to be described in the new table.
     * 
     */
    public void addPane(Class objType) {
        PaneInfo pi = new PaneInfo();
        boolean isReadOnly = device_.equals(Media.DBASE);

        pi.title = BusinessObject.getDisplayName(objType);
        pi.objType = objType;
        SummaryModel model = new SummaryModel(objType, isReadOnly);
        pi.model = model;
        FolderGridTableSorter sorter = new FolderGridTableSorter(model);
        pi.table = new SummaryTable(sorter);
        JTableHeader th = pi.table.getTableHeader();
        th.addMouseListener(new ColumnSortListener());
        //sorter.addMouseListenerToHeaderInTable(pi.table);   
        // Use a scrollbar, in case there are many columns. 
        pi.table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        if (pi.ancestor_ == null) {
            pi.ancestor_ = new Ancestor(pi.table);
            pi.table.addAncestorListener(pi.ancestor_);
        }
        // Install a mouse listener in the TableHeader as the sorter UI.      
        pi.listener = new ListListener(pi.model, objType);

        pi.table.setRowSelectionAllowed(true);
        // Java1.4 port related to Java bug #4347815
        // pi.table.setCellSelectionEnabled(false);
        pi.table.setColumnSelectionAllowed(false);
        pi.table.setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);

        pi.table.setDefaultRenderer(Object.class, new Renderer(objType));
        pi.table.setDefaultEditor(
            Object.class,
            new Editor(device_, objType, pi.table));

        ListSelectionModel rowSM = pi.table.getSelectionModel();
        rowSM.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

        JScrollPane jsp = new JScrollPane(pi.table);
        jsp.setBorder(BorderFactory.createRaisedBevelBorder());
        tabPane_.addTab(pi.title, jsp);

        // Store the pane info under _both_ objType and title
        paneInfo_.put(objType, pi);
        paneInfo_.put(pi.title, pi);
        //stdlog_.debug("Model Row Count " + pi.model.getRowCount());
    }

    /**
     * Default constructor for a Folder (summary information) object.
     * 
     * @param device	medium on which the folder exists.
     * @param folder	folder summary to be displayed.
     */
    public FolderGridView(Media device, Folder folder) {
        folder_ = folder;
        device_ = device;

        tabPane_ = new JTabbedPane();

        Class[] objTypes = AppConfig.getAppConfig().getDisplayedObjectTypes();

        for (int j = 0; j < objTypes.length; j++) {
            addPane(objTypes[j]);
        }

        setLayout(new BorderLayout());
        add(tabPane_, BorderLayout.CENTER);

        tabPane_.addChangeListener(new TabChangeListener());

        listenerList_ = new EventListenerList();
        syncListener_ = new SyncListener(paneInfo_);

        setFolder(folder);
    }

    public void addTabChangeListener(ChangeListener listener) {
        tabPane_.addChangeListener(listener);
    }

    /**
     * Hook for the window being activated.
     * We use it to calculate the viewport size, which can only be
     * done once the the window is live, ie after addNotify()
     * has been called.  Also add a SystemStatusListener
     */
    public void addNotify() {
        super.addNotify();

        calculateInitialViewPortSizes();
        P2PP.addSystemStatusListener(syncListener_);
    }

    /**
     * Hook for the window being deactivated.
     * We use it to remove the SystemStatusListener
     */
    public void removeNotify() {
        super.removeNotify();

        P2PP.removeSystemStatusListener(syncListener_);
    }

    /**
     * Calculates the size of the JTable viewport and sets the
     * preferred size appropriately.  It should not be smaller
     * than a predefined minimum size but cannot occupy more than
     * 60% of the screen width or height.
     */
    private void calculateInitialViewPortSizes() {
        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        Enumeration enum = paneInfo_.keys();
        Rectangle r1 = new Rectangle();
        Rectangle r2 = new Rectangle();

        while (enum.hasMoreElements()) {
            Object obj = enum.nextElement();

            if (obj instanceof Class) {
                Class objType = (Class) obj;
                PaneInfo pi = (PaneInfo) paneInfo_.get(objType);

                // calculate _initial_ size for scrollpane.
                // Using Rectangle's is easier than direct.

                // how big it would like to be...
                r1.setSize(pi.table.getPreferredSize());

                // make r1 at least as big as min size.
                r2.setSize(MIN_TABLE_SIZE);
                r1 = r1.union(r2);

                // make r1 at most 60% screen width/height.
                r2.setSize(
                    new Dimension(
                        6 * screen.width / 10,
                        6 * screen.height / 10));
                r1 = r1.intersection(r2);

                pi.table.setPreferredScrollableViewportSize(r1.getSize());
            }
        }
    }

    /**
     * Utility method to call stopCellEditing() on the table in the currently
     * visible tabpane.
     */
    public void stopEditing() {
        PaneInfo pi = (PaneInfo) paneInfo_.get(getDisplayedObjectClass());
        CellEditor ed = pi.table.getCellEditor();

        if (ed != null) {
            ed.stopCellEditing();
        }
    }

    /**
     * Select a new tabPane panel to be displayed.
     * 
     * @param tabTitle	label of the tabbedPane panel to display.
     */
    public void setDisplayedObjectClass(Class objType) {
        PaneInfo pi = (PaneInfo) paneInfo_.get(objType);

        if (pi != null) {
            int index = tabPane_.indexOfTab(pi.title);

            if (index >= 0) {
                tabPane_.setSelectedIndex(index);
            }
        }
    }

    /**
     * Select a new tabPane panel to be displayed.
     * 
     * @param tabTitle	label of the tabbedPane panel to display.
     */
    public Class getDisplayedObjectClass() {
        int index = tabPane_.getSelectedIndex();
        String title = tabPane_.getTitleAt(index);
        PaneInfo pi = (PaneInfo) paneInfo_.get(title);
        return pi.objType;
    }

    /**
     * Returns the currently displayed folder.
     * 
     * @return	the folder whose contents are displayed
     */
    public Folder getFolder() {
        return folder_;
    }

    /**
     * Returns the external medium on which the displayed objects reside.
     * 
     * @return	medium whose contents are displayed
     */
    public Media getMedia() {
        return device_;
    }

    /**
     * Select a new folder to be displayed.
     * 
     * @param folder	The folder whose contents are to be displayed.
     */
    public void setFolder(Folder folder) {
        if (folder_ != null) {
            folder_.removeObjectTableListener(this);
        }

        folder_ = folder;

        if (folder_ != null) {
            folder_.addObjectTableListener(this);
        }

        Enumeration enum = paneInfo_.keys();

        while (enum.hasMoreElements()) {
            Object obj = enum.nextElement();

            if (obj instanceof Class) {
                Class objType = (Class) obj;
                PaneInfo pi = (PaneInfo) paneInfo_.get(objType);

                CellEditor ed = pi.table.getCellEditor();
                if (ed != null) {
                    ed.stopCellEditing();
                }

                ListSelectionModel lsm = pi.table.getSelectionModel();
                lsm.removeListSelectionListener(pi.listener);

                pi.model.setFolder(folder_);

                if (pi.table.getColumnCount() == pi.model.getColumnCount()) {
                    TableColumn deletedColumn = pi.table.getColumn("Id");
                    pi.table.removeColumn(deletedColumn);
                }

                pi.table.sizeColumnsToFit(-1);

                lsm.addListSelectionListener(pi.listener);

                makeDefaultSelection(pi);
            }
        }
        PaneInfo pi = (PaneInfo) paneInfo_.get(ObservationBlock.class);
        makeDefaultSelection(pi);
        // MainWindow.getView().setDisplayedObjClass(getDisplayedObjectClass());
    }

    /**
     * Selects the row containing the given object or the first
     * row if not found.
     * 
     * @param bo	Object to be selected
     */
    public void makeSelection(StorableObject sobj) {
        if (sobj == null) {
            return; // XXX no object?  Should never happen
        }

        makeSelection(sobj.getId(), sobj.getClass());
    }

    /**
     * Selects the row containing the object with the given id
     * or the first row if not found.
     * 
     * @param id	id of object to be selected
     * @param objType	type of the object to be selected
     */
    private void makeSelection(long id, Class objType) {
        PaneInfo pi = (PaneInfo) paneInfo_.get(objType);

        if (pi == null) {
            return; // there's no tabpane for this object type
        }

        for (int row = pi.model.getRowCount() - 1; row >= 0; row--) {
            StorableObject so = pi.model.getObjectAt(row);

            if (so.getId() == id) {
                pi.table.clearSelection();
                pi.table.addRowSelectionInterval(row, row);
                return;
            }
        }
        makeDefaultSelection(pi);
    }

    /**
     * Selects the row containing the object with the given id
     * or the first row if not found.                fireObjectSelectionEvent(
                       device_,
                       id,
                       pi.objType,
                       false,
                       true,
                       pi.model.getRowCount());
     * 
     * @param id	id of object to be selected
     * @param objType	type of the object to be selected
     */
    private void makeDefaultSelection(PaneInfo pi) {
        int numRows = pi.model.getRowCount();
        pi.table.clearSelection();
        if (pi.table.getRowCount() > 0) {
            pi.table.setRowSelectionInterval(0, 0);

        } else {
            fireObjectSelectionEvent(
                device_,
                ObjectSelectionEvent.NO_OBJECT,
                pi.objType,
                false,
                true,
                pi.model.getRowCount());
        }
    }

    /**
     * A new business object was added to the folder summary information.
     * Event handler implementing ObjectTableListener.
     * 
     * @param ev	description of affected object.
     */
    public void tableObjectAdded(ObjectTableEvent ev) {
        long id = ev.getObjectId();
        Class objType = ev.getObjectClass();

        makeSelection(id, objType);
    }

    /**
     * A business object in the folder summary information was changed.
     * Event handler implementing ObjectTableListener.
     * 
     * @param ev	description of affected object.
     */
    public void tableObjectChanged(ObjectTableEvent ev) {
    }

    /**
     * A business object was removed from the folder summary information.
     * Event handler implementing ObjectTableListener.
     * 
     * @param ev	description of affected object.
     */
    public void tableObjectRemoved(ObjectTableEvent ev) {
        long id = ev.getObjectId();
        Class objType = ev.getObjectClass();

        makeSelection(id, objType);
    }

    /**
     * Selects all BusinessObjects stored in the table for type objType.
     * Also stops editing of any cell in that table.
     * 
     * @param objType	type of objects to be selected
     */
    public void selectAllObjects() {
        PaneInfo pi = (PaneInfo) paneInfo_.get(getDisplayedObjectClass());
        ListSelectionModel lsm = pi.table.getSelectionModel();
        CellEditor ed = pi.table.getCellEditor();

        if (ed != null) {
            ed.stopCellEditing();
        }
        lsm.addSelectionInterval(0, pi.model.getRowCount() - 1);
    }

    /**
     * Selects the BusinessObject with given id that is stored in the table
     * for type objType.
     * 
     * @param id	id of object to be selected
     * @param objType	type of objects to be selected
     */
    public void selectObject(Class objType, long id) {
        PaneInfo pi = (PaneInfo) paneInfo_.get(objType);
        ListSelectionModel lsm = pi.table.getSelectionModel();
        StorableObject sobj;

        for (int j = pi.model.getRowCount() - 1; j >= 0; j--) {
            sobj = pi.model.getObjectAt(j);
            if (sobj.getId() == id) {
                lsm.addSelectionInterval(j, j);
                break;
            }
        }
    }

    /**
     * Deselects the BusinessObject with given id that is stored in the table
     * for type objType.  Useful for operations that work on multiple objects
     * but require certain crteria to be satisfied by the objects.  Those
     * not meeting the criteria, can be deselected as feedback to the user.
     * 
     * @param id	id of object to be selected
     * @param objType	type of objects to be selected
     */
    public void deselectObject(Class objType, long id) {
        PaneInfo pi = (PaneInfo) paneInfo_.get(objType);
        ListSelectionModel lsm = pi.table.getSelectionModel();
        StorableObject sobj;

        for (int j = pi.model.getRowCount() - 1; j >= 0; j--) {
            sobj = pi.model.getObjectAt(j);
            if (sobj.getId() == id) {
                lsm.removeSelectionInterval(j, j);
                break;
            }
        }
    }

    /**
     * Returns the id of the business object selected in the
     * given table, or null if none is selected.
     * 
     * @param table	table of business object summaries.
     * 
     * @return		the id of the selected object wrapped in a Long,
     *			or null if no object has been selected.
     */
    public Long[] getAllSelected() {
        PaneInfo pi = (PaneInfo) paneInfo_.get(getDisplayedObjectClass());
        ListSelectionModel lsm = pi.table.getSelectionModel();
        int minSelectedRow = lsm.getMinSelectionIndex();
        int maxSelectedRow = lsm.getMaxSelectionIndex();
        int rowCount = pi.table.getRowCount();
        Vector objVec = new Vector();
        StorableObject sobj;

        if (minSelectedRow < 0)
            minSelectedRow = 0;

        if (maxSelectedRow >= rowCount)
            maxSelectedRow = rowCount - 1;

        for (int j = minSelectedRow; j <= maxSelectedRow; j++) {
            if (lsm.isSelectedIndex(j)) {
                sobj = pi.model.getObjectAt(j);

                if (sobj != null) {
                    objVec.addElement(new Long(sobj.getId()));
                }
            }
        }

        Long[] objIds = new Long[objVec.size()];
        objVec.copyInto(objIds);

        return objIds;
    }

    public Long[] getAll() {
        Vector objVec = new Vector();
        StorableObject sobj;
        PaneInfo pi = (PaneInfo) paneInfo_.get(getDisplayedObjectClass());
        ListSelectionModel lsm = pi.table.getSelectionModel();
        int rowCount = pi.table.getRowCount();

        for (int j = 0; j < rowCount; j++) {
            sobj = pi.model.getObjectAt(j);
            if (sobj != null) {
                objVec.addElement(new Long(sobj.getId()));
            }
        }
        Long[] objIds = new Long[objVec.size()];
        objVec.copyInto(objIds);
        return objIds;
    }

    /**
     * Returns the id of the business object selected in the
     * given table, or null if none is selected.
     * 
     * @param table	table of business object summaries.
     * 
     * @return		the id of the selected object wrapped in a Long,
     *			or null if no object has been selected.
     */
    public Long getSelected() {
        return getSelectedInternal(getDisplayedObjectClass());
    }

    public Long getSelectedObsBlocks() {
        return getSelectedInternal(ObservationBlock.class);
    }

    public Long getSelectedCalBlocks() {
        return getSelectedInternal(CalibrationBlock.class);
    }

    public Long getSelectedInternal(Class objClass) {
        PaneInfo pi = (PaneInfo) paneInfo_.get(objClass);
        ListSelectionModel lsm = pi.table.getSelectionModel();
        int selectedRow = lsm.getMinSelectionIndex();
        StorableObject sobj;

        if (selectedRow >= 0 && selectedRow < pi.table.getRowCount()) {
            sobj = pi.model.getObjectAt(selectedRow);

            if (sobj != null) {
                return new Long(sobj.getId());
            }
        }
        return null;
    }

    public BusinessObject getBOByName(String name,String type)
        throws ObjectIOException, ObjectNotFoundException {
        Persistence objMgr = Persistence.getInstance();
        Class c = CalibrationBlock.class;
        if(type.equals("O")) {
            c = ObservationBlock.class;
        }
        PaneInfo pi = (PaneInfo) paneInfo_.get(c);
        for (int i = 0; i < pi.model.getRowCount(); i++) {
            StorableObject sobj = pi.model.getObjectAt(i);
            stdlog_.debug("getBOByName StorableObject: " + sobj.getName());
            if (sobj.getName().equals(name)) {
                Long busId = (Long) pi.model.getValueAt(i, 0);
                return (BusinessObject) objMgr.getBusObj(
                    getMedia(),
                    busId.longValue(),
                    c);

            }
        }
        return null;
    }

    /**
     * Returns the business object selected in the visible table,
     * or null if none is selected.
     * 
     * @return		the id of the selected object wrapped in a Long,
     *			or null if no object has been selected.
     */
    public BusinessObject getDisplayedObject() {
        Persistence objMgr = Persistence.getInstance();
        Class objType = getDisplayedObjectClass();
        Long idLong = getSelected();
        BusinessObject bo = null;

        try {
            if (idLong != null) {
                bo =
                    (BusinessObject) objMgr.getBusObj(
                        getMedia(),
                        idLong.longValue(),
                        objType);
            }
        } catch (ObjectIOException e) {
            stdlog_.error(e);
        } catch (ObjectNotFoundException e) {
            stdlog_.error(e);
        }

        return bo;
    }

    /**
     * Adds a new listener to be notified of selection changes.
     * 
     * @param lnr the listener
     */
    public void addObjectSelectionListener(ObjectSelectionListener lnr) {
        listenerList_.add(ObjectSelectionListener.class, lnr);
    }

    /**
     * Removes a new listener, which will no longer be notified of
     * selection changes.
     * 
     * @param lnr the listener
     */
    public void removeObjectSelectionListener(ObjectSelectionListener lnr) {
        listenerList_.remove(ObjectSelectionListener.class, lnr);
    }

    /**
     * Informs listeners of a selection change.
     */
    protected void fireObjectSelectionEvent(
        Media device,
        long id,
        Class objType,
        boolean forceShow,
        boolean forceToFront,
        int rowCount) {
        Object[] listeners = listenerList_.getListenerList();
        ObjectSelectionListener lnr = null;
        ObjectSelectionEvent selnEvt =
            new ObjectSelectionEvent(
                this,
                device_,
                id,
                objType,
                forceShow,
                forceToFront,
                rowCount);

        for (int i = 0; i < listeners.length; i += 2) {
            lnr = (ObjectSelectionListener) listeners[i + 1];
            lnr.objectSelected(selnEvt);
        }
    }

    /**
     * Causes the selected object to be displayed in a ViewContainer
     * and references to it to be shown in a LinksView.
     */
    private class ListListener implements ListSelectionListener {
        SummaryModel model_;
        Class objType_;

        public ListListener(SummaryModel model, Class objType) {
            model_ = model;
            objType_ = objType;
        }

        public void valueChanged(ListSelectionEvent ev) {
            if (ev.getValueIsAdjusting()) {
                return;
            }

            ListSelectionModel lsm = (ListSelectionModel) ev.getSource();
            int selectedRow = lsm.getMinSelectionIndex();
            long id = ObjectSelectionEvent.NO_OBJECT;
            StorableObject sobj = null;
            if (!objType_.equals(getDisplayedObjectClass()))
                return;
            if (!lsm.isSelectionEmpty()) {
                if (selectedRow >= 0 && selectedRow < model_.getRowCount()) {
                    sobj = model_.getObjectAt(selectedRow);
                    //stdlog_.debug("We are here !!!!!!!!!!!!!!");
                    id = sobj.getId();
                }
            }            
            fireObjectSelectionEvent(
                device_,
                id,
                objType_,
                false,
                objType_.equals(getDisplayedObjectClass()),
                model_.getRowCount());
        }
    }

    /**
     * Listener for new TabbedPane selections.
     * Fires an ObjectSelectionEvent when a different tabPane is selected.
     */
    private class TabChangeListener implements ChangeListener {

        /**
         * Handler for TabbedPane selection events.
         * Fires an ObjectSelectionEvent when a different tabPane is selected.
         */
        public void stateChanged(ChangeEvent ev) {
            String tabTitle = tabPane_.getTitleAt(tabPane_.getSelectedIndex());
            PaneInfo pi = (PaneInfo) paneInfo_.get(tabTitle);

            if (pi != null) {
                CellEditor ed = pi.table.getCellEditor();
                if (ed != null) {
                    ed.stopCellEditing();
                }

                Long idLong = getSelected();
                fireObjectSelectionEvent(
                    device_,
                    (idLong != null
                        ? idLong.longValue()
                        : ObjectSelectionEvent.NO_OBJECT),
                    pi.objType,
                    false,
                    true,
                    pi.model.getRowCount());
            }
        }
    }

    /**
     * This class handles rendering of the value for every cell on the grid.
     */
    private class Renderer implements TableCellRenderer {
        private JLabel label_ = new DefaultTableCellRenderer();
        private String[] colProps_;
        private ObjectAdaptor[] propMods_;

        public Renderer(Class objType) {
            label_.setBackground(Color.blue.darker());
            label_.setFont((Font) UIManager.get("Table.font"));

            colProps_ =
                AppConfig.getAppConfig().folderColumnProperties(objType);
            propMods_ = new ObjectAdaptor[colProps_.length];

            for (int j = 0; j < colProps_.length; j++) {
                propMods_[j] = AdaptorFactory.getAdaptor(objType, colProps_[j]);
            }
        }

        /**
        * Return the appropriate rendering component for a given cell.
        * 
        * @param table JTable reference
        * @param value value of cell
        * @param isSelected selected value
        * @param hasFocus focus
        * @param row row index
        * @param column column index    
        */
        public Component getTableCellRendererComponent(
            JTable table,
            Object value,
            boolean isSelected,
            boolean hasFocus,
            int row,
            int column) {

            label_.setOpaque(isSelected);

            if (isSelected && hasFocus) {
                label_.setBorder(BorderFactory.createLineBorder(Color.yellow));
                label_.setForeground(Color.white);
            } else if (isSelected) {
                label_.setForeground(Color.white);
                label_.setBorder(null);
            } else {
                label_.setForeground(Color.black);
                label_.setBorder(null);
            }

            String text = "";

            if (value != null) {
                ObjectAdaptor oa =
                    propMods_[table.convertColumnIndexToModel(column)];
                text = oa.fromMetaLevelObject(value);
            }
            FolderGridTableSorter model =
                (FolderGridTableSorter) table.getModel();
            StorableObject obj = model.getObjectAt(row);
            Summary summ = (Summary) obj;
            if(column == 0) {
                boolean allValid = folder_.allValid(summ);
                if (summ.isCheckedIn())
                    label_.setIcon(ObjectView.locked_);
                else if (!allValid)
                    label_.setIcon(ObjectView.invalidIcon_);
                else
                    label_.setIcon(null);
                           	
            } else {
            	label_.setIcon(null);
            }            
            label_.setText(text); 
            TableColumn tcol =
                table.getTableHeader().getColumnModel().getColumn(column);
            String head = (String) tcol.getHeaderValue();
            if (head.equals("FindingCharts")) {
                label_.setToolTipText(text);
            } else {
                label_.setToolTipText(null);
            }
            return label_;

        }


    }

    /**
     * Subclass of JTextField with overridden isManagingFocus()
     * to avoid problems with tabbing, etc.
     */
    private static class EditorJTextField extends JTextField {
        public EditorJTextField() {
            super();
            setFocusTraversalKeys(
                KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS,
                Collections.EMPTY_SET);
            setFocusTraversalKeys(
                KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS,
                Collections.EMPTY_SET);
            setFocusTraversalKeys(
                KeyboardFocusManager.UP_CYCLE_TRAVERSAL_KEYS,
                Collections.EMPTY_SET);
        }

        /**
         * Remove for java 1.4 port - see setFocusTraversalKeys above
        public boolean isManagingFocus() {
           return true; 
        }
        */

    }

    /**
     * This class handles construction of a cell editors.
     */
    private static class Editor extends DefaultCellEditor {

        /** Property names for each column of the table, even hidden columns */
        private String[] colProps_;

        /** Property modifiers for each column of the table,
         * even hidden columns */
        private ObjectAdaptor[] propMods_;

        /** Type of object stored in this table. */
        private Class cls_;

        /** Media containing the objects in the table */
        private Media device_;

        /** Row in model corresponding to the cell that is being edited */
        private int modelRow_;

        /** Column in model corresponding to the cell that is being edited */
        private int modelColumn_;

        /** model for the table that is being edited */
        private FolderGridTableSorter model_;

        /** table that is being edited */
        private SummaryTable table_;

        /** JTextField that the user interacts with while editing the cell */
        private EditorJTextField tf_;

        /**
         * Constructor.
         * 
         * @param   device	Media containing the objects in the table
         * @param   objType	type of object that is to be edited
         */
        public Editor(Media device, Class objType, SummaryTable table) {
            super((JTextField) new EditorJTextField());
            cls_ = objType;
            device_ = device;
            table_ = table;

            tf_ = (EditorJTextField) editorComponent;
            tf_.setCaret(new FolderDefaultCaret(tf_));
            addKeyListener();
            addMouseListener();
            colProps_ =
                AppConfig.getAppConfig().folderColumnProperties(objType);
            propMods_ = new ObjectAdaptor[colProps_.length];

            for (int j = 0; j < colProps_.length; j++) {
                propMods_[j] = AdaptorFactory.getAdaptor(objType, colProps_[j]);
            }

        }

        /**
         * Method to add a key listener to the JTextField.
         * The key listener passes tab, backtab, enter and escape
         * to the table.
         */
        private void addKeyListener() {
            tf_.addKeyListener(new KeyListener() {
                public void keyPressed(KeyEvent evt) {
                    if (evt.getKeyCode() == KeyEvent.VK_TAB
                        || KeyEvent.getKeyText(evt.getKeyCode()).equals(
                            "Shift-Tab")
                        || evt.getKeyCode() == KeyEvent.VK_ENTER
                        || evt.getKeyCode() == KeyEvent.VK_ESCAPE) {
                        table_.processKeyEvent(evt);
                    }
                }
                public void keyReleased(KeyEvent evt) {
                }
                public void keyTyped(KeyEvent evt) {
                }
            });
        }

        private void addMouseListener() {
            tf_.addMouseListener(new MouseListener() {

                /**
                 * Mouse clicked event 
                 * @param e mouse clicked event
                 */

                public void mouseClicked(MouseEvent e) {
                    //stdlog_.debug("Mouse Clicked");
                }

                /** 
                 * Mouse pressed event
                 * @param e mouse pressed event
                 */

                public void mousePressed(MouseEvent e) {
                    //stdlog_.debug("Mouse Pressed");
                }

                /**
                 * Mouse released event
                 * @param e mouse released event
                 */
                public void mouseReleased(MouseEvent e) {
                    //stdlog_.debug("Mouse Released");
                }

                /**  
                 * Mouse entered event
                 * @param e mouse entered event
                 */

                public void mouseEntered(MouseEvent e) {
                    //stdlog_.debug("Mouse Entered");          
                    //                           table_.stopEditing(); 
                }

                /**
                 * Mouse exited event
                 * @param e mouse exited event
                 */

                public void mouseExited(MouseEvent e) {
                    //stdlog_.debug("Mouse Exited");
                    table_.stopEditing();
                }
            });
        }

        /**
         * Modified Caret class to select all of the JTextField if the
         * mouse is double-clicked.
         */
        private static class FolderDefaultCaret extends DefaultCaret {
            /** JTextField corresponding to this caret */
            private JTextField textField_;

            /**
             * Constructor
             * 
             * @param	field	JTextField on which this caret operates.
             */
            public FolderDefaultCaret(JTextField field) {
                super();
                textField_ = field;
            }

            /**
             * Mouse click event handler.
             * 
             * @param	e	mouse event specifying (eg) how many clicks.
             */
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() >= 2) {
                    textField_.selectAll();
                }
            }

        }

        /**
        * Return the appropriate rendering component for a given cell.
        * 
        * @param table JTable reference
        * @param value value of cell
        * @param isSelected selected value
        * @param row row index
        * @param column column index    
        */
        public Component getTableCellEditorComponent(
            JTable table,
            Object value,
            boolean isSelected,
            int row,
            int column) {
            tf_ =
                (EditorJTextField) super.getTableCellEditorComponent(
                    table,
                    value,
                    isSelected,
                    row,
                    column);

            tf_.setOpaque(isSelected);
            if (isSelected) {
                tf_.setBorder(BorderFactory.createLineBorder(Color.red, 1));
                tf_.setForeground(Color.white);
                tf_.setBackground(Color.blue);
            } else {
                tf_.setForeground(Color.black);
                tf_.setBackground(Color.blue.darker());
            }
            tf_.setCaretColor(Color.red);

            String text = "";

            if (value != null) {
                ObjectAdaptor oa =
                    propMods_[table.convertColumnIndexToModel(column)];
                text = oa.fromMetaLevelObject(value);
            }

            modelRow_ = row;
            modelColumn_ = table.convertColumnIndexToModel(column);
            model_ = (FolderGridTableSorter) table.getModel();

            Document doc =
                DocumentFactory.getDocument(cls_, colProps_[modelColumn_]);

            if (doc != null) {
                tf_.setDocument(doc);
            }

            tf_.setText(text);
            tf_.selectAll();
            return tf_;
        }

        /**
         * If any cell in the associated JTable is being edited, that
         * editing is stopped and any changes are saved.
         * 
         * @return	true if editing was successfully stopped,
         *              false otherwise.
         */
        public boolean stopCellEditing() {
            String value = (String) getCellEditorValue();
            ObjectAdaptor oa = propMods_[modelColumn_];
            Summary summ;

            if (modelRow_ >= model_.getRowCount()) {
                return true;
            }

            try {
                summ = (Summary) model_.getObjectAt(modelRow_);
                Persistence objMgr = Persistence.getInstance();
                StorableObject sobj =
                    objMgr.getBusObj(
                        device_,
                        summ.getId(),
                        summ.getObjectClass());

                oa.setObject((BusinessObject) sobj);
                Object oldValue = oa.getRawValue();

                if (oldValue instanceof StorableObject) {
                    ((StorableObject) oldValue).setName(value);
                } else {
                    oa.setReadWrite(true);
                    try {
                        oa.setValue(value);
                    } catch (IllegalArgumentException e) {
                        // illegal value could not be saved -- ignore
                    }
                }
            } catch (ObjectNotFoundException e) {
                stdlog_.error(e);
                return false;
            } catch (ObjectIOException e) {
                stdlog_.error(e);
                return false;
            }

            return super.stopCellEditing();
        }

    }

    /**
     * Subclass of JTable with bug fix for editCellAt() and
     * processKeyEvent() as a public method.
     */
    public class SummaryTable extends JTable {

        /**
         * Constructor.
         * 
         * @param model		model for this table.
         */
        public SummaryTable(TableModel model) {
            super(model);
        }

        /**
         * Method to start editing of the specified table cell.
         * Overridden to add a bug fix which caused the editing
         * to be very slow to happen because the component was not redrawn.
         * 
         * @param row		row of the cell to be edited
         * @param column	column of the cell to be edited
         * @param e		null or the event that requested editing.
         * @return		true if started editing, false otherwise.
         */
        public boolean editCellAt(int row, int column, EventObject e) {
            boolean edit = super.editCellAt(row, column, e);
            if (edit) {
                editorComp.repaint();
            }
            return edit;
        }

        /**
         * Processes a KeyEvent: overridden to make it public for use by
         * (eg) KeyListener classes.
         * 
         * @param e		a KeyEvent
         */
        public void processKeyEvent(KeyEvent e) {
            super.processKeyEvent(e);
        }

        public void stopEditing() {
            CellEditor ed = getCellEditor();
            if (ed != null) {
                ed.stopCellEditing();
            }
        }
    }

    /**
     * P2PP-specific table model: data corresponds to a subset of the
     * summary info in a Folder.
     */
    public static class SummaryModel
        extends AbstractDbbTableModel
        implements ObjectTableListener {
        /** Folder containing the summarized objects */
        private Folder folder_;

        /** Contents of the model: a vector of Summary objects */
        private Vector contents_;

        /** Type of business object stored in this model */
        private Class objType_;

        /** Property names for the columns stored of this model */
        private String[] colProps_;

        /** Display names for the columns stored of this model */
        private String[] colNames_;

        /** Should all entries in the table be readonly? */
        private boolean isReadOnly_;

        /**
         * Default constructor.
         */
        public SummaryModel() {
        }

        /**
         * Constructor.
         * 
         * @param objType	type of object to be stored in the table.
         * @param isReadOnly	should all entries in the table be readonly?
         */
        public SummaryModel(Class objType, boolean isReadOnly) {
            objType_ = objType;
            contents_ = new Vector();
            colProps_ =
                AppConfig.getAppConfig().folderColumnProperties(objType_);
            colNames_ = AppConfig.getAppConfig().folderColumnNames(objType_);
            isReadOnly_ = isReadOnly;
        }

        /**
         * Reorder the summary objects stored in this model.
         * 
         * @param objType	type of object to be stored in the table.
         * @param isReadOnly	should all entries in the table be readonly?
         */
        public void reorderContents(int indexes[]) {
            Vector contents = new Vector();
            for (int i = 0; i < indexes.length; i++) {
                Summary summ = (Summary) contents_.elementAt(indexes[i]);
                contents.addElement(summ);
            }
            contents_.removeAllElements();
            contents_ = contents;
            fireTableDataChanged();
        }

        /**
         * Select a new folder for the model.
         * 
         * @param folder	the folder to be displayed.
         */
        public void setFolder(Folder folder) {
            if (folder_ != null) {
                folder_.removeObjectTableListener(this);
            }
            folder_ = folder;

            contents_.setSize(0);
            if (folder_ != null) {
                Summary[] summ = folder_.getSummaryInformation();

                for (int j = 0; j < summ.length; j++) {
                    //stdlog_.debug(summ[j].getObjectClass());
                    //stdlog_.debug("Object Type " + objType_);	 
                    if (objType_.equals(summ[j].getObjectClass())) {
                        contents_.addElement(summ[j]);
                    }
                }

                folder_.addObjectTableListener(this);
            }

            fireTableDataChanged();
        }

        /**
         * Returns the value of the property displayed in the given row
         * and column.  The row number determines the summary object and
         * the column determines the property.
         * 
         * <p>
         * NB: model columns may not be the same as table columns.
         * @see JTable#convertColumnIndexToModel
         * 
         * @param	row	row of the model
         * @param	column	column of the model
         * 
         * @return	current value of the object's property
         */
        public Object getValueAt(int row, int column) {
            Summary sum = (Summary) contents_.elementAt(row);
            return sum.getPropertyValue(colProps_[column]);
        }

        /**
         * Returns the summary object displayed in the given row.
         * 
         * @param	row	row of the model
         * @return	current value of the selected summary object
         */
        public StorableObject getObjectAt(int row) {
            return (StorableObject) contents_.elementAt(row);
        }

        /**
         * Returns the summary object displayed in the given row.
         * 
         * @param	row	row of the model
         * @return	current value of the selected summary object
         */
        public String getColumnName(int col) {
            return colNames_[col];
        }

        /**
         * Returns the number of columns displayed in the model,
         * including non-displayed columns.
         * 
         * @return	number of columns in the table
         */
        public int getColumnCount() {
            String[] colProps =
                AppConfig.getAppConfig().folderColumnProperties(objType_);
            return colProps.length;
        }

        /**
         * Returns the number of rows displayed in the model, ie
         * the number of summary objects in the model.
         * 
         * @return	number of summary objects in the table
         */
        public int getRowCount() {
            return contents_.size();
        }

        /**
         * Editable if table is not read-only and col is editable
         * and BO is not checked in.
         * 
         * @return	true if cell is editable, false otherwise
         */
        public boolean isCellEditable(int row, int column) {
            if (isReadOnly_) { // Whole table is read-only?
                return false;
            }

            if (row >= getRowCount() || column > getColumnCount()) {
                return false;
            }

            AppConfig cfg = AppConfig.getAppConfig();

            if (!cfg.isEditableProperty(objType_, colProps_[column])) {
                return false;
            }
            return !((Summary) getObjectAt(row)).isCheckedIn();
        }

        /**
         * Handler for events due to addition of an object to the folder summary
         * 
         * @param ev	description of affected object.
         */
        public void tableObjectAdded(ObjectTableEvent ev) {
            long id = ev.getObjectId();
            Class addedObjType = ev.getObjectClass();

            if (!addedObjType.equals(objType_)) {
                return;
            }

            Summary sum = folder_.findSummary(id);

            contents_.insertElementAt(sum, 0);
            fireTableRowsInserted(contents_.size(), contents_.size());
        }

        /**
         * Handler for events due to changes to an object to the folder summary
         * 
         * @param ev	description of affected object.
         */
        public void tableObjectChanged(ObjectTableEvent ev) {
            long id = ev.getObjectId();
            Class objType = ev.getObjectClass();
            for (int row = contents_.size() - 1; row >= 0; row--) {
                StorableObject sobj = (StorableObject) contents_.elementAt(row);
                if (Config.getCfg().uniqueToTableId(sobj.getId())
                    == Config.getCfg().uniqueToTableId(id)) {
                    fireTableRowsUpdated(row, row);
                    return;
                }
            }

        }

        /**
         * Handler for events due to removal of an object to the folder summary
         * 
         * @param ev	description of affected object.
         */
        public void tableObjectRemoved(ObjectTableEvent ev) {
            long id = ev.getObjectId();
            Class objType = ev.getObjectClass();

            if (!objType.equals(objType_)) {
                return;
            }

            for (int row = contents_.size() - 1; row >= 0; row--) {
                StorableObject sobj = (StorableObject) contents_.elementAt(row);
                if (sobj.getId() == id) {
                    contents_.removeElementAt(row);
                    fireTableRowsDeleted(row, row);
                    //if(contents_.size() == 0) {
                    //    MainWindow.getView().hideViews();
                    //    //MainWindow.Views.restoreViews();
                    //}
                    return;
                }
            }

            //stdlog_.debug("@@ Row to remove not found for id: " + id);
            fireTableDataChanged(); // play safe and update everything!

            // play safe and update everything!
        }

    }

    /**
     * Listener for system synchronization events.
     * Stops all editing when it receives such an event.
     */
    public static class SyncListener implements SystemStatusListener {

        /** Reference to paneInfo_ table in containing class. */
        private Hashtable paneInfo_ = new Hashtable();

        public SyncListener(Hashtable paneInfo) {
            paneInfo_ = paneInfo;
        }

        public void systemStatusChange(SystemStatusEvent ev) {
            if (ev.getType() == SystemStatusEvent.SYNC) {
                Enumeration enum = paneInfo_.keys();

                while (enum.hasMoreElements()) {
                    Object obj = enum.nextElement();

                    if (obj instanceof Class) {
                        Class objType = (Class) obj;
                        PaneInfo pi = (PaneInfo) paneInfo_.get(objType);
                        CellEditor ed = pi.table.getCellEditor();

                        if (ed != null) {
                            ed.stopCellEditing();
                        }

                    }
                }
            }
        }
    }

    private class Ancestor implements AncestorListener {

        /**  
         * Table mouse listener
         */

        TableMouseListener listener;

        /**
         * Container for the JTable
         */

        Container c;

        private SummaryTable table_;

        public Ancestor(SummaryTable table) {
            table_ = table;
        }

        /**
         * If the ancestor is added
         * @param ancestor addedevent
         */

        public void ancestorAdded(AncestorEvent event) {
            c = event.getAncestor();

            for (; c != null; c = c.getParent()) {
                if (c instanceof Window) {
                    //                    if(listener != null)
                    listener = new TableMouseListener(table_);
                    c.addMouseListener(listener);
                }
            }
        }

        /**
         * Ancestor moved
         * @param event Ancestor moved event
         */

        public void ancestorMoved(AncestorEvent event) {
        }

        /**
         * Ancestor removed
         * @param event Ancestor removed event
         */

        public void ancestorRemoved(AncestorEvent event) {

            if (c != null) {
                c.removeMouseListener(listener);
            }

            stopEditing();

        }

    }

    /**
     * Table Mouse Listener to make sure data is saved
     * to the business objects    
     *
     */

    private class TableMouseListener implements MouseListener {

        private SummaryTable table_;

        public TableMouseListener(SummaryTable table) {
            table_ = table;
        }

        /**
         * Mouse clicked event 
         * @param e mouse clicked event
         */

        public void mouseClicked(MouseEvent e) {
        }

        /** 
         * Mouse pressed event
         * @param e mouse pressed event
         */

        public void mousePressed(MouseEvent e) {
        }

        /**
         * Mouse released event
         * @param e mouse released event
         */
        public void mouseReleased(MouseEvent e) {
        }

        /**  
         * Mouse entered event
         * @param e mouse entered event
         */

        public void mouseEntered(MouseEvent e) {
            //stdlog_.debug("Mouse Entered");          
            stopEditing();
        }

        /**
         * Mouse exited event
         * @param e mouse exited event
         */

        public void mouseExited(MouseEvent e) {
            stopEditing();
        }
    }

    public class ColumnSortListener extends MouseAdapter {
        public void mouseClicked(MouseEvent e) {
            Class cls = getDisplayedObjectClass();
            PaneInfo pi = (PaneInfo) paneInfo_.get(cls);
            SummaryTable table = (SummaryTable) pi.table;
            FolderGridTableSorter sorter =
                (FolderGridTableSorter) table.getModel();
            sorter.sortModel(table, e);
            SummaryModel model = pi.model;
            model.reorderContents(sorter.getIndexes());
            stdlog_.debug("@@@@@ Reordered contents ......");
        };
    }
}
