package org.eso.ohs.phase2.apps.p2pp;

import java.awt.event.ActionEvent;
import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.Vector;

import javax.swing.JOptionPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.eso.ohs.core.dbb.client.DbaseDec;
import org.eso.ohs.core.dbb.client.DbaseRA;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbSelectionList;
import org.eso.ohs.core.dbb.client.DbbTextField;
import org.eso.ohs.core.dbb.client.DbbWidget;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.DbbInvalidSqlOperatorType;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.dbase.phase1.DbaseHandlerTG1;
import org.eso.ohs.phase2.apps.p2pp.dbb.P2PPView;
import org.eso.ohs.phase2.apps.p2ppAppServer.DbbLocalEngineP2PP;
import org.eso.ohs.phase2.apps.p2ppAppServer.Phase1BrowserConfig;

/**
 * Displays Phase 1 targets for the user to download or copy+paste.
 * 
 * <p>
 * XXX ToDo: there should be a central period property and all ComboBoxes,etc
 * should update from it via DocView.  Likewise all Phase1TargetsViewer
 * instances can be property change listeners to it.
 * 
 * <p>
 * $Id: Phase1TargetsViewer.java,v 1.15 2005/04/25 16:44:06 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * <br>ChangeLog: cmavros 21May02 [1408/1536/P2PP] Change "Download" menu option to 
 *    "Duplicate OB", add warning message to clarify to the user what Duplicate does.
 */
public class Phase1TargetsViewer extends MenuGroups  {

    private final static int PROGRAMMEID_COL = 1;

    private final static int INDEX_COL = 2;

    private final static int RUNREFS_COL = 3;


    private HierarchyView hView_;

    private DbbTextField sqlPeriodField_;

    private TargetDbbView view_;

    private DbbSelectionList selector_;


   


    public Phase1TargetsViewer(HierarchyView hView)
				    throws ObjectIOException {
	hView_ = hView;

	AppConfig	appcfg = AppConfig.getAppConfig();
	Config		cfg = Config.getCfg();

	DbbPanel        panel = null;
	DbbSqlEngine    sqle  = null;
	DbbSqlChunk     chunk;
	DbbWidget	widget;
	DbbSqlOperator  op;
	DbbSqlTable     targets_table = Phase1BrowserConfig.getTargetTable();
	DbbSqlTable	schedRep_table = Phase1BrowserConfig.getSchedRepTable();
	DbbSqlTable	obsRuns_table = Phase1BrowserConfig.getObsRunsTable();

	String labProgId  = "ObservingRun ID";

	setTitle("Phase 1 Targets");

	try {

	    // Build an SQL engine using the connection

	    if (appcfg.shouldUseAppServer()) 
		sqle = new DbbLocalEngineP2PP(appcfg.getAppServerUrl(), false);
	    else {
		sqle = Phase1BrowserConfig.getDbbSqlEngine(targets_table, schedRep_table, obsRuns_table);
	    }

	    // Build a view for interfacing to the SQL engine
	    view_ = new TargetDbbView(Media.PHASE1_DB, sqle);

	    chunk = new DbbSqlChunk(schedRep_table, "progid", "ObsRun", String.class);
	    view_.addDefaultColumnsChunk(chunk);

	    chunk = new DbbSqlChunk(targets_table, Phase1BrowserConfig.colProgId, labProgId, Integer.class);
	    view_.addDefaultColumnsChunk(chunk, false);

	    chunk = new DbbSqlChunk(targets_table, "id", "Index", Integer.class);
	    view_.addDefaultColumnsChunk(chunk, false);

	    // chunk = new DbbSqlChunk(targets_table, "run_references", "RunRefs");
	    // view_.addDefaultColumnsChunk(chunk);

	    chunk = new DbbSqlChunk(targets_table, "item_name", "Name");
	    view_.addDefaultColumnsChunk(chunk);

	    chunk = new DbbSqlChunk(targets_table, "r_a", "RA", DbaseRA.class);
	    view_.addDefaultColumnsChunk(chunk);

	    chunk = new DbbSqlChunk(targets_table, "dec", "Dec", DbaseDec.class);
	    view_.addDefaultColumnsChunk(chunk);

	    // chunk = new DbbSqlChunk(targets_table, "equinox", "Equinox");
	    // view_.addDefaultColumnsChunk(chunk);

	    // chunk = new DbbSqlChunk(targets_table, "magnitude", "Magnitude");
	    // view_.addDefaultColumnsChunk(chunk);

	    // chunk = new DbbSqlChunk(targets_table, "angular_diam", "AngDiam");
	    // view_.addDefaultColumnsChunk(chunk);

	    chunk = new DbbSqlChunk(targets_table, "comments", "Comments");
	    view_.addDefaultColumnsChunk(chunk);

	    // Build the rows selection panel.
	    // Add a widget to allow the user to 
	    // search by name with 'LIKE'
	    // ----------------------------------------
	    panel = new DbbPanel( "Selection Criteria", 3 );

	    op = new DbbSqlOperator("IN");
	    chunk = new DbbSqlChunk(targets_table, Phase1BrowserConfig.colProgId, labProgId,
					    op, Integer.class);
	    sqlPeriodField_ = new DbbTextField(labProgId , chunk);

	    panel.addWidget(sqlPeriodField_);

		    // Important: setEnabled(false) stops "Clear"
		    //		  from resetting the field's value!
	    sqlPeriodField_.setEnabled(false);
	    sqlPeriodField_.setVisible(false);
	    updatePeriod(hView_.getDisplayedPeriod());


	    op = new DbbSqlOperator("IN");
	    chunk = new DbbSqlChunk(schedRep_table, "progid", labProgId, op, String.class);

	    String[] orNames = getOrProgids();
	    selector_ = new DbbSelectionList("ObservingRun ID", chunk,
						orNames, orNames);
	    panel.addWidget(selector_);


	    // Add this panel to the view as the rows selection panel
	    //----------------------------------------
	    view_.setWherePanel(panel, "North");
	    getContentPane().add(view_);

	    addClose();		// Add menu item to close the window.

	    addTargetMenu("Synchronise", view_);

	    addMenuItem("Duplicate OB",
			new DownloadTargetsAction(view_,false));

	    addMenuItem("Duplicate OB",
			new DownloadTargetsAction(view_, true));

	    view_.addObjectSelectionListener(MainWindow.getView());
	    view_.addListSelectionListener(new MenuEnabler());

	}
	catch (SQLException e) {
	    throw new ObjectIOException(e);
	}
	catch (RemoteException e) {
	    throw new ObjectIOException(e);
	}
	catch (NotBoundException e) {
	    throw new ObjectIOException(e);
	}
	catch (MalformedURLException e) {
	    throw new ObjectIOException(e);
	}
	catch (DbbInvalidSqlOperatorType e) {
	    throw new ObjectIOException(e);
	}

    } 

 
    public void addNotify() {
	super.addNotify();
	setDefaultSelection();
    }


    public void setDefaultSelection() {
	try {
	    Folder folder = hView_.getSelectedFolder();

	    if (folder != null && selector_ != null) {
		selector_.set(folder.findObsRun().getProgId());
	    }
	}
	catch (ObjectIOException e) {
	    ErrorMessages.announceIOError(null,e);
	}
    }


    public void updatePeriod(int period)
					throws ObjectIOException {
	sqlPeriodField_.set(getSelectedProgrammeIds(period));

	if (selector_ != null) {
	    selector_.setListData(getOrProgids());
	    setDefaultSelection();
	}
    }


    private String getSelectedProgrammeIds(int displayedPeriod)
					throws ObjectIOException {
	Persistence objMgr = Persistence.getInstance();
	StringBuffer sb = new StringBuffer();
	DirectoryNode[] obsRunDirs;
	ObservingRun obsRun;
	Folder folder;

	obsRunDirs = objMgr.getRoot(Media.LOCAL).getDirectories();
	if (displayedPeriod != HierarchyView.ALL_PERIODS) {
	    obsRunDirs = selectObsRunDirs(obsRunDirs, displayedPeriod);
	}

	sb.append("(");

		// Note: explicit break inside of loop
	for (int j=0; j < obsRunDirs.length; j++) {
	    folder = FolderFactory.getFolder(Media.LOCAL, obsRunDirs[j]);
	    obsRun = folder.findObsRun();

	    sb.append(obsRun.getProgrammeId());

	    if (j >= obsRunDirs.length-1) {
		break;
	    }
	    sb.append(", ");
	}

	sb.append(")");

	return sb.toString();
    }


    /**
     * Filters the given array of DirectoryNode objects to include only
     * folders for ObservingRuns that are in the selected period.
     * 
     * @param subdirs	array of directories to be filtered.
     * 
     * @return		array containing a subset of directories
     *			from it's subdirs argument.
     */
    public static DirectoryNode[] selectObsRunDirs(DirectoryNode[] subdirs,
						   int period) {
	Vector visible = new Vector();
	DirectoryNode tmp;

	for (int j=0; j < subdirs.length; j++) {
	    Folder folder = (Folder)subdirs[j].getObject();
	    ObservingRun or = folder.findObsRun();

	    if ( or != null && or.getPeriod() == period) {
		visible.addElement(subdirs[j]);
	    }
	}

	subdirs = new DirectoryNode[visible.size()];
	visible.copyInto(subdirs);

	return subdirs;
    }


    private String[] getOrProgids()
			throws ObjectIOException {
	int period = hView_.getDisplayedPeriod();
	Persistence objMgr = Persistence.getInstance();
	DirectoryNode root = objMgr.getRoot(Media.LOCAL);
	DirectoryNode[] children = HierarchyView.sortDirNodes(root.getDirectories());
	Vector names = new Vector();

	for (int j=0; j < children.length; j++) {
	    Folder folder = FolderFactory.getFolder(Media.LOCAL, children[j]);
	    ObservingRun or = folder.findObsRun();

	    if (period == HierarchyView.ALL_PERIODS
				    || or.getPeriod() == period) {
		names.addElement(or.getProgId());
	    }
	}

	String[] result = new String[names.size()];
	names.copyInto(result);

	return result;
    }


    public static class TargetDbbView extends P2PPView {

	public TargetDbbView(Media device, DbbSqlEngine sqle) {
	    super(device,sqle);

	    hideSortButton();
	}

	public Long[] getAllSelected() {
	    int[] rows = getSelectedRows();

	    return rowNumbersToIds(rows);
	}

	public Long[] getAll() {
	    int[] rows = new int[getRows()];

	    for (int j=0; j < rows.length; j++) {
		rows[j] = j;
	    }

	    return rowNumbersToIds(rows);
	}

	private Long[] rowNumbersToIds(int[] rows) {
	    Long[] ids = new Long[rows.length];

	    for(int i=0;i<rows.length;i++) {         
		ids[i] = new Long(rowNumberToId(rows[i]));
	    }

	    return ids;
	}

	public long rowNumberToId(int rowNo) {
	    Integer programmeId = (Integer)getValueAt(rowNo, PROGRAMMEID_COL);
	    Integer index    = (Integer)getValueAt(rowNo, INDEX_COL);
	    Config cfg = Config.getCfg();

	    long dbaseId = DbaseHandlerTG1.idForPhase1Target(
				programmeId.intValue(),
				index.intValue()
			    );

	    return cfg.tableToUniqueId(dbaseId, Target.class);
	}

	public Class getDisplayedObjectClass() {
	    return Target.class;
	}

	public Long getSelected() {
	    Long[] selection = getAllSelected();

	    return (selection == null || selection.length == 0)
			? null
			: selection[0];
	}

    }

    private class MenuEnabler implements ListSelectionListener {

	public void valueChanged(ListSelectionEvent ev) {     
	    if (ev.getValueIsAdjusting()) {
		return;
	    }

	    ListSelectionModel lsm = (ListSelectionModel)ev.getSource();
	    int selectedRow = lsm.getMinSelectionIndex();

		// Is this a selection or deselection event?
	    if (lsm.isSelectedIndex(selectedRow)) {
		ObjectManager objMgr = ObjectManager.getObjectManager();
		Long idLong = view_.getSelected();
		Target tg;

		try {
		    tg = (Target)objMgr.getBusObj(
					Media.PHASE1_DB,
					idLong.longValue(),
					Target.class
				    );

		    setEnabledSynchroniseMenu(tg);
		}
		catch (ObjectIOException e) {
		  ErrorMessages.announceIOError(null,e);
		}
		catch (ObjectNotFoundException e) {
		  ErrorMessages.announceIOError(null,e);
		}
	    }
	    else {
		setEnabledSynchroniseMenu(null);
	    }
	}
    }


    public class DownloadTargetsAction extends ActionSuperclass {

	private TargetDbbView view_;

	private boolean downloadAll_;


	public DownloadTargetsAction(TargetDbbView view, boolean all) {
	    super(view, all   ? "with All targets" : "with Selected targets");

	    downloadAll_ = all;
	    view_        = view;
	}

	public void actionPerformedImpl(ActionEvent ev) {
	    MainWindow mainWdw = MainWindow.getPrincipalWindow();
	    FolderGridView folderView = mainWdw.getFolderView();

           /*
            * get the current selected tab of the FolderGridView and store it
            */
            String ctab = new String(((Class)folderView.getDisplayedObjectClass()).getName());

	    folderView.setDisplayedObjectClass(ObservationBlock.class);

           /*
            * get the tab of the FolderGridView, that should be selected and compare with
            * what was selected.
            */
            String stab = new String(((Class)folderView.getDisplayedObjectClass()).getName());
            if(!stab.equals(ctab)){
		JOptionPane.showMessageDialog(
		    mainWdw,
		    new Object[] {
                      "You can not paste a Phase 1 Target to a Calibration Block!",
		    },
		    "Nothing selected",
		    JOptionPane.ERROR_MESSAGE
		);
		return;		// nothing selected?
	    }

	    Long idLong = folderView.getSelectedObsBlocks();

	    if (idLong == null) {
		JOptionPane.showMessageDialog(
		    mainWdw,
		    new Object[] {
		      "There are no selected observation blocks.",
		      "Which observation blocks should be updated?",
		    },
		    "Nothing selected",
		    JOptionPane.ERROR_MESSAGE
		);
		return;		// nothing selected?
	    }

	    if (view_.getRows() <= 0) {
		JOptionPane.showMessageDialog(
		    mainWdw,
		    new Object[] {
		      "There are no displayed Phase 1 targets.",
		      "Select 'Query' to update the view.",
		    },
		    "Nothing selected",
		    JOptionPane.ERROR_MESSAGE
		);
		return;		// nothing selected?
	    }

	    if (downloadAll_) {
		view_.selectAllRows();
	    }
	    else {
		int[] selectedRows = view_.getSelectedRows();
		if (selectedRows == null || selectedRows.length == 0) {
		    JOptionPane.showMessageDialog(
			mainWdw,
			new Object[] {
			  "There are no selected Phase 1 targets.",
			  "Select the targets you wish to copy"
					+ " or use 'Download all targets'.",
			},
			"Nothing selected",
			JOptionPane.ERROR_MESSAGE
		    );
		    return;		// nothing selected?
		}
	    }

            Long [] idLongs = folderView.getAllSelected();
            if (idLongs.length > 1){
                String msg = "Please note only the first observation block will be\n"+
                             "duplicated with the selected target(s).\n\n" +
                             "If you want to copy a target into multiple observation blocks\n"+
                             "use the Synchronise copy / paste menu option.\n\n" +
                             "Are you sure you want to continue?\n";

                int option = JOptionPane.showConfirmDialog(
                    mainWdw,
                    new Object[] {
                    "There are "+idLongs.length + " observation blocks selected." ,
                    msg
                    },
                    "Continue with Check-In?",
                    JOptionPane.OK_CANCEL_OPTION
                );
                switch (option) {
                    case JOptionPane.CLOSED_OPTION : return;
                    case JOptionPane.CANCEL_OPTION : return;
                }
            }
            else {
		String msg = downloadAll_
		    ? "Are you sure you want to download all phase 1 targets"
		    : "Are you sure you want to download selected phase 1 targets";

		int option = JOptionPane.showConfirmDialog(
		    mainWdw,
		    new Object[] {
		      msg,
		      "onto the selected observation block?"
		    },
		    "Continue with Check-In?",
		    JOptionPane.OK_CANCEL_OPTION
		);

		switch (option) {
		    case JOptionPane.CLOSED_OPTION : return;
		    case JOptionPane.CANCEL_OPTION : return;
		}
            }

	    int[] tgRows = view_.getSelectedRows();

	    try{
		if (tgRows.length > 0) {
		    if ( ! OHSApplication.ensureAuthenticated() ) {
			return;
		    }

		    Persistence objMgr = Persistence.getInstance();

		    ObservationBlock ob = (ObservationBlock)objMgr.getBusObj(
						Media.LOCAL,
						idLong.longValue(),
						ObservationBlock.class
					    );
                    if (objMgr.isAllowedToModify(ob)) {
		        new IncrementalActionAutomaton(
			    new IncrementalCopyTarget(view_, ob, tgRows),
			    mainWdw,
			    "Downloading..."
		        );
                    } else {
		        JOptionPane.showMessageDialog(
			    mainWdw,
			    new Object[] {
			      "The observation block is locked",
			      "and may not be updated with target data."
			    },
			    "OB is locked",
			    JOptionPane.ERROR_MESSAGE
		        );
		        return;
                    }

		}
		else {		// There are no selected Targets now!
		    JOptionPane.showMessageDialog(
			mainWdw,
			new Object[] {
			  "There are no selected targets",
			  "after invalid objects have been excluded.",
			  " ",
			  "None of the targets could be downloaded to this programme."
			},
			"Nothing selected",
			JOptionPane.ERROR_MESSAGE
		    );
		    return;
		}
	    }
	    catch (ObjectNotFoundException e) {	      
		ErrorMessages.announceNoObject(null, e);
	    }
	    catch (ObjectIOException e) {
		ErrorMessages.announceIOError(null, e);
	    }


	}

    }

    public static class IncrementalCopyTarget implements IncrementalAction {

	private TargetDbbView view_;

	private Media device_;

	private DirectoryNode obDir_;

	private ObservationBlock ob_;

	private int[] selectedRows_;


	public IncrementalCopyTarget(TargetDbbView view,
					ObservationBlock ob, int[] tgRows) {
	    view_ = view;
	    selectedRows_ = tgRows;

	    ob_ = ob;

	    Persistence objMgr = Persistence.getInstance();

	    device_ = objMgr.getMediaFor(ob);
	    obDir_ = objMgr.getDirectoryFor(ob);	       

	}

	public int getNumStepsRemaining(int counter) {
	    return selectedRows_.length - counter;
	}

	public void startOfAction() { }

	public void endOfAction() { }

	/**
	 * Method to check-in the n'th element of cbs, where n = counter.
	 * 
	 * @param counter   counter to show which element of cbs to check in.
	 * @return	    true if the checking-in should continue
	 *		    and false otherwise.
	 */
	public void performStep(int counter) {
	    doCopyOnto(counter);
	}

	/**
	 * Utility method to duplicate a given business object
	 * 
	 * @param id	    id of business object to duplicate
	 */
	public void doCopyOnto(int counter) {
	    Persistence objMgr = Persistence.getInstance();

	    try {
		if (counter > 0) {
		    ob_ = (ObservationBlock)objMgr.deepCopy(
							device_,
							obDir_,
							ob_
						    );
		}

		Target destTG = ob_.getTarget();
		long srcTgId = view_.rowNumberToId(selectedRows_[counter]);
		String[] props = destTG.simpleProperties();

		Target srcTG = (Target)objMgr.getBusObj(
					    Media.PHASE1_DB,
					    srcTgId,
					    Target.class
					);

		srcTG.copyProperties(props, destTG);
	    }              
	    catch (NoSuchMethodException e) {	      
		ErrorMessages.announceNoObject(null, e);
	    }
	    catch (IllegalAccessException e) {	    
		ErrorMessages.announceNoObject(null, e);
	    }
	    catch (InvocationTargetException e) {	       
		ErrorMessages.announceNoObject(
		    null,
		    new ObjectIOException(e.getTargetException())
		);
	    }
	    catch (ObjectNotFoundException e) {	     
		ErrorMessages.announceNoObject(null, e);
	    }
	    catch (ObjectIOException e) {
		ErrorMessages.announceIOError(null, e);
	    }

	}

    }

}


 