package org.eso.ohs.phase2.apps.p2pp.views;


import javax.swing.Icon;
import javax.swing.JPanel;

import org.apache.log4j.Logger;
import org.eso.ohs.core.docview.datatrans.AdaptorFactory;
import org.eso.ohs.core.docview.datatrans.ObjectAdaptor;
import org.eso.ohs.core.gui.utilities.ImageUtils;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.phase2.apps.p2pp.Persistence;
/**
 * An interface satisfied by all document (object) views.
 * 
 * <p>
 * $Id: ObjectView.java,v 1.9 2005/02/01 14:51:49 tcanavan Exp $
 */

public abstract class ObjectView extends JPanel {
    
    
    
    
    private static Logger stdlog_ = Logger.getLogger(ObjectView.class);

    /** Icon to signify that a business o  bject is locked, ie readonly */
    public final static Icon locked_ = ImageUtils.getIcon("lock.gif");

    /** Icon to signify that a business object is invalid, 
    i.e. did not pass verification */
    public final static Icon invalidIcon_ = ImageUtils.getIcon("invalid.gif");

    /**
     * Returns the business object currently displayed by the view.
     * The result will be null if there is no currently displayed object.
     * 
     * @return currently displayed object.
     */
    abstract public BusinessObject getObject();

    /**
     * Hook to allow notification of a ObjectView subclass object
     * that it has been added to a ViewContainer.
     * 
     * @param vc    ViewContainer that now contains this ObjectView
     */
    public void viewAddedHook(ViewContainer vc) {
        // override in subclasses
    }

    /**
     * Is this object editable (from a client standpoint) i.e.:
     * checks its status - if it is checked in, it is not editable.
     */

    protected boolean isAllowedToModify(BusinessObject bo) {
        Persistence objMgr = Persistence.getInstance();
        if(objMgr == null) {
            stdlog_.debug("We have a problem objMgr not instanciated");
        }
        return objMgr.isAllowedToModify(bo);
    }

    /**
     * Hook to allow notification of a ObjectView subclass object
     * that it has been removed from a ViewContainer.
     * 
     * @param vc    ViewContainer that now contains this ObjectView
     */
    public void viewRemovedHook(ViewContainer vc) {
        // override in subclasses
    }

    /**
     * Sets the business object currently displayed by the view.
     * Setting it to null, clears all of the fields and disables
     * the GUI component.
     * 
     * <p>The object must be of the correct type for this viewer component.
     */
    abstract public void setObject(BusinessObject bo);

    /**
     * Utility method to get the property modifier for the given property
     * of a particular business object.
     * 
     * @param bo    	business object for which property modifier is needed.
     * @param propName	name of the property.
     */
    protected ObjectAdaptor getPropMod(BusinessObject bo, String propName) {
        return getPropMod(bo, propName, true);
    }

    /**
     * Utility method to get the property modifier for the given property
     * of a particular business object.
     * 
     * @param bo    	    object for which property modifier is needed.
     * @param propName	    name of the property.
     * @param readWrite	    true if the property may also be modified.
     */
    protected ObjectAdaptor getPropMod(
        BusinessObject bo,
        String propName,
        boolean readWrite) {
        ObjectAdaptor pm;

        if (bo == null) {
            return null;
        }

        pm = AdaptorFactory.getAdaptor(bo, propName);

        if (pm != null) {
            pm.setReadWrite(readWrite);
        }

        return pm;
    }

}
