package org.eso.ohs.phase2.apps.p2pp.views;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.eso.ohs.core.docview.gui.DocView;
import org.eso.ohs.core.gui.models.BoNameModel;
import org.eso.ohs.core.gui.models.DeclinationModel;
import org.eso.ohs.core.gui.models.EpochModel;
import org.eso.ohs.core.gui.models.EquinoxModel;
import org.eso.ohs.core.gui.models.FloatModel;
import org.eso.ohs.core.gui.models.RightAscensionModel;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.persistence.Config;


/**
 * This is a specialised JPanel with subcomponents linked to the
 * given Target object's fields.  When this component is
 * displayed, changes to it will be reflected in the Target
 * object.  Likewise changes to the Target object will be
 * reflected in the displayed fields.  Some fields of an Target
 * may not be visible.  Some fields may not be modifiable.
 * 
 * <p>
 * <br>
 * @author Bob Kemp
 */

public class TargetView extends ObjectView implements PropertyChangeListener {

    private final String rcsid = "$Id: TargetView.java,v 1.5 2004/02/17 12:37:59 tcanavan Exp $";
    /** the observation description currently displayed, or null */
    Target currentTarget_;

    /* GUI component for the Name field */
    private DocView fieldName_;

    /* GUI component for the ObjectClass field */
    private DocView fieldObjectClass_;

    /* GUI component for the Ra field */
    private DocView fieldRa_;

    /* GUI component for the Dec field */
    private DocView fieldDec_;

    /* GUI component for the PropRa field */
    private DocView fieldPropRa_;  

    /* GUI component for the PropDec field */
    private DocView fieldPropDec_;

    /* GUI component for the Equinox field */
    private DocView fieldEquinox_;

    /* GUI component for the Epoch field */
    private DocView fieldEpoch_;

    /* GUI component for the DiffRA field */
    private DocView fieldDiffRA_;

    /* GUI component for the DiffDec field */
    private DocView fieldDiffDec_;

    private DocView fieldTest_;

    private JLabel nameLabel_;


    /**
     * Construct the JPanel and add the subcomponents and listeners, etc.
     * Some fields of an Target may not be visible.
     * Some fields may not be modifiable.
     * 
     * <p> Initially, no Target is displayed.
     */
    public TargetView() {
        JLabel lab;     
        JComboBox gComboBox;
        JTextField tf;
    
        setLayout(new BorderLayout());
        GridLayout outerGrid  = new GridLayout(1,0);
        JPanel outerPanel = new JPanel(outerGrid);
        add(outerPanel,BorderLayout.NORTH);

        JPanel leftPanel = new JPanel();
        JPanel rightPanel = new JPanel();
        leftPanel.setLayout(new BorderLayout());
        rightPanel.setLayout(new BorderLayout());
        outerPanel.add(leftPanel);
        outerPanel.add(rightPanel);
       
        JPanel leftCol1 = new JPanel();
        JPanel leftCol2 = new JPanel();
        leftCol1.setLayout(new GridLayout(0,1));
        leftCol2.setLayout(new GridLayout(0,1));
        leftPanel.add(leftCol1,BorderLayout.WEST);
        leftPanel.add(leftCol2,BorderLayout.CENTER);
        JPanel rightCol1 = new JPanel();
        JPanel rightCol2 = new JPanel();
        rightCol1.setLayout(new GridLayout(0,1));
        rightCol2.setLayout(new GridLayout(0,1));
        rightPanel.add(rightCol1,BorderLayout.WEST);
        rightPanel.add(rightCol2,BorderLayout.CENTER);

        nameLabel_ = new JLabel("Name: ");
        nameLabel_.setHorizontalAlignment(JLabel.RIGHT);
        leftCol1.add(nameLabel_);
    
        tf = new JTextField(Config.getCfg().getMaxNameLength());
        tf.setDocument(new BoNameModel());
        fieldName_ = new DocView(tf);    
        leftCol2.add(fieldName_);
    
        lab = new JLabel("Right Ascension: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        leftCol1.add(lab);
    
    
        tf = new JTextField();
        tf.setDocument(new RightAscensionModel());
        fieldRa_ = new DocView(tf);
        leftCol2.add(fieldRa_);
    
        lab = new JLabel("Declination: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        leftCol1.add(lab);
    
        tf = new JTextField();
        tf.setDocument(new DeclinationModel());
        fieldDec_ = new DocView(tf);   
        leftCol2.add(fieldDec_);
    
        lab = new JLabel("Equinox: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        leftCol1.add(lab);
    
    
        tf = new JTextField();
        tf.setDocument(new EquinoxModel());
        fieldEquinox_ = new DocView(tf);    
        leftCol2.add(fieldEquinox_);
    
    
        lab = new JLabel("Epoch: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        leftCol1.add(lab);
    
        tf = new JTextField();
        tf.setDocument(new EpochModel());
        fieldEpoch_ = new DocView(tf);          
        leftCol2.add(fieldEpoch_);
    
        lab = new JLabel("Class: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        rightCol1.add(lab);
    
        gComboBox = new JComboBox(Target.getObjectClassValues());
        gComboBox.setToolTipText("This is a test");
        // XXX Fix and restore tooltips
        // gComboBox.setToolTips(Target.getObjectClassTooltips());
        fieldObjectClass_ = new DocView(gComboBox);         
        rightCol2.add(fieldObjectClass_);
    
    
        lab = new JLabel("proper motion RA: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        rightCol1.add(lab);
    
    
        tf = new JTextField();
        tf.setDocument(new FloatModel());
        fieldPropRa_ = new DocView(tf);
        rightCol2.add(fieldPropRa_);
    
    
        lab = new JLabel("proper motion DEC: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        rightCol1.add(lab);
    
    
        tf = new JTextField();
        tf.setDocument(new FloatModel());
        fieldPropDec_ = new DocView(tf);
        rightCol2.add(fieldPropDec_);
    
    
        lab = new JLabel("Diff RA: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        rightCol1.add(lab);
    
    
        tf = new JTextField();
        fieldDiffRA_ = new DocView(tf);
        rightCol2.add(fieldDiffRA_);
    
    
        lab = new JLabel("Diff DEC: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        rightCol1.add(lab);
    
        tf = new JTextField();
        fieldDiffDec_ = new DocView(tf);    
        rightCol2.add(fieldDiffDec_);
        setObject(null);
    }

    /**
     * Construct the JPanel and add the subcomponents and listeners, etc.
     * Some fields of a Target may not be visible.
     * Some fields may not be modifiable.
     * 
     * @param target    the Target object to link to.
     */
    public TargetView(Target target) {
        this();
        setObject(target);
    }

    public void setEnabled(boolean enable) {
       super.setEnabled(enable);
   
       //  nameLabel_.setIcon(enable ? null : locked_);
   
       fieldName_.setEnabled(enable);
       fieldRa_.setEnabled(enable);
       fieldDec_.setEnabled(enable);
       fieldPropRa_.setEnabled(enable);
       fieldPropDec_.setEnabled(enable);
       fieldEquinox_.setEnabled(enable);
       fieldEpoch_.setEnabled(enable);
       fieldDiffRA_.setEnabled(enable);
       fieldDiffDec_.setEnabled(enable);
       fieldObjectClass_.setEnabled(enable);
    }

   
   /*
    *
    */

    public void propertyChange(PropertyChangeEvent ev) {
        setEnabled(isAllowedToModify(currentTarget_));
    }


    /**
     * Returns a reference to the currently displayed Target.
     * 
     * @return reference to a Target.
     */
    public BusinessObject getObject() {
        return currentTarget_;
    }

    /**
     * Sets a new value of the current Target.
     * This method is included to satisfy the ObjectView
     * interface, which in turn allows for generalisation of
     * other parts of the system.
     * 
     * @param newValue reference to a BusinessObject.
     */
    public void setObject(BusinessObject newBO) {
        setObject((Target)newBO);
    }

    /**
    * Creates a component with subcomponents linked to the
    * Target object's fields.  When the returned component is
    * displayed, changes to it will be reflected in the Target
    * object.  Some fields of a Target may not be visible.
    * Some fields may not be modifiable.
    * 
    * @param newTarget the Target object to link to.
    *
    */
    public void setObject(Target newTarget) {

        if (newTarget == currentTarget_) {
            return;     // nothing to do -- just return
        }
    
        if (currentTarget_ != null) {
            currentTarget_.removePropertyChangeListener(this);
        }
    
        currentTarget_ = newTarget;
    
        setEnabled(isAllowedToModify(currentTarget_));
    
        fieldName_.setObjectProperty(
            currentTarget_, "Name"
        );
    
        fieldRa_.setObjectProperty(
            currentTarget_, "RA"
        );
    
        fieldDec_.setObjectProperty(
            currentTarget_, "Declination"
        );
    
        fieldPropRa_.setObjectProperty(
            currentTarget_, "PropRA"
        );
    
        fieldPropDec_.setObjectProperty(
            currentTarget_, "ProperDeclination"
        );
    
        fieldEquinox_.setObjectProperty(
            currentTarget_, "Equinox"
        );
    
        fieldEpoch_.setObjectProperty(
            currentTarget_, "Epoch"
        );
    
        fieldDiffRA_.setObjectProperty(
            currentTarget_, "DiffRA"
        );
    
        fieldDiffDec_.setObjectProperty(
            currentTarget_, "DiffDec"
        );
    
        fieldObjectClass_.setObjectProperty(
            currentTarget_, "ObjectClass"
        );
    
        if (currentTarget_ != null) {
            currentTarget_.addPropertyChangeListener(this);
        }
    
        fieldName_.requestFocus();
    }
}