package org.eso.ohs.phase2.apps.p2ppAppServer;

import java.io.File;
import java.io.IOException;
import java.rmi.AlreadyBoundException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.RMISocketFactory;
import java.rmi.server.ServerNotActiveException;
import java.sql.SQLException;
import java.util.StringTokenizer;
import java.util.Vector;

import org.eso.ohs.core.dbb.rmi.DbbRemoteEngine;
import org.eso.ohs.core.dbb.rmi.DbbRemoteEngineImpl;
import org.eso.ohs.core.utilities.ExtFileList;
import org.eso.ohs.core.utilities.Logger;
import org.eso.ohs.core.utilities.NotASCIIException;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.ObjConfig;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.rmi.OHSRMISocketFactory;
import org.eso.ohs.core.utilities.rmi.OHSServer;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.EVMStorageMgr;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.ProtectingStorageMgr;
import org.eso.ohs.persistence.StorageManager;
import org.eso.ohs.persistence.dbase.phase1.OpcDbaseMgr;
import org.eso.ohs.persistence.dbase.phase2.DBInstrumentListFactory;
import org.eso.ohs.persistence.dbase.phase2.DbaseMgr;
import org.eso.ohs.phase2.apps.p2ppAppServer.MyPersistCfg;
import org.eso.ohs.phase2.apps.p2ppAppServer.BrowserConfig;

/**
 * @version $Id: AppServer.java,v 1.22 2005/04/25 16:40:48 tcanavan Exp $
 *
 * The Application Server main class.
 *
 * @author Bob Kemp
 * <br>ChangeLog: amchavan, 11-Oct-2001: re-implemented as subclass of OHSServer.
 * <br>ChangeLog: cmavros 26JUNE02 Wrap DbaseMgr() with EVMStorageMgr class.
 *
 * @see StorageManager
 */

public class AppServer extends OHSServer implements AppServerFactory {

    /**
     * Creates a P2PP AppServer object.
     *
     * @param name      Name to be used when contacting this server.
     *
     * @param port      Port number on which the server is
     *			going to listen for connection requests.
     *                  A value of zero means the system default.
     *
     *
     * @param logger    Logger object for system information.
     */
    public AppServer(String name, int port, Logger logger)
        throws RemoteException, IOException, AlreadyBoundException, Exception {

        super(name, port, logger);
        startUp();
        try {
            // Setup internal data structures
            //----------------------------------------
            log("Initializing business objects");
            File objClassFile =
                new File(AppConfig.getAppConfig().getConfigDir(), "objClass.dat");
            Target.initialiseBusinessObjects(objClassFile);

            log("Loading instruments");

            /*
             * Load the DBInstrumentListFactory() if set in the configuration.
             */
            boolean mode = AppConfig.getAppConfig().isIPVersioningSingleIPMode();
            String instDir = AppConfig.getAppConfig().getInstrumDir();
            String ipCacheDir = AppConfig.getAppConfig().getIPVersionIPCache();
            if (!mode
                && AppConfig
                    .getAppConfig()
                    .getIPVersioningAutodownloadIPEnabled()) {
                /*
                 * Code here assumes that the configuration is correct - i.e. that
                 * the ipCacheDir exists in the configuration. This is tested in main().
                 */
                InstrumentList.setInstrumentListFactory(
                    new DBInstrumentListFactory(mode,instDir,ipCacheDir,
                            AppConfig.getAppConfig().getDbUserName(),
                            AppConfig.getAppConfig().getDbPassword()
                    ));
            }
            InstrumentList.setupInstrumentList(mode,instDir,ipCacheDir);

            // Send a warning message to old 
            // P2PP version installations
            //----------------------------------------
            log("Binding UpdateMsgAppServer objects in registry");
            String url = null;

            url = computeURL("P2PP", port);
            Naming.bind(url, new UpdateMsgAppServer());

            url = computeURL("P2PP-1", port);
            Naming.bind(url, new UpdateMsgAppServer());

            String[] l = Naming.list(getURL());
            for (int i = 0; i < l.length; i++)
                log("Bound: " + l[i]);

        } catch (Exception e) {
            log("Constructor", e);
            throw (e);
        }

        log("Initialization complete");
    }

    ////////////////////////////////////////////////////
    ///// 
    ///// StorageManager Factory Methods, etc
    ///// 
    ////////////////////////////////////////////////////

    /**
     * Backward-compatibility method for old clients.
     * Not needed after the protocol version is next increased.
     * Equivalent to <code>getStorageManager(Media.DBASE)</code>.
     */
    public RmiManagerInterface getStorageManager()
        throws RemoteException, ObjectIOException {
        return getStorageManager(Media.DBASE);
    }

    /**
     * Remote method for client to acquire a StorageManager object.
     * 
     * @param  device	Media for which a StorageManager is required.
     * @return	RMI object wrapping a storage manager.
     */
    public RmiManagerInterface getStorageManager(Media device)
        throws RemoteException, ObjectIOException {
        return wrapStorageManager(createStorageManager(device));
    }

    public RmiManagerInterface wrapStorageManager(StorageManager stMgr)
        throws RemoteException {
        try {
            System.out.println(
                "AppServer: new instance for: " + getClientHost());
        } catch (ServerNotActiveException e) {
            e.printStackTrace();
        }

        return new RmiStorageManager(
            new LoggingStorageManager(
                getLogger(),
                new ProtectingStorageMgr(stMgr)));
    }

    protected StorageManager createStorageManager(Media device)
        throws ObjectIOException {
            
        if (Media.PHASE1_DB.equals(device)) {
            return new OpcDbaseMgr();
        } else if (Media.PHASE2_DB.equals(device)) {
            boolean checkIPVersion = AppConfig.getAppConfig().getBoolean(OHSKeys.IPVersionsingEnsureCorrectIPVersionOnOBCheckin);
            DbaseMgr dbm = new DbaseMgr();
            dbm.setEnforceIPVersionCheck(checkIPVersion);
            // set the acceptable period set from the config files.
            dbm.setAcceptablePeriods(getAcceptableCheckInPeriods());
            return new EVMStorageMgr(dbm, new OpcDbaseMgr());
        }

        throw new IllegalArgumentException("Unsupported device: " + device);
    }
    
    /**
     * @return int[] the set of acceptable periods for check-in, else empty set for
     * all periods.
     */
    private int[] getAcceptableCheckInPeriods() {
        String list = AppConfig.getAppConfig().getValue(OHSKeys.AcceptableCheckinPeriodSet);
        Vector set = new Vector();
        if (list != null && list.length() > 0) {
            String delim = " ,;";
            StringTokenizer st = new StringTokenizer(list, delim);
            while (st.hasMoreTokens()) {
                String vs = st.nextToken();
                int v = -1;
                try {
                    v = Integer.parseInt(vs);
                } catch (NumberFormatException e) {
                    v = -1;
                }
                if (v > 0) {
                    set.addElement(new Integer(v));
                }
            }
        }
        int[] rc = new int[set.size()];
        for (int i = 0; i < rc.length; i++) {
            rc[i] = ((Integer)set.elementAt(i)).intValue();
        }
        return rc;
    }

    ////////////////////////////////////////////////////
    ///// 
    ///// DBB Factory Methods
    ///// 
    ////////////////////////////////////////////////////

    public DbbRemoteEngine getPhase1Engine()
        throws SQLException, RemoteException {

        // TODO: get rid of this reference to Phase1TargetsViewer
        return new DbbRemoteEngineImpl(
            Phase1BrowserConfig.getDbbSqlEngine(),
            getLogger(),
            getClientHostName());
    }

    public DbbRemoteEngine getPhase2Engine()
        throws SQLException, RemoteException {
        return new DbbRemoteEngineImpl(
            BrowserConfig.getEngine(),
            getLogger(),
            getClientHostName());
    }

    ////////////////////////////////////////////////////
    ///// 
    ///// Server initialisation
    ///// 
    ////////////////////////////////////////////////////

    public void start(String RmiUrl) {
    }

    public static void main(String[] args) {

        AppServer appserver = null;

        try {
            // Read and verify configuration options,
            // start logging messages.
            //----------------------------------------

            AppConfig.setAppConfig(new AppConfig());
            AppConfig.getAppConfig().setStandardOHSKeys("config");
            readCfgs();
            setupSubsystems();
            initObjInfo();

            AppConfig conf = AppConfig.getAppConfig();
            String logfile = conf.getAppServerLog();
            String name = conf.getAppServerName();
            OHSApplication.setShortName(name);
            String port = conf.getAppServerPort();
            String dataPort = conf.getAppServerDataPort();
            Logger logger = new Logger(logfile);

            logger.log("");
            logger.log("----------------------------------------");
            logger.log("AppServer.main() started");
            logger.log("Checking configuration options");

            if (!AppConfig.getAppConfig().isIPVersioningSingleIPMode()) {
                if (AppConfig
                    .getAppConfig()
                    .getIPVersioningAutodownloadIPEnabled()) {
                    // Check if the IP cache dir is configured - this is required
                    // if the AppServer is not running in Single IP mode.
                    String ipCacheDir =
                        AppConfig.getAppConfig().getIPVersionIPCache();
                    if (ipCacheDir == null || ipCacheDir.length() == 0) {
                        logger.log(
                            "FATAL: Configuration Error: Automatic IP download is enabled, but "
                                + "an IP Cache directory was not configured: exiting");
                        logger.log(
                            "Automatic IP Download Keyword: "
                                + OHSKeys.IPVersioningAutodownloadIP);
                        logger.log(
                            "IP Cache Directory Keyword: "
                                + OHSKeys.IPVersioningIPCache);
                        System.exit(1);
                    }
                }
            } else {
                if (AppConfig
                    .getAppConfig()
                    .getIPVersioningAutodownloadIPEnabled()) {
                    logger.log(
                        "FATAL: Configuration Error: Single IP Mode can not be enabled along with automatic IP download. Exiting.");
                    logger.log(
                        "Single IP Mode Keyword: "
                            + OHSKeys.IPVersioningSingleIPModeEnabled);
                    logger.log(
                        "Automatic IP Download Keyword: "
                            + OHSKeys.IPVersioningAutodownloadIP);
                    System.exit(1);
                }
            }
            if (name.length() == 0) {
                logger.log("FATAL: APPSERVER.NAME not found: exiting");
                System.exit(1);
            }

            if (port.length() == 0) {
                logger.log("FATAL: APPSERVER.PORT not found: exiting");
                System.exit(1);
            }

            if (dataPort.length() == 0) {
                logger.log("FATAL: APPSERVER.DATAPORT not found: exiting");
                System.exit(1);
            }
            int portn = (new Integer(port)).intValue();

            // PATCH: this should not be hardcoded.
            //        See 1634/P2PP
            // @author amchavan, 22-Jan-2002
            //----------------------------------------
            //int anonymPortn = 1201;
            int anonymPortn = (new Integer(dataPort)).intValue();

            // Create our own socket factory,
            // because we want to control how 
            // 'anonymous' ports are created.
            // NOTE: we must do this *before*
            //       the OHSServer is created
            // @author amchavan, 22-Jan-2002
            //----------------------------------------
            OHSRMISocketFactory sf = new OHSRMISocketFactory(anonymPortn);
            RMISocketFactory.setSocketFactory(sf);
            logger.log("Created socket factory: anonymPortn=" + anonymPortn);

            // then create an AppServer object: it 
            // will bind itself to the RMI registry
            // and start acceting connections
            //----------------------------------------
            logger.log("Creating AppServer object");
            appserver = new AppServer(name, portn, logger);
        } catch (Exception e) {

            // If there is any problem all we can do 
            // here is printout the exception and exit
            //----------------------------------------
            System.out.println();
            System.out.println("AppServer: " + e.getMessage());
            System.out.println();
            e.printStackTrace();
            System.exit(1);
        }
    }

    /**
     * Read required config files into AppConfig.
     */
    protected static void readCfgs() throws IOException {
        AppConfig ac = AppConfig.getAppConfig();
        ExtFileList extList = new ExtFileList(ac.getConfigDir(), ".cf");
        if (extList.exists("p2pp-db.cf")) {
            File dbFile = extList.getFileHandle("p2pp-db.cf");
            // ac.setupDbDirectConfig(OHSKeys.DBDirectSubConfig, dbFile);
            // need to put db config file in root dir (not in a sub config) or it can not
            // be overridden from the users config file.
            ac.setupDbDirectConfig("", dbFile);
        }
        if (extList.exists("server.cf")) {
            File srvFile = extList.getFileHandle("server.cf");
            try {
                ac.addParamfile(srvFile.getAbsolutePath());
            } catch (NotASCIIException e) {
                throw new IOException(e.getMessage());
            }
        }
        String currDir = System.getProperty("user.dir");
        File siteCf = extList.getFileHandle("site.cf");
        if (siteCf != null) {
            if (siteCf.exists() && siteCf.canRead()) {
                try {
                    ac.addParamfile(siteCf.getAbsolutePath());
                } catch (NotASCIIException e) {
                    throw new IOException(e.getMessage());
                }
            }
        }
        if (!ac.getStartAppServer()) {
            File userCf =
                new File(ac.getHomeDir() + File.separator + ".p2pp.cf");
            if (userCf.exists() && userCf.canRead()) {
                // the User CF file over writes settings in the site.cf
                // file - these settings are stored under a
                // UserOverrideKey subconfig
                // and used to over write settings when required.
                // Examine prependBase() for details on the effects of
                // the oser-ridden parameters.
                try {
                    ac.addParamfile(
                        userCf.getAbsolutePath(),
                        OHSKeys.UserOverrideKey);
                } catch (NotASCIIException e) {
                    throw new IOException(e.getMessage());
                }
            }
        }
        File ccsCf = new File(ac.getConfigDir() + File.separator + "ccs.cf");
        if (ccsCf.exists()) {
            try {
                ac.addParamfile(ccsCf.getAbsolutePath());
            } catch (NotASCIIException e) {
                throw new IOException(e.getMessage());
            }
        }

        // Here we supersede the contents of the CCS config file, if
        // any, with the value of the "ccs.wanted" property.
        // Under Unix, that can be selected using the "-noccs"
        // command line option.
        //
        // A M Chavan, 17-Jan-2000

        if (ac.getCCSEnabled()
            && "false".equals(System.getProperty("ccs.wanted"))) {
            ac.setValue(OHSKeys.CcsConnEnabledKey, false);
        }
    }

    /**
     * Set singleton instances of required implementation classes.
     */
    protected static void setupSubsystems() throws IOException {
        Config.setCfg(new MyPersistCfg());
    }

    /**
     * Initialize required business object definitions.
     */

    protected static void initObjInfo() {
        AppConfig appCfg = AppConfig.getAppConfig();
        appCfg.putObjCfg(new ObjConfig(ConstraintSet.class, new String[] {
            // Displayed column names
            "Id",
                "Name",
                "Airmass",
                "Sky Transparency",
                "Lunar Illumination",
                "Seeing",
                "Moon Distance" },
                new String[] { // Names of displayed properties
            "Id",
                "Name",
                "Airmass",
                "SkyTransparency",
                "FractionalLunarIllumination",
                "Seeing",
                "MoonAngularDistance" },
                new String[] { // Editable property names
            "Name",
                "Airmass",
                "FractionalLunarIllumination",
                "Seeing",
                "MoonAngularDistance" }));

        appCfg.putObjCfg(new ObjConfig(Target.class, new String[] {
            // Displayed column names
            "Id",
                "Name",
                "Ra",
                "Dec",
                "Equinox",
                "Epoch" },
                new String[] { // Names of displayed properties
            "Id",
                "Name",
                "RA",
                "Declination",
                "Equinox",
                "Epoch" },
                new String[] { // Editable property names
            "Name", "RA", "Declination", "Equinox", "Epoch" }));
        appCfg.putObjCfg(new ObjConfig(ObservationBlock.class, new String[] {
            // Displayed column names
            "Id",
                "Name",
                "DbaseId",
                "Status",
                "Target",
                "OD",
                "CS",
                "Acquisition" },
                new String[] { // Names of displayed properties
            "Id",
                "Name",
                "DbaseId",
                "Status",
                "Target",
                "Od",
                "ConstraintSet",
                "TemplateSignature" },
                new String[] {
        }));

        appCfg.putObjCfg(new ObjConfig(CalibrationBlock.class, new String[] {
            // Displayed column names
            "Id",
                "Name",
                "DbaseId",
                "Status",
                "OD" },
                new String[] { // Names of displayed properties
            "Id", "Name", "DbaseId", "Status", "Od" }, new String[] {
        }));

        appCfg
            .putObjCfg(new ObjConfig(
                ObservationDescription.class,
                new String[] {
            // Displayed column names
            "Id",
                "Name",
                "Instrument" },
                new String[] { // Names of displayed properties
            "Id",
                "Name",
                "Instrument" },
                new String[] { // Editable property names
            "Name" }));

        appCfg.putObjCfg(new ObjConfig(ObservingRun.class, new String[] {
            // Displayed column names
            // Period is required by MainWindow.Phase1Information
            "Id",
                "Period",
                "Title" },
                new String[] { // Names of displayed properties
            // Period is required by MainWindow.Phase1Information
            "Id",
                "Period",
                "Title" },
                new String[] { // Editable property names
        }));
    }
}
