package org.eso.ohs.phase2.icdVcs;

import java.lang.reflect.InvocationTargetException;

import javax.swing.SwingUtilities;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.widgets.OutOfMemoryReport;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.evm.EVMException;
import org.eso.ohs.evm.EvmExecutorImpl;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.ObjectManager;

/**
 * Listener for script events from the BOB interface
 * 
 * <p>
 * NOTE: This class must be public in order for Java/Ccs to work.
 *       I'm not sure of the reason but it's probably to do with
 *       their use of the Reflection API.
 * 
 * <p>
 * $Id: ScriptCmds.java,v 1.21 2005/04/14 09:41:30 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */
public class ScriptCmds implements ScriptCmdListener {

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(ScriptCmds.class);

    /**
     * Used to set the status of the BusinessObject that was sent to BOB.
     * If the BO was cloned before sending, the original BO's status is
     * also updated.
     * 
     * @param BOBEventType	Type of event as sent from BOB, eg "ABORTED"
     * @param idstr The id of the ob that the status event was for.
     */
    public void setBusObjStatusFromEventType(
        String BOBEventType,
        String idstr) {
        ProcessOBStatusEvents obEvents =
            new ProcessOBStatusEvents(BOBEventType, idstr);
        try {
            SwingUtilities.invokeAndWait(obEvents);
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
    }

    /**
        * Used to set the status of the BusinessObject that was sent to BOB.
        * If the BO was cloned before sending, the original BO's status is
        * also updated.
        * 
        * @param BOBEventType   Type of event as sent from BOB, eg "ABORTED"
        * @param idstr The id of the ob that the status event was for.
        */
    public void setTemplateStatusFromEventType(
        String BOBEventType,
        String idstr,
        String tsNo) {
        if (IcdVcsCfg.getCfg().isInVisitorMode()
            || (IcdVcsCfg.getCfg().isInEngineeringMode()
                && IcdVcsCfg.getCfg().isInCloneMode())) {
            int id = (int) new Long(idstr).longValue();
            int tsNum = Integer.valueOf(tsNo).intValue();
            ProcessTemplateStatusEvents templateEvents =
                new ProcessTemplateStatusEvents(
                    BOBEventType,
                    id,
                    tsNum - 1);
            try {
                SwingUtilities.invokeAndWait(templateEvents);
            } catch (InterruptedException e) {
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Handler method for NextObsBlocks requests from BOB.
     * Ignores "offline" requests.
     * 
     * @param numOBs	Number of OBs to be transferred
     * @param isOffline	Boolean.TRUE if is offline request,
     *			Boolean.FALSE if is not an offline request
     *			null if unspecified
     * 
     * @return		Observation block in OBD format 
     *		    	or null string if an error occurred.
     */
    public String NextObsBlocks(int numOBs, Boolean isOffline) {
        stdlog_.debug(
            ">>> Cmds.NextObsBlocks(" + numOBs + ", " + isOffline + ")");

        if (Boolean.TRUE.equals(isOffline)) {
            return ""; // Not for us.
        }
        try {
            OBChecker chk = new OBChecker(Media.LOCAL);
            SwingUtilities.invokeAndWait(chk);

            CalibrationBlock cb = chk.getOB();

            if (cb.isFlames()) {
                if (EvmExecutorImpl.evmConfigured()) {
                    EvmExecutorImpl evm = EvmExecutorImpl.getInstance();
                    //Check that the evm is initialized
                    if (evm.isInitialized()) {
                        try {
                            evm.calculateETCS(cb);
                        } catch (EVMException e) {
                            // If no exec time calcilation the OB can 
                            // still be passed to BOB...
                        }
                    }
                }
            }
            if (cb != null) {
                ObjectManager objMgr = ObjectManager.getObjectManager();
                String paf = cb.toOBD(true,objMgr.getDbaseId(cb),OHSApplication.getShortDesc());
                return paf;
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } catch (OutOfMemoryError e) {
            e.printStackTrace();
            OutOfMemoryReport.showMessage();
            throw e;
        }

        return "";
    }

}
