package org.eso.ohs.phase2.orang;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Hashtable;

import org.eso.oca.fits.OCAFile;
import org.eso.oca.parser.ASTStart;
import org.eso.oca.parser.InterpretationException;
import org.eso.oca.parser.OcaParser;
import org.eso.oca.parser.ParseException;
import org.eso.oca.parser.SimpleNode;

/**
 * 
 * This class implements the OBRankingEngine using the OCA rules engine
 * @author pnunes
 *
 */
public class OCARanker implements OBRankingEngine {

	
	private String scriptsDir_ ;	
	private String scriptFile_ =  "ranking.oca" ;
	
	private ObScheduleInfoAPI [] obs_;
	
	public OCARanker() {
		super();
	
	}

	public void setOBsForRanking(OBScheduleInfo[] obs) {
    	if (obs == null) {
            obs_ = new ObScheduleInfoAPI[0];
        } else {
            obs_ = new ObScheduleInfoAPI[obs.length];
            for (int i = 0; i < obs.length; i++) {
				obs_[i] =  new  OBScheduleInfoAdaptor(obs[i]);
			}
        }

	}

	public void setup(String scriptsDir) throws OrangScriptException {
		scriptsDir_ = scriptsDir;
	}

	public void rank() throws OrangScriptException {
		
		if (scriptsDir_ == null )
			throw new OrangScriptException("OCA ranker: script directory not initialized");
		
		String ocaRulesPath = this.scriptsDir_ + this.scriptFile_;
	
		
		/* Parse the rules. */
		OcaParser parser;
		try {
			parser = new OcaParser(new FileInputStream(ocaRulesPath));
		
			ASTStart start = parser.Start();
		
		
			
			//start.dumpSymtab();
			
			/*
			 * Find out which keyword values are needed from the HDR file.
			 */
			String[] requestedData = SimpleNode.getRequestedData();
			
			
			OCAFile[] rawFiles =  this.createOCAFiles();                         
			rawFiles = start.classify(rawFiles);
			
			applyClassification(rawFiles);
            
            
		} catch (FileNotFoundException e) {
			
			throw new OrangScriptException(e);
		} catch (ParseException e) {
			
			throw new OrangScriptException(e);
		} catch (InterpretationException e) {
			
			throw new OrangScriptException(e);
		}
    

	}

	private void applyClassification(OCAFile[] rawFiles) {
		
		for (int i = 0; i < rawFiles.length; i++) {
			OCAFile file = rawFiles[i];
			
			Hashtable table = file.getMetaCards();
			
			String rank = (String) table.get("SCORE");
			
			obs_[i].setRankScore(rank);
		}
		
	}

	private OCAFile[] createOCAFiles() throws OrangScriptException {
		String exMsg = "OCA ranker: OBs to rank not defined";
		
		if (obs_ == null) {
			throw new OrangScriptException(exMsg);			
		} else {
			if (obs_.length == 0)
				throw new OrangScriptException(exMsg);	
		}
		
		OCAFile [] res  = new OCAFile[obs_.length];
		
		for (int i = 0; i < obs_.length; i++) {
			res[i] = new OCAFile(obs_[i].getKWValues());
		}
		
		return(res);
	}

	public String getOBScore(int index) {
		return(obs_[index].getRankScore());
	}

	public ObScheduleInfoAPI[] getObs() {
		return obs_;
	}

	public String[] getAlgorithmNames() {
		// TODO Auto-generated method stub
		return null;
	}

	public void rank(String algorithmName) throws OrangScriptException {
		// TODO Auto-generated method stub
		
	}

	public OBScheduleInfo[] getRankedOBs() {
		// TODO Auto-generated method stub
		return null;
	}

}
