package org.eso.ohs.phase2.visibility;

import java.util.Date;

import org.apache.log4j.Logger;
import org.eso.ohs.dfs.TimeInterval;

/**
 * 
 * This class calculates the visibility of an <code>ObservationBlock</code>
 * 
 * @author pnunes
 *  Created on May 19, 2005
 */
public class VisibilityJNICalc implements VisibilityCalc {
	
	/**
	 * Logger
	 */
	private static Logger stdlog_ = Logger.getLogger(VisibilityJNICalc.class);
	 
    /**
     * Constraint values
     */
    
 
	public VisibilityJNICalc() {
		//PLONG = AppConfig.getAppConfig().getLongitude();
        //PLAT = AppConfig.getAppConfig().getLongitude();
        
	}
	
	
    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getLST(double, double)
     * @see SiderealTime#getST(double)
     */
    public double getLST(double jdate, double longitude) {
        double stValue = SiderealTime.getST(jdate);
        double lstValue = stValue - longitude;
        return (lstValue);
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getFLIValue(double, double, double)
     * @see FLI#getFLI(double, double, double, boolean)
     */
    public double getFLIValue(double jdate, double longit, double lat) {
        double fliValue = FLI.getFLI(jdate, longit, lat, true);
        return (fliValue);
    }

    
    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getAirMassValue(double, double, double, double, double)
     * @see AirMass#calculateAirMass(double, double, double, double, double)
     */
    public double getAirMassValue(
        double jdate,
        double lst,
        double targetRA,
        double targetDEC,
        double lat) {
        double airMassValue =
            AirMass.calculateAirMass(jdate, lst, targetRA, targetDEC, lat);
        return (airMassValue);
    }

    
    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getMoonDistValue(double, double, double, double, double)
     * @see MoonDistance#getMoonDistance(double, double, double, double, double)
     */
    public double getMoonDistValue(
        double jdate,
        double targetRA,
        double targetDEC,
        double longit,
        double lat) {
        double moonValue =
            MoonDistance.getMoonDistance(
                jdate,
                targetRA,
                targetDEC,
                longit,
                lat);
        return (moonValue);
    }
    

    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getMoonVisible(double, double, double)
     */
    public double getMoonVisible(double jdate, double longit, double lat) {
        double moonValue = MoonDistance.getMoonVisible(jdate, longit, lat);
        return (moonValue);
    }

    
    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getSunValue(double, double, double, double)
     * @see SunAngle#getSunAngle(double, double, double, double) 
     */
    public double getSunValue(
        double jdate,
        double longit,
        double lat,
        double lst) {
        double saValue = SunAngle.getSunAngle(jdate, longit, lat, lst);
        return (saValue);
    }
    
   
    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getSiderealTimeIntervalValue(org.eso.ohs.dfs.ObservationBlock, java.util.Date, double)
     */
    public double getSiderealTimeIntervalValue(
    	TimeInterval[] timeInts,
        Date utDate,
        double jdate) {

        // Convet the jdate to LST in radioans.

        //Convert user values into radian
        //Start and end dates for ST Times.
        // If the end date is < start date then 
        // add 2 PI to end date. 

        // Then compate the values         

        
        double timeValue = 1.;
        LST lst = new LST();

        double siderealTimeInRadians = lst.getLST(jdate);
        siderealTimeInRadians = lst.getLST(jdate) - VisibilityCalcUtils.PLONG;
        if (siderealTimeInRadians < 0) {
            siderealTimeInRadians = siderealTimeInRadians + (2 * Math.PI);
        }
        if (siderealTimeInRadians > (2 * Math.PI)) {
            siderealTimeInRadians = siderealTimeInRadians - (2 * Math.PI);
        }
        if (timeInts != null && timeInts.length != 0) {
            for (int i = 0; i < timeInts.length; i++) {
                double startTime =
                    timeInts[i].getStartTime() * (Math.PI / (12 * 3600));
                double endTime = timeInts[i].getEndTime() * (Math.PI / (12 * 3600));
                if (((siderealTimeInRadians >= startTime)
                    && (siderealTimeInRadians <= endTime))
                       || ((siderealTimeInRadians + (2 * Math.PI) >= startTime)
                        && (siderealTimeInRadians + (2 * Math.PI) <= endTime))) {
                    timeValue = 1.;
                    return (timeValue);
                } else {
                    timeValue = 0.;

                }
            }
        }
        return (timeValue);
    }
    


	/* (non-Javadoc)
	 * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getMoonAngle(double, double, double)
	 */
	public double getMoonAngle(double jdate, double longit, double lat) {
		double[] moonCoords = MoonDistance.getMoonCoord(jdate,longit,lat);
		//System.out.println(" RA: " + moonCoords[MoonDistance.MOON_RA]);
		//System.out.println(" DEC: " + moonCoords[MoonDistance.MOON_DEC]);
		double moonAngle = MoonDistance.getMoonAngle(jdate,moonCoords[MoonDistance.MOON_RA],moonCoords[MoonDistance.MOON_DEC],longit,lat);
		return(moonAngle);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getMoonAngle(double, double, double)
	 */
	public double getMoonRa(double jdate, double longit, double lat) {
		double[] moonCoords = MoonDistance.getMoonCoord(jdate,longit,lat);
		//System.out.println(" DEC: " + moonCoords[MoonDistance.MOON_DEC]);
		return moonCoords[MoonDistance.MOON_RA];
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getMoonAngle(double, double, double)
	 */
	public double getMoonDec(double jdate, double longit, double lat) {
		double[] moonCoords = MoonDistance.getMoonCoord(jdate,longit,lat);
		//System.out.println(" RA: " + moonCoords[MoonDistance.MOON_RA]);
		//System.out.println(" DEC: " + moonCoords[MoonDistance.MOON_DEC]);
	//	double moonAngle = MoonDistance.getMoonAngle(jdate,moonCoords[MoonDistance.MOON_RA],moonCoords[MoonDistance.MOON_DEC],longit,lat);
		return(moonCoords[MoonDistance.MOON_DEC]);
	}



}
