package org.eso.ohs.phase2.visiplot;

/**
 * The ColorChangeBar renders a Bar for a BarGraph using either the 
 * Gc from the original dataset, or from the colorChange dataset,
 * depending on the value of the data point in the color change dataset.
 *
 * This is used in Visiplot package to render the bars that fall in the
 * execution time in a different manner than the regular bars.
 *
 * @author Nick Kornweibel for ESO.
 */

import javachart.chart.AxisInterface;
import javachart.chart.Bar;
import javachart.chart.Dataset;
import javachart.chart.Datum;
import javachart.chart.Gc;
import javachart.chart.Plotarea;

public class ColorChangeBar extends Bar {

    private Dataset colorControl_ = null;

    public ColorChangeBar(Dataset[] d, Dataset colorControl, 
            AxisInterface xAx, AxisInterface yAx, Plotarea subplot)  {
        super(d, xAx, yAx, subplot);

        colorControl_ = colorControl;
    }

    /**
     * Draw a bar as per the Bar class, unless the colorControl dataset
     * contains a non-zero Y value, in which case, draw a bar, but use
     * the Gc from the colorControl dataset instead of the normal
     * dataset.
     */

    protected void doBar(java.awt.Graphics g, double howWide,
            int whichSet, int whichBar, boolean individualColors)  {
        Datum colorPoint = colorControl_.getDataElementAt(whichBar);
        if (colorPoint != null && colorPoint.getY() > 0.0) {
        	
            Gc origGc = datasets[whichSet].getGc();
            datasets[whichSet].setGc(colorControl_.getGc());
            super.doBar(g, howWide, whichSet, whichBar, individualColors);
            datasets[whichSet].setGc(origGc);
        } else {
            super.doBar(g, howWide, whichSet, whichBar, individualColors);
        }
    }
}
