package org.eso.ohs.scripting;

/**
 * Defines a Template within an OB. Like the ObservationBlockAPI, there is 
 * no means to set data, or create an "empty" template. 
 *
 * IMPORTANT: Changes to this interface must result in a new version string 
 * in the ScriptingBox class.
 *
 * @author Nick Kornweibel
 */

public interface TemplateAPI {

    /**
     * Get the name of the template
     * @return String containing the name of this template
     */

    public String getName(); 

    /**
     * Get the id of this Template
     * @return string containing the id of this template
     */

    public String getId();

    /**
     * Get the value of a given parameter, or empty string if not found
     * @param the parameter name to locate
     * @return string containing the value for the parameter
     */
    public String getParameterValue(String paramName); 

    /**
     * Get the number of parameters in this template
     * @return integer number of parameters
     */

    public int getNumberOfParameters(); 

    /**
     * Get parameter value for a specified parameter index, empty if
     * not a valid index
     * @param parameter index (>=0)
     * @return string containing the value of the parameter
     */

    public String getParameterValueAt(int index); 
    
    /**
     * Get parameter name for a specified parameter index or null if 
     * not a valid index
     * @param parameter index (>=0)
     * @return string containing the name of the parameter
     */

    public String getParameterNameAt(int index); 

    /**
     * Get the type of the Template - based on the TPL.TYPE keyword
     * @return the template type, or empty string if not known
     */

    public String getTemplateType();
    
    
    /**
     * Set the parameter value for a given keyword.
     * @param name
     * @param value
     */
    public void setParameterValue(String name,String value);

    /**
     * @return the Acquisition time, in seconds, of this template, or
     * 0 if it is not known or template is not an acquisition template.
     */

    public int getAcquisitionTime();
 
}
